@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2014 - 2015 Piotr Wójcik <chocimier@tlen.pl>
 * Copyright (C) 2015 Jan Bajer aka bajasoft <jbajer@gmail.com>
 *
@@ -20,7 +20,7 @@
 **************************************************************************/
 
 #include "ActionsManager.h"
-#include "Utils.h"
+#include "ThemesManager.h"
 #include "../ui/MainWindow.h"
 
 #include <QtCore/QCoreApplication>
@@ -32,9 +32,9 @@
 namespace Otter
 {
 
-ActionsManager* ActionsManager::m_instance = NULL;
+ActionsManager* ActionsManager::m_instance(nullptr);
 QVector<ActionsManager::ActionDefinition> ActionsManager::m_definitions;
-
+int ActionsManager::m_actionIdentifierEnumerator(0);
 
 Action::Action(int identifier, QObject *parent) : QAction(parent),
 	m_identifier(identifier),
@@ -79,8 +79,8 @@ void Action::update(bool reset)
 		return;
 	}
 
-	const ActionsManager::ActionDefinition action = ActionsManager::getActionDefinition(m_identifier);
-	QString text = QCoreApplication::translate("actions", (m_isOverridingText ? m_overrideText : action.text).toUtf8().constData());
+	const ActionsManager::ActionDefinition action(ActionsManager::getActionDefinition(m_identifier));
+	QString text(QCoreApplication::translate("actions", (m_isOverridingText ? m_overrideText : action.text).toUtf8().constData()));
 
 	if (!action.shortcuts.isEmpty())
 	{
@@ -94,7 +94,30 @@ void Action::update(bool reset)
 		setEnabled(action.flags.testFlag(ActionsManager::IsEnabledFlag));
 		setCheckable(action.flags.testFlag(ActionsManager::IsCheckableFlag));
 		setChecked(action.flags.testFlag(ActionsManager::IsCheckedFlag));
-		setIcon(action.icon);
+
+		switch (m_identifier)
+		{
+			case ActionsManager::GoBackAction:
+				setIcon(ThemesManager::getIcon(QGuiApplication::isLeftToRight() ? QLatin1String("go-previous") : QLatin1String("go-next")));
+
+				break;
+			case ActionsManager::GoForwardAction:
+				setIcon(ThemesManager::getIcon(QGuiApplication::isLeftToRight() ? QLatin1String("go-next") : QLatin1String("go-previous")));
+
+				break;
+			case ActionsManager::RewindAction:
+				setIcon(ThemesManager::getIcon(QGuiApplication::isLeftToRight() ? QLatin1String("go-first") : QLatin1String("go-last")));
+
+				break;
+			case ActionsManager::FastForwardAction:
+				setIcon(ThemesManager::getIcon(QGuiApplication::isLeftToRight() ? QLatin1String("go-last") : QLatin1String("go-first")));
+
+				break;
+			default:
+				setIcon(action.icon);
+
+				break;
+		}
 	}
 }
 
@@ -123,14 +146,51 @@ int Action::getIdentifier() const
 
 bool Action::event(QEvent *event)
 {
-	if (event->type() == QEvent::LanguageChange)
+	switch (event->type())
 	{
-		update();
+		case QEvent::LanguageChange:
+			update();
+
+			break;
+		case QEvent::LayoutDirectionChange:
+			switch (m_identifier)
+			{
+				case ActionsManager::GoBackAction:
+				case ActionsManager::GoForwardAction:
+				case ActionsManager::RewindAction:
+				case ActionsManager::FastForwardAction:
+					update(true);
+
+					break;
+				default:
+					break;
+			}
+
+			break;
+		default:
+			break;
 	}
 
 	return QAction::event(event);
 }
 
+bool Action::calculateCheckedState(const QVariantMap &parameters, Action *action)
+{
+	if (parameters.contains(QLatin1String("isChecked")))
+	{
+		return parameters.value(QLatin1String("isChecked")).toBool();
+	}
+
+	if (action)
+	{
+		action->toggle();
+
+		return action->isChecked();
+	}
+
+	return true;
+}
+
 bool Action::isLocal(int identifier)
 {
 	switch (identifier)
@@ -139,6 +199,7 @@ bool Action::isLocal(int identifier)
 		case ActionsManager::CloneTabAction:
 		case ActionsManager::ClearTabHistoryAction:
 		case ActionsManager::PurgeTabHistoryAction:
+		case ActionsManager::MuteTabMediaAction:
 		case ActionsManager::OpenLinkAction:
 		case ActionsManager::OpenLinkInCurrentTabAction:
 		case ActionsManager::OpenLinkInNewTabAction:
@@ -171,6 +232,7 @@ bool Action::isLocal(int identifier)
 		case ActionsManager::MediaLoopAction:
 		case ActionsManager::MediaPlayPauseAction:
 		case ActionsManager::MediaMuteAction:
+		case ActionsManager::FillPasswordAction:
 		case ActionsManager::GoBackAction:
 		case ActionsManager::GoForwardAction:
 		case ActionsManager::RewindAction:
@@ -193,6 +255,7 @@ bool Action::isLocal(int identifier)
 		case ActionsManager::PasteNoteAction:
 		case ActionsManager::DeleteAction:
 		case ActionsManager::SelectAllAction:
+		case ActionsManager::UnselectAction:
 		case ActionsManager::ClearAllAction:
 		case ActionsManager::CheckSpellingAction:
 		case ActionsManager::SelectDictionaryAction:
@@ -272,12 +335,12 @@ ActionsManager::ActionsManager(QObject *parent) : QObject(parent),
 
 	m_definitions.reserve(ActionsManager::OtherAction);
 
-	registerAction(NewTabAction, QT_TRANSLATE_NOOP("actions", "New Tab"), QString(), Utils::getIcon(QLatin1String("tab-new")));
-	registerAction(NewTabPrivateAction, QT_TRANSLATE_NOOP("actions", "New Private Tab"), QString(), Utils::getIcon(QLatin1String("tab-new-private")));
-	registerAction(NewWindowAction, QT_TRANSLATE_NOOP("actions", "New Window"), QString(), Utils::getIcon(QLatin1String("window-new")));
-	registerAction(NewWindowPrivateAction, QT_TRANSLATE_NOOP("actions", "New Private Window"), QString(), Utils::getIcon(QLatin1String("window-new-private")));
-	registerAction(OpenAction, QT_TRANSLATE_NOOP("actions", "Open…"), QString(), Utils::getIcon(QLatin1String("document-open")));
-	registerAction(SaveAction, QT_TRANSLATE_NOOP("actions", "Save…"), QString(), Utils::getIcon(QLatin1String("document-save")));
+	registerAction(NewTabAction, QT_TRANSLATE_NOOP("actions", "New Tab"), QString(), ThemesManager::getIcon(QLatin1String("tab-new")));
+	registerAction(NewTabPrivateAction, QT_TRANSLATE_NOOP("actions", "New Private Tab"), QString(), ThemesManager::getIcon(QLatin1String("tab-new-private")));
+	registerAction(NewWindowAction, QT_TRANSLATE_NOOP("actions", "New Window"), QString(), ThemesManager::getIcon(QLatin1String("window-new")));
+	registerAction(NewWindowPrivateAction, QT_TRANSLATE_NOOP("actions", "New Private Window"), QString(), ThemesManager::getIcon(QLatin1String("window-new-private")));
+	registerAction(OpenAction, QT_TRANSLATE_NOOP("actions", "Open…"), QString(), ThemesManager::getIcon(QLatin1String("document-open")));
+	registerAction(SaveAction, QT_TRANSLATE_NOOP("actions", "Save…"), QString(), ThemesManager::getIcon(QLatin1String("document-save")));
 	registerAction(CloneTabAction, QT_TRANSLATE_NOOP("actions", "Clone Tab"));
 	registerAction(PinTabAction, QT_TRANSLATE_NOOP("actions", "Pin Tab"));
 	registerAction(DetachTabAction, QT_TRANSLATE_NOOP("actions", "Detach Tab"));
@@ -287,8 +350,10 @@ ActionsManager::ActionsManager(QObject *parent) : QObject(parent),
 	registerAction(AlwaysOnTopTabAction, QT_TRANSLATE_NOOP("actions", "Stay on Top"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag));
 	registerAction(ClearTabHistoryAction, QT_TRANSLATE_NOOP("actions", "Clear Tab History"), QT_TRANSLATE_NOOP("actions", "Remove Local Tab History"));
 	registerAction(PurgeTabHistoryAction, QT_TRANSLATE_NOOP("actions", "Purge Tab History"), QT_TRANSLATE_NOOP("actions", "Remove Local and Global Tab History"));
-	registerAction(CloseTabAction, QT_TRANSLATE_NOOP("actions", "Close Tab"), QString(), Utils::getIcon(QLatin1String("tab-close")), NoFlags);
-	registerAction(CloseOtherTabsAction, QT_TRANSLATE_NOOP("actions", "Close Other Tabs"), QString(), Utils::getIcon(QLatin1String("tab-close-other")));
+	registerAction(MuteTabMediaAction, QT_TRANSLATE_NOOP("actions", "Mute Tab Media"), QString(), ThemesManager::getIcon(QLatin1String("audio-volume-medium")), NoFlags);
+	registerAction(SuspendTabAction, QT_TRANSLATE_NOOP("actions", "Suspend Tab"), QString(), QIcon(), NoFlags);
+	registerAction(CloseTabAction, QT_TRANSLATE_NOOP("actions", "Close Tab"), QString(), ThemesManager::getIcon(QLatin1String("tab-close")), NoFlags);
+	registerAction(CloseOtherTabsAction, QT_TRANSLATE_NOOP("actions", "Close Other Tabs"), QString(), ThemesManager::getIcon(QLatin1String("tab-close-other")));
 	registerAction(ClosePrivateTabsAction, QT_TRANSLATE_NOOP("actions", "Close All Private Tabs"), QT_TRANSLATE_NOOP("actions", "Close All Private Tabs in Current Window"), QIcon(), NoFlags);
 	registerAction(ClosePrivateTabsPanicAction, QT_TRANSLATE_NOOP("actions", "Close Private Tabs and Windows"));
 	registerAction(ReopenTabAction, QT_TRANSLATE_NOOP("actions", "Reopen Previously Closed Tab"));
@@ -301,7 +366,7 @@ ActionsManager::ActionsManager(QObject *parent) : QObject(parent),
 	registerAction(SessionsAction, QT_TRANSLATE_NOOP("actions", "Manage Sessions…"));
 	registerAction(SaveSessionAction, QT_TRANSLATE_NOOP("actions", "Save Current Session…"));
 	registerAction(OpenUrlAction, QT_TRANSLATE_NOOP("actions", "Open URL"));
-	registerAction(OpenLinkAction, QT_TRANSLATE_NOOP("actions", "Open"), QString(), Utils::getIcon(QLatin1String("document-open")));
+	registerAction(OpenLinkAction, QT_TRANSLATE_NOOP("actions", "Open"), QString(), ThemesManager::getIcon(QLatin1String("document-open")));
 	registerAction(OpenLinkInCurrentTabAction, QT_TRANSLATE_NOOP("actions", "Open in This Tab"));
 	registerAction(OpenLinkInNewTabAction, QT_TRANSLATE_NOOP("actions", "Open in New Tab"));
 	registerAction(OpenLinkInNewTabBackgroundAction, QT_TRANSLATE_NOOP("actions", "Open in New Background Tab"));
@@ -313,7 +378,7 @@ ActionsManager::ActionsManager(QObject *parent) : QObject(parent),
 	registerAction(OpenLinkInNewPrivateWindowBackgroundAction, QT_TRANSLATE_NOOP("actions", "Open in New Private Background Window"));
 	registerAction(OpenLinkInApplicationAction, QT_TRANSLATE_NOOP("actions", "Open with…"), QT_TRANSLATE_NOOP("actions", "Open Link with External Application"), QIcon(), (IsEnabledFlag | IsMenuFlag));
 	registerAction(CopyLinkToClipboardAction, QT_TRANSLATE_NOOP("actions", "Copy Link to Clipboard"));
-	registerAction(BookmarkLinkAction, QT_TRANSLATE_NOOP("actions", "Bookmark Link…"), QString(), Utils::getIcon(QLatin1String("bookmark-new")));
+	registerAction(BookmarkLinkAction, QT_TRANSLATE_NOOP("actions", "Bookmark Link…"), QString(), ThemesManager::getIcon(QLatin1String("bookmark-new")));
 	registerAction(SaveLinkToDiskAction, QT_TRANSLATE_NOOP("actions", "Save Link Target As…"));
 	registerAction(SaveLinkToDownloadsAction, QT_TRANSLATE_NOOP("actions", "Save to Downloads"));
 	registerAction(OpenSelectionAsLinkAction, QT_TRANSLATE_NOOP("actions", "Go to This Address"));
@@ -337,47 +402,50 @@ ActionsManager::ActionsManager(QObject *parent) : QObject(parent),
 	registerAction(MediaLoopAction, QT_TRANSLATE_NOOP("actions", "Looping"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag));
 	registerAction(MediaPlayPauseAction, QT_TRANSLATE_NOOP("actions", "Play"));
 	registerAction(MediaMuteAction, QT_TRANSLATE_NOOP("actions", "Mute"));
-	registerAction(GoAction, QT_TRANSLATE_NOOP("actions", "Go"), QString(), Utils::getIcon(QLatin1String("go-jump-locationbar")));
-	registerAction(GoBackAction, QT_TRANSLATE_NOOP("actions", "Back"), QString(), Utils::getIcon(QLatin1String("go-previous")), (IsEnabledFlag | IsMenuFlag));
-	registerAction(GoForwardAction, QT_TRANSLATE_NOOP("actions", "Forward"), QString(), Utils::getIcon(QLatin1String("go-next")), (IsEnabledFlag | IsMenuFlag));
+	registerAction(FillPasswordAction, QT_TRANSLATE_NOOP("actions", "Log In"), QString(), ThemesManager::getIcon(QLatin1String("fill-password")), NoFlags);
+	registerAction(GoAction, QT_TRANSLATE_NOOP("actions", "Go"), QString(), ThemesManager::getIcon(QLatin1String("go-jump-locationbar")));
+	registerAction(GoBackAction, QT_TRANSLATE_NOOP("actions", "Back"), QString(), ThemesManager::getIcon(QLatin1String("go-previous")), (IsEnabledFlag | IsMenuFlag));
+	registerAction(GoForwardAction, QT_TRANSLATE_NOOP("actions", "Forward"), QString(), ThemesManager::getIcon(QLatin1String("go-next")), (IsEnabledFlag | IsMenuFlag));
 	registerAction(GoToPageAction, QT_TRANSLATE_NOOP("actions", "Go to Page or Search"));
-	registerAction(GoToHomePageAction, QT_TRANSLATE_NOOP("actions", "Go to Home Page"), QString(), Utils::getIcon(QLatin1String("go-home")));
+	registerAction(GoToHomePageAction, QT_TRANSLATE_NOOP("actions", "Go to Home Page"), QString(), ThemesManager::getIcon(QLatin1String("go-home")));
 	registerAction(GoToParentDirectoryAction, QT_TRANSLATE_NOOP("actions", "Go to Parent Directory"));
-	registerAction(RewindAction, QT_TRANSLATE_NOOP("actions", "Rewind"), QString(), Utils::getIcon(QLatin1String("go-first")));
-	registerAction(FastForwardAction, QT_TRANSLATE_NOOP("actions", "Fast Forward"), QString(), Utils::getIcon(QLatin1String("go-last")));
-	registerAction(StopAction, QT_TRANSLATE_NOOP("actions", "Stop"), QString(), Utils::getIcon(QLatin1String("process-stop")));
+	registerAction(RewindAction, QT_TRANSLATE_NOOP("actions", "Rewind"), QString(), ThemesManager::getIcon(QLatin1String("go-first")));
+	registerAction(FastForwardAction, QT_TRANSLATE_NOOP("actions", "Fast Forward"), QString(), ThemesManager::getIcon(QLatin1String("go-last")));
+	registerAction(StopAction, QT_TRANSLATE_NOOP("actions", "Stop"), QString(), ThemesManager::getIcon(QLatin1String("process-stop")));
 	registerAction(StopScheduledReloadAction, QT_TRANSLATE_NOOP("actions", "Stop Scheduled Page Reload"));
-	registerAction(ReloadAction, QT_TRANSLATE_NOOP("actions", "Reload"), QString(), Utils::getIcon(QLatin1String("view-refresh")));
-	registerAction(ReloadOrStopAction, QT_TRANSLATE_NOOP("actions", "Reload"), QT_TRANSLATE_NOOP("actions", "Reload or Stop"), Utils::getIcon(QLatin1String("view-refresh")));
+	registerAction(StopAllAction, QT_TRANSLATE_NOOP("actions", "Stop All Pages"), QString(), ThemesManager::getIcon(QLatin1String("process-stop")));
+	registerAction(ReloadAction, QT_TRANSLATE_NOOP("actions", "Reload"), QString(), ThemesManager::getIcon(QLatin1String("view-refresh")));
+	registerAction(ReloadOrStopAction, QT_TRANSLATE_NOOP("actions", "Reload"), QT_TRANSLATE_NOOP("actions", "Reload or Stop"), ThemesManager::getIcon(QLatin1String("view-refresh")));
 	registerAction(ReloadAndBypassCacheAction, QT_TRANSLATE_NOOP("actions", "Reload and Bypass Cache"));
-	registerAction(ReloadAllAction, QT_TRANSLATE_NOOP("actions", "Reload All Tabs"), QString(), Utils::getIcon(QLatin1String("view-refresh")));
+	registerAction(ReloadAllAction, QT_TRANSLATE_NOOP("actions", "Reload All Tabs"), QString(), ThemesManager::getIcon(QLatin1String("view-refresh")));
 	registerAction(ScheduleReloadAction, QT_TRANSLATE_NOOP("actions", "Reload Every"), QString(), QIcon(), (IsEnabledFlag | IsMenuFlag));
 	registerAction(ContextMenuAction, QT_TRANSLATE_NOOP("actions", "Show Context Menu"));
-	registerAction(UndoAction, QT_TRANSLATE_NOOP("actions", "Undo"), QString(), Utils::getIcon(QLatin1String("edit-undo")));
-	registerAction(RedoAction, QT_TRANSLATE_NOOP("actions", "Redo"), QString(), Utils::getIcon(QLatin1String("edit-redo")));
-	registerAction(CutAction, QT_TRANSLATE_NOOP("actions", "Cut"), QString(), Utils::getIcon(QLatin1String("edit-cut")));
-	registerAction(CopyAction, QT_TRANSLATE_NOOP("actions", "Copy"), QString(), Utils::getIcon(QLatin1String("edit-copy")));
+	registerAction(UndoAction, QT_TRANSLATE_NOOP("actions", "Undo"), QString(), ThemesManager::getIcon(QLatin1String("edit-undo")));
+	registerAction(RedoAction, QT_TRANSLATE_NOOP("actions", "Redo"), QString(), ThemesManager::getIcon(QLatin1String("edit-redo")));
+	registerAction(CutAction, QT_TRANSLATE_NOOP("actions", "Cut"), QString(), ThemesManager::getIcon(QLatin1String("edit-cut")));
+	registerAction(CopyAction, QT_TRANSLATE_NOOP("actions", "Copy"), QString(), ThemesManager::getIcon(QLatin1String("edit-copy")));
 	registerAction(CopyPlainTextAction, QT_TRANSLATE_NOOP("actions", "Copy as Plain Text"));
 	registerAction(CopyAddressAction, QT_TRANSLATE_NOOP("actions", "Copy Address"));
 	registerAction(CopyToNoteAction, QT_TRANSLATE_NOOP("actions", "Copy to Note"));
-	registerAction(PasteAction, QT_TRANSLATE_NOOP("actions", "Paste"), QString(), Utils::getIcon(QLatin1String("edit-paste")));
+	registerAction(PasteAction, QT_TRANSLATE_NOOP("actions", "Paste"), QString(), ThemesManager::getIcon(QLatin1String("edit-paste")));
 	registerAction(PasteAndGoAction, QT_TRANSLATE_NOOP("actions", "Paste and Go"));
 	registerAction(PasteNoteAction, QT_TRANSLATE_NOOP("actions", "Insert Note"), QString(), QIcon(), (IsEnabledFlag | IsMenuFlag));
-	registerAction(DeleteAction, QT_TRANSLATE_NOOP("actions", "Delete"), QString(), Utils::getIcon(QLatin1String("edit-delete")));
-	registerAction(SelectAllAction, QT_TRANSLATE_NOOP("actions", "Select All"), QString(), Utils::getIcon(QLatin1String("edit-select-all")));
+	registerAction(DeleteAction, QT_TRANSLATE_NOOP("actions", "Delete"), QString(), ThemesManager::getIcon(QLatin1String("edit-delete")));
+	registerAction(SelectAllAction, QT_TRANSLATE_NOOP("actions", "Select All"), QString(), ThemesManager::getIcon(QLatin1String("edit-select-all")));
+	registerAction(UnselectAction, QT_TRANSLATE_NOOP("actions", "Deselect"));
 	registerAction(ClearAllAction, QT_TRANSLATE_NOOP("actions", "Clear All"));
 	registerAction(CheckSpellingAction, QT_TRANSLATE_NOOP("actions", "Check Spelling"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag));
 	registerAction(SelectDictionaryAction, QT_TRANSLATE_NOOP("actions", "Dictionaries"), QString(), QIcon(), (IsEnabledFlag | IsMenuFlag));
-	registerAction(FindAction, QT_TRANSLATE_NOOP("actions", "Find…"), QString(), Utils::getIcon(QLatin1String("edit-find")));
+	registerAction(FindAction, QT_TRANSLATE_NOOP("actions", "Find…"), QString(), ThemesManager::getIcon(QLatin1String("edit-find")));
 	registerAction(FindNextAction, QT_TRANSLATE_NOOP("actions", "Find Next"));
 	registerAction(FindPreviousAction, QT_TRANSLATE_NOOP("actions", "Find Previous"));
 	registerAction(QuickFindAction, QT_TRANSLATE_NOOP("actions", "Quick Find"));
 	registerAction(SearchAction, QT_TRANSLATE_NOOP("actions", "Search"));
 	registerAction(SearchMenuAction, QT_TRANSLATE_NOOP("actions", "Search Using"));
 	registerAction(CreateSearchAction, QT_TRANSLATE_NOOP("actions", "Create Search…"));
-	registerAction(ZoomInAction, QT_TRANSLATE_NOOP("actions", "Zoom In"), QString(), Utils::getIcon(QLatin1String("zoom-in")));
-	registerAction(ZoomOutAction, QT_TRANSLATE_NOOP("actions", "Zoom Out"), QString(), Utils::getIcon(QLatin1String("zoom-out")));
-	registerAction(ZoomOriginalAction, QT_TRANSLATE_NOOP("actions", "Zoom Original"), QString(), Utils::getIcon(QLatin1String("zoom-original")));
+	registerAction(ZoomInAction, QT_TRANSLATE_NOOP("actions", "Zoom In"), QString(), ThemesManager::getIcon(QLatin1String("zoom-in")));
+	registerAction(ZoomOutAction, QT_TRANSLATE_NOOP("actions", "Zoom Out"), QString(), ThemesManager::getIcon(QLatin1String("zoom-out")));
+	registerAction(ZoomOriginalAction, QT_TRANSLATE_NOOP("actions", "Zoom Original"), QString(), ThemesManager::getIcon(QLatin1String("zoom-original")));
 	registerAction(ScrollToStartAction, QT_TRANSLATE_NOOP("actions", "Go to Start of the Page"));
 	registerAction(ScrollToEndAction, QT_TRANSLATE_NOOP("actions", "Go to the End of the Page"));
 	registerAction(ScrollPageUpAction, QT_TRANSLATE_NOOP("actions", "Page Up"));
@@ -387,8 +455,8 @@ ActionsManager::ActionsManager(QObject *parent) : QObject(parent),
 	registerAction(StartDragScrollAction, QT_TRANSLATE_NOOP("actions", "Enter Drag Scroll Mode"));
 	registerAction(StartMoveScrollAction, QT_TRANSLATE_NOOP("actions", "Enter Move Scroll Mode"));
 	registerAction(EndScrollAction, QT_TRANSLATE_NOOP("actions", "Exit Scroll Mode"));
-	registerAction(PrintAction, QT_TRANSLATE_NOOP("actions", "Print…"), QString(), Utils::getIcon(QLatin1String("document-print")));
-	registerAction(PrintPreviewAction, QT_TRANSLATE_NOOP("actions", "Print Preview"), QString(), Utils::getIcon(QLatin1String("document-print-preview")));
+	registerAction(PrintAction, QT_TRANSLATE_NOOP("actions", "Print…"), QString(), ThemesManager::getIcon(QLatin1String("document-print")));
+	registerAction(PrintPreviewAction, QT_TRANSLATE_NOOP("actions", "Print Preview"), QString(), ThemesManager::getIcon(QLatin1String("document-print-preview")));
 	registerAction(ActivateAddressFieldAction, QT_TRANSLATE_NOOP("actions", "Activate Address Field"));
 	registerAction(ActivateSearchFieldAction, QT_TRANSLATE_NOOP("actions", "Activate Search Field"));
 	registerAction(ActivateContentAction, QT_TRANSLATE_NOOP("actions", "Activate Content"));
@@ -397,8 +465,8 @@ ActionsManager::ActionsManager(QObject *parent) : QObject(parent),
 	registerAction(ActivateTabAction, QT_TRANSLATE_NOOP("actions", "Activate Tab"));
 	registerAction(ActivateTabOnLeftAction, QT_TRANSLATE_NOOP("actions", "Go to Tab on Left"));
 	registerAction(ActivateTabOnRightAction, QT_TRANSLATE_NOOP("actions", "Go to Tab on Right"));
-	registerAction(BookmarksAction, QT_TRANSLATE_NOOP("actions", "Manage Bookmarks"), QString(), Utils::getIcon(QLatin1String("bookmarks-organize")));
-	registerAction(BookmarkPageAction, QT_TRANSLATE_NOOP("actions", "Bookmark Page…"), QString(), Utils::getIcon(QLatin1String("bookmark-new")));
+	registerAction(BookmarksAction, QT_TRANSLATE_NOOP("actions", "Manage Bookmarks"), QString(), ThemesManager::getIcon(QLatin1String("bookmarks-organize")));
+	registerAction(BookmarkPageAction, QT_TRANSLATE_NOOP("actions", "Bookmark Page…"), QString(), ThemesManager::getIcon(QLatin1String("bookmark-new")));
 	registerAction(BookmarkAllOpenPagesAction, QT_TRANSLATE_NOOP("actions", "Bookmark All Open Pages"));
 	registerAction(OpenBookmarkAction, QT_TRANSLATE_NOOP("actions", "Open Bookmark"));
 	registerAction(QuickBookmarkAccessAction, QT_TRANSLATE_NOOP("actions", "Quick Bookmark Access"));
@@ -408,7 +476,7 @@ ActionsManager::ActionsManager(QObject *parent) : QObject(parent),
 	registerAction(CookiesPolicyAction, QT_TRANSLATE_NOOP("actions", "Cookies Policy"));
 	registerAction(ThirdPartyCookiesPolicyAction, QT_TRANSLATE_NOOP("actions", "Third-party Cookies Policy"));
 	registerAction(PluginsPolicyAction, QT_TRANSLATE_NOOP("actions", "Plugins"));
-	registerAction(LoadPluginsAction, QT_TRANSLATE_NOOP("actions", "Load Plugins"), QString(), Utils::getIcon(QLatin1String("preferences-plugin")));
+	registerAction(LoadPluginsAction, QT_TRANSLATE_NOOP("actions", "Load Plugins"), QString(), ThemesManager::getIcon(QLatin1String("preferences-plugin")));
 	registerAction(EnableJavaScriptAction, QT_TRANSLATE_NOOP("actions", "Enable JavaScript"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag | IsCheckedFlag));
 	registerAction(EnableJavaAction, QT_TRANSLATE_NOOP("actions", "Enable Java"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag | IsCheckedFlag));
 	registerAction(EnableReferrerAction, QT_TRANSLATE_NOOP("actions", "Enable Referrer"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag | IsCheckedFlag));
@@ -420,19 +488,22 @@ ActionsManager::ActionsManager(QObject *parent) : QObject(parent),
 	registerAction(InspectPageAction, QT_TRANSLATE_NOOP("actions", "Inspect Page"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag));
 	registerAction(InspectElementAction, QT_TRANSLATE_NOOP("actions", "Inspect Element…"));
 	registerAction(WorkOfflineAction, QT_TRANSLATE_NOOP("actions", "Work Offline"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag));
-	registerAction(FullScreenAction, QT_TRANSLATE_NOOP("actions", "Full Screen"), QString(), Utils::getIcon(QLatin1String("view-fullscreen")));
+	registerAction(FullScreenAction, QT_TRANSLATE_NOOP("actions", "Full Screen"), QString(), ThemesManager::getIcon(QLatin1String("view-fullscreen")));
 	registerAction(ShowTabSwitcherAction, QT_TRANSLATE_NOOP("actions", "Show Tab Switcher"));
 	registerAction(ShowMenuBarAction, QT_TRANSLATE_NOOP("actions", "Show Menubar"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag | IsCheckedFlag));
 	registerAction(ShowTabBarAction, QT_TRANSLATE_NOOP("actions", "Show Tabbar"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag | IsCheckedFlag));
 	registerAction(ShowSidebarAction, QT_TRANSLATE_NOOP("actions", "Show Sidebar"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag));
 	registerAction(ShowErrorConsoleAction, QT_TRANSLATE_NOOP("actions", "Show Error Console"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag));
 	registerAction(LockToolBarsAction, QT_TRANSLATE_NOOP("actions", "Lock Toolbars"), QString(), QIcon(), (IsEnabledFlag | IsCheckableFlag));
-	registerAction(OpenPanelAction, QT_TRANSLATE_NOOP("actions", "Open Panel as Tab"), QString(), Utils::getIcon(QLatin1String("arrow-right")));
-	registerAction(ClosePanelAction, QT_TRANSLATE_NOOP("actions", "Close Panel"), QString(), Utils::getIcon(QLatin1String("window-close")));
+	registerAction(ResetToolBarsAction, QT_TRANSLATE_NOOP("actions", "Reset to Defaults…"), QT_TRANSLATE_NOOP("actions", "Reset Toolbars to Defaults…"), QIcon(), (IsEnabledFlag | IsCheckableFlag));
+	registerAction(OpenPanelAction, QT_TRANSLATE_NOOP("actions", "Open Panel as Tab"), QString(), ThemesManager::getIcon(QLatin1String("arrow-right")));
+	registerAction(ClosePanelAction, QT_TRANSLATE_NOOP("actions", "Close Panel"), QString(), ThemesManager::getIcon(QLatin1String("window-close")));
 	registerAction(ContentBlockingAction, QT_TRANSLATE_NOOP("actions", "Content Blocking…"));
-	registerAction(HistoryAction, QT_TRANSLATE_NOOP("actions", "View History"), QString(), Utils::getIcon(QLatin1String("view-history")));
-	registerAction(ClearHistoryAction, QT_TRANSLATE_NOOP("actions", "Clear History…"), QString(), Utils::getIcon(QLatin1String("edit-clear-history")));
+	registerAction(HistoryAction, QT_TRANSLATE_NOOP("actions", "View History"), QString(), ThemesManager::getIcon(QLatin1String("view-history")));
+	registerAction(ClearHistoryAction, QT_TRANSLATE_NOOP("actions", "Clear History…"), QString(), ThemesManager::getIcon(QLatin1String("edit-clear-history")));
+	registerAction(AddonsAction, QT_TRANSLATE_NOOP("actions", "Addons"));
 	registerAction(NotesAction, QT_TRANSLATE_NOOP("actions", "Notes"));
+	registerAction(PasswordsAction, QT_TRANSLATE_NOOP("actions", "Passwords"));
 	registerAction(TransfersAction, QT_TRANSLATE_NOOP("actions", "Transfers"));
 	registerAction(PreferencesAction, QT_TRANSLATE_NOOP("actions", "Preferences…"));
 	registerAction(WebsitePreferencesAction, QT_TRANSLATE_NOOP("actions", "Website Preferences…"));
@@ -440,14 +511,14 @@ ActionsManager::ActionsManager(QObject *parent) : QObject(parent),
 	registerAction(ResetQuickPreferencesAction, QT_TRANSLATE_NOOP("actions", "Reset Options"));
 	registerAction(WebsiteInformationAction, QT_TRANSLATE_NOOP("actions", "Website Information…"));
 	registerAction(WebsiteCertificateInformationAction, QT_TRANSLATE_NOOP("actions", "Website Certificate Information…"));
-	registerAction(SwitchApplicationLanguageAction, QT_TRANSLATE_NOOP("actions", "Switch Application Language…"), QString(), Utils::getIcon(QLatin1String("preferences-desktop-locale")));
+	registerAction(SwitchApplicationLanguageAction, QT_TRANSLATE_NOOP("actions", "Switch Application Language…"), QString(), ThemesManager::getIcon(QLatin1String("preferences-desktop-locale")));
 	registerAction(CheckForUpdatesAction, QT_TRANSLATE_NOOP("actions", "Check for Updates…"));
 	registerAction(DiagnosticReportAction, QT_TRANSLATE_NOOP("actions", "Diagnostic Report…"));
-	registerAction(AboutApplicationAction, QT_TRANSLATE_NOOP("actions", "About Otter…"), QString(), Utils::getIcon(QLatin1String("otter-browser"), NoFlags));
-	registerAction(AboutQtAction, QT_TRANSLATE_NOOP("actions", "About Qt…"), QString(), Utils::getIcon(QLatin1String("qt"), NoFlags));
-	registerAction(ExitAction, QT_TRANSLATE_NOOP("actions", "Exit"), QString(), Utils::getIcon(QLatin1String("application-exit")));
+	registerAction(AboutApplicationAction, QT_TRANSLATE_NOOP("actions", "About Otter…"), QString(), ThemesManager::getIcon(QLatin1String("otter-browser"), NoFlags));
+	registerAction(AboutQtAction, QT_TRANSLATE_NOOP("actions", "About Qt…"), QString(), ThemesManager::getIcon(QLatin1String("qt"), NoFlags));
+	registerAction(ExitAction, QT_TRANSLATE_NOOP("actions", "Exit"), QString(), ThemesManager::getIcon(QLatin1String("application-exit")));
 
-	connect(SettingsManager::getInstance(), SIGNAL(valueChanged(QString,QVariant)), this, SLOT(optionChanged(QString)));
+	connect(SettingsManager::getInstance(), SIGNAL(valueChanged(int,QVariant)), this, SLOT(optionChanged(int)));
 }
 
 void ActionsManager::createInstance(QObject *parent)
@@ -455,6 +526,7 @@ void ActionsManager::createInstance(QObject *parent)
 	if (!m_instance)
 	{
 		m_instance = new ActionsManager(parent);
+		m_actionIdentifierEnumerator = m_instance->metaObject()->indexOfEnumerator(QLatin1String("ActionIdentifier").data());
 
 		loadProfiles();
 	}
@@ -472,9 +544,9 @@ void ActionsManager::timerEvent(QTimerEvent *event)
 	}
 }
 
-void ActionsManager::optionChanged(const QString &option)
+void ActionsManager::optionChanged(int identifier)
 {
-	if ((option == QLatin1String("Browser/KeyboardShortcutsProfilesOrder") || option == QLatin1String("Browser/EnableSingleKeyShortcuts")) && m_reloadTimer == 0)
+	if ((identifier == SettingsManager::Browser_KeyboardShortcutsProfilesOrderOption || identifier == SettingsManager::Browser_EnableSingleKeyShortcutsOption) && m_reloadTimer == 0)
 	{
 		m_reloadTimer = startTimer(250);
 	}
@@ -484,19 +556,19 @@ void ActionsManager::loadProfiles()
 {
 	QHash<int, QVector<QKeySequence> > actionShortcuts;
 	QVector<QKeySequence> allShortcuts;
-	const QStringList shortcutProfiles = SettingsManager::getValue(QLatin1String("Browser/KeyboardShortcutsProfilesOrder")).toStringList();
-	const bool enableSingleKeyShortcuts = SettingsManager::getValue(QLatin1String("Browser/EnableSingleKeyShortcuts")).toBool();
+	const QStringList shortcutProfiles(SettingsManager::getValue(SettingsManager::Browser_KeyboardShortcutsProfilesOrderOption).toStringList());
+	const bool enableSingleKeyShortcuts(SettingsManager::getValue(SettingsManager::Browser_EnableSingleKeyShortcutsOption).toBool());
 	QRegularExpression functionKeyExpression(QLatin1String("F\\d+"));
 
 	for (int i = 0; i < shortcutProfiles.count(); ++i)
 	{
 		const QSettings profile(SessionsManager::getReadableDataPath(QLatin1String("keyboard/") + shortcutProfiles.at(i) + QLatin1String(".ini")), QSettings::IniFormat);
-		const QStringList rawActions = profile.childGroups();
+		const QStringList rawActions(profile.childGroups());
 
 		for (int j = 0; j < rawActions.count(); ++j)
 		{
-			const int action = ActionsManager::getActionIdentifier(rawActions.at(j));
-			const QStringList rawShortcuts = profile.value(rawActions.at(j) + QLatin1String("/shortcuts"), QString()).toString().split(QLatin1Char(' '), QString::SkipEmptyParts);
+			const int action(ActionsManager::getActionIdentifier(rawActions.at(j)));
+			const QStringList rawShortcuts(profile.value(rawActions.at(j) + QLatin1String("/shortcuts"), QString()).toString().split(QLatin1Char(' '), QString::SkipEmptyParts));
 			QVector<QKeySequence> shortcuts;
 
 			if (actionShortcuts.contains(action))
@@ -537,7 +609,7 @@ void ActionsManager::loadProfiles()
 
 void ActionsManager::triggerAction(int identifier, QObject *parent, const QVariantMap &parameters)
 {
-	MainWindow *window = MainWindow::findMainWindow(parent);
+	MainWindow *window(MainWindow::findMainWindow(parent));
 
 	if (window)
 	{
@@ -545,6 +617,18 @@ void ActionsManager::triggerAction(int identifier, QObject *parent, const QVaria
 	}
 }
 
+void ActionsManager::registerAction(int identifier, const QString &text, const QString &description, const QIcon &icon, ActionFlags flags)
+{
+	ActionsManager::ActionDefinition action;
+	action.text = text;
+	action.description = description;
+	action.icon = icon;
+	action.identifier = identifier;
+	action.flags = flags;
+
+	m_definitions.append(action);
+}
+
 ActionsManager* ActionsManager::getInstance()
 {
 	return m_instance;
@@ -552,9 +636,9 @@ ActionsManager* ActionsManager::getInstance()
 
 Action* ActionsManager::getAction(int identifier, QObject *parent)
 {
-	MainWindow *window = MainWindow::findMainWindow(parent);
+	MainWindow *window(MainWindow::findMainWindow(parent));
 
-	return (window ? window->getAction(identifier) : NULL);
+	return (window ? window->getAction(identifier) : nullptr);
 }
 
 QString ActionsManager::getReport()
@@ -592,7 +676,7 @@ QString ActionsManager::getReport()
 
 QString ActionsManager::getActionName(int identifier)
 {
-	QString name = m_instance->metaObject()->enumerator(m_instance->metaObject()->indexOfEnumerator(QLatin1String("ActionIdentifier").data())).valueToKey(identifier);
+	QString name(m_instance->metaObject()->enumerator(m_actionIdentifierEnumerator).valueToKey(identifier));
 
 	if (!name.isEmpty())
 	{
@@ -619,32 +703,14 @@ ActionsManager::ActionDefinition ActionsManager::getActionDefinition(int identif
 	return m_definitions[identifier];
 }
 
-int ActionsManager::registerAction(int identifier, const QString &text, const QString &description, const QIcon &icon, ActionFlags flags)
-{
-	ActionsManager::ActionDefinition action;
-	action.text = text;
-	action.description = description;
-	action.icon = icon;
-	action.identifier = identifier;
-	action.flags = flags;
-
-	m_definitions.append(action);
-
-	return (m_definitions.count() - 1);
-}
-
 int ActionsManager::getActionIdentifier(const QString &name)
 {
-	const int enumerator = m_instance->metaObject()->indexOfEnumerator(QLatin1String("ActionIdentifier").data());
-
 	if (!name.endsWith(QLatin1String("Action")))
 	{
-		const QString fullName = name + QLatin1String("Action");
-
-		return m_instance->metaObject()->enumerator(enumerator).keyToValue(fullName.toLatin1());
+		return m_instance->metaObject()->enumerator(m_actionIdentifierEnumerator).keyToValue(QString(name + QLatin1String("Action")).toLatin1());
 	}
 
-	return m_instance->metaObject()->enumerator(enumerator).keyToValue(name.toLatin1());
+	return m_instance->metaObject()->enumerator(m_actionIdentifierEnumerator).keyToValue(name.toLatin1());
 }
 
 }
