@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -20,11 +20,14 @@
 #include "BookmarksModel.h"
 #include "Console.h"
 #include "HistoryManager.h"
+#include "SessionsManager.h"
+#include "ThemesManager.h"
 #include "Utils.h"
 
 #include <QtCore/QCoreApplication>
 #include <QtCore/QFile>
 #include <QtCore/QMimeData>
+#include <QtCore/QSaveFile>
 #include <QtWidgets/QMessageBox>
 
 namespace Otter
@@ -36,7 +39,7 @@ BookmarksItem::BookmarksItem() : QStandardItem()
 
 void BookmarksItem::remove()
 {
-	BookmarksModel *model = qobject_cast<BookmarksModel*>(this->model());
+	BookmarksModel *model(qobject_cast<BookmarksModel*>(this->model()));
 
 	if (model)
 	{
@@ -67,7 +70,7 @@ void BookmarksItem::setItemData(const QVariant &value, int role)
 
 QStandardItem* BookmarksItem::clone() const
 {
-	BookmarksItem *item = new BookmarksItem();
+	BookmarksItem *item(new BookmarksItem());
 	item->setData(data(BookmarksModel::TypeRole), BookmarksModel::TypeRole);
 	item->setData(data(BookmarksModel::UrlRole), BookmarksModel::UrlRole);
 	item->setData(data(BookmarksModel::TitleRole), BookmarksModel::TitleRole);
@@ -85,11 +88,11 @@ QVariant BookmarksItem::data(int role) const
 {
 	if (role == Qt::DisplayRole)
 	{
-		const BookmarksModel::BookmarkType type = static_cast<BookmarksModel::BookmarkType>(data(BookmarksModel::TypeRole).toInt());
+		const BookmarksModel::BookmarkType type(static_cast<BookmarksModel::BookmarkType>(data(BookmarksModel::TypeRole).toInt()));
 
 		if (type == BookmarksModel::RootBookmark)
 		{
-			BookmarksModel *model = qobject_cast<BookmarksModel*>(this->model());
+			BookmarksModel *model(qobject_cast<BookmarksModel*>(this->model()));
 
 			if (model && model->getFormatMode() == BookmarksModel::NotesMode)
 			{
@@ -107,16 +110,16 @@ QVariant BookmarksItem::data(int role) const
 
 	if (role == Qt::DecorationRole)
 	{
-		const BookmarksModel::BookmarkType type = static_cast<BookmarksModel::BookmarkType>(data(BookmarksModel::TypeRole).toInt());
+		const BookmarksModel::BookmarkType type(static_cast<BookmarksModel::BookmarkType>(data(BookmarksModel::TypeRole).toInt()));
 
 		if (type == BookmarksModel::RootBookmark || type == BookmarksModel::FolderBookmark)
 		{
-			return Utils::getIcon(QLatin1String("inode-directory"));
+			return ThemesManager::getIcon(QLatin1String("inode-directory"));
 		}
 
 		if (type == BookmarksModel::TrashBookmark)
 		{
-			return Utils::getIcon(QLatin1String("user-trash"));
+			return ThemesManager::getIcon(QLatin1String("user-trash"));
 		}
 
 		if (type == BookmarksModel::UrlBookmark)
@@ -134,11 +137,11 @@ QVariant BookmarksItem::data(int role) const
 
 	if (role == BookmarksModel::IsTrashedRole)
 	{
-		QModelIndex parent = index().parent();
+		QModelIndex parent(index().parent());
 
 		while (parent.isValid())
 		{
-			const BookmarksModel::BookmarkType type = static_cast<BookmarksModel::BookmarkType>(parent.data(BookmarksModel::TypeRole).toInt());
+			const BookmarksModel::BookmarkType type(static_cast<BookmarksModel::BookmarkType>(parent.data(BookmarksModel::TypeRole).toInt()));
 
 			if (type == BookmarksModel::TrashBookmark)
 			{
@@ -170,14 +173,14 @@ QList<QUrl> BookmarksItem::getUrls() const
 
 	for (int i = 0; i < rowCount(); ++i)
 	{
-		BookmarksItem *bookmark = dynamic_cast<BookmarksItem*>(child(i, 0));
+		BookmarksItem *bookmark(dynamic_cast<BookmarksItem*>(child(i, 0)));
 
 		if (!bookmark)
 		{
 			continue;
 		}
 
-		const BookmarksModel::BookmarkType type = static_cast<BookmarksModel::BookmarkType>(bookmark->data(BookmarksModel::TypeRole).toInt());
+		const BookmarksModel::BookmarkType type(static_cast<BookmarksModel::BookmarkType>(bookmark->data(BookmarksModel::TypeRole).toInt()));
 
 		if (type == BookmarksModel::FolderBookmark)
 		{
@@ -193,28 +196,27 @@ QList<QUrl> BookmarksItem::getUrls() const
 }
 
 BookmarksModel::BookmarksModel(const QString &path, FormatMode mode, QObject *parent) : QStandardItemModel(parent),
+	m_rootItem(new BookmarksItem()),
+	m_trashItem(new BookmarksItem()),
 	m_mode(mode)
 {
-	BookmarksItem *rootItem = new BookmarksItem();
-	rootItem->setData(RootBookmark, TypeRole);
-	rootItem->setData(((mode == NotesMode) ? tr("Notes") : tr("Bookmarks")), TitleRole);
-	rootItem->setDragEnabled(false);
+	m_rootItem->setData(RootBookmark, TypeRole);
+	m_rootItem->setData(((mode == NotesMode) ? tr("Notes") : tr("Bookmarks")), TitleRole);
+	m_rootItem->setDragEnabled(false);
+	m_trashItem->setData(TrashBookmark, TypeRole);
+	m_trashItem->setData(tr("Trash"), TitleRole);
+	m_trashItem->setDragEnabled(false);
+	m_trashItem->setEnabled(false);
 
-	BookmarksItem *trashItem = new BookmarksItem();
-	trashItem->setData(TrashBookmark, TypeRole);
-	trashItem->setData(tr("Trash"), TitleRole);
-	trashItem->setDragEnabled(false);
-	trashItem->setEnabled(false);
-
-	appendRow(rootItem);
-	appendRow(trashItem);
+	appendRow(m_rootItem);
+	appendRow(m_trashItem);
 	setItemPrototype(new BookmarksItem());
 
 	QFile file(path);
 
-	if (!file.open(QFile::ReadOnly | QFile::Text))
+	if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
 	{
-		Console::addMessage(((mode == NotesMode) ? tr("Failed to open notes file: %1") : tr("Failed to open bookmarks file: %1")).arg(file.errorString()), OtherMessageCategory, ErrorMessageLevel, path);
+		Console::addMessage(((mode == NotesMode) ? tr("Failed to open notes file: %1") : tr("Failed to open bookmarks file: %1")).arg(file.errorString()), Console::OtherCategory, Console::ErrorLevel, path);
 
 		return;
 	}
@@ -227,7 +229,7 @@ BookmarksModel::BookmarksModel(const QString &path, FormatMode mode, QObject *pa
 		{
 			if (reader.name() == QLatin1String("folder") || reader.name() == QLatin1String("bookmark") || reader.name() == QLatin1String("separator"))
 			{
-				readBookmark(&reader, rootItem);
+				readBookmark(&reader, m_rootItem);
 			}
 			else
 			{
@@ -238,9 +240,9 @@ BookmarksModel::BookmarksModel(const QString &path, FormatMode mode, QObject *pa
 			{
 				getRootItem()->removeRows(0, getRootItem()->rowCount());
 
-				Console::addMessage(((m_mode == NotesMode) ? tr("Failed to load notes file: %1") : tr("Failed to load bookmarks file: %1")).arg(reader.errorString()), OtherMessageCategory, ErrorMessageLevel, path);
+				Console::addMessage(((m_mode == NotesMode) ? tr("Failed to load notes file: %1") : tr("Failed to load bookmarks file: %1")).arg(reader.errorString()), Console::OtherCategory, Console::ErrorLevel, path);
 
-				QMessageBox::warning(NULL, tr("Error"), ((m_mode == NotesMode) ? tr("Failed to load notes file.") : tr("Failed to load bookmarks file.")), QMessageBox::Close);
+				QMessageBox::warning(nullptr, tr("Error"), ((m_mode == NotesMode) ? tr("Failed to load notes file.") : tr("Failed to load bookmarks file.")), QMessageBox::Close);
 
 				return;
 			}
@@ -249,7 +251,9 @@ BookmarksModel::BookmarksModel(const QString &path, FormatMode mode, QObject *pa
 
 	connect(this, SIGNAL(itemChanged(QStandardItem*)), this, SIGNAL(modelModified()));
 	connect(this, SIGNAL(rowsInserted(QModelIndex,int,int)), this, SIGNAL(modelModified()));
+	connect(this, SIGNAL(rowsInserted(QModelIndex,int,int)), this, SLOT(notifyBookmarkModified(QModelIndex)));
 	connect(this, SIGNAL(rowsRemoved(QModelIndex,int,int)), this, SIGNAL(modelModified()));
+	connect(this, SIGNAL(rowsRemoved(QModelIndex,int,int)), this, SLOT(notifyBookmarkModified(QModelIndex)));
 	connect(this, SIGNAL(rowsMoved(QModelIndex,int,int,QModelIndex,int)), this, SIGNAL(modelModified()));
 }
 
@@ -260,7 +264,7 @@ void BookmarksModel::trashBookmark(BookmarksItem *bookmark)
 		return;
 	}
 
-	const BookmarkType type = static_cast<BookmarkType>(bookmark->data(TypeRole).toInt());
+	const BookmarkType type(static_cast<BookmarkType>(bookmark->data(TypeRole).toInt()));
 
 	if (type != RootBookmark && type != TrashBookmark)
 	{
@@ -270,7 +274,7 @@ void BookmarksModel::trashBookmark(BookmarksItem *bookmark)
 		}
 		else
 		{
-			BookmarksItem *trashItem = getTrashItem();
+			BookmarksItem *trashItem(getTrashItem());
 
 			m_trash[bookmark] = qMakePair(bookmark->parent()->index(), bookmark->row());
 
@@ -293,7 +297,7 @@ void BookmarksModel::restoreBookmark(BookmarksItem *bookmark)
 		return;
 	}
 
-	BookmarksItem *formerParent = (m_trash.contains(bookmark) ? dynamic_cast<BookmarksItem*>(itemFromIndex(m_trash[bookmark].first)) : getRootItem());
+	BookmarksItem *formerParent(m_trash.contains(bookmark) ? getBookmark(m_trash[bookmark].first) : getRootItem());
 
 	if (!formerParent || static_cast<BookmarkType>(formerParent->data(TypeRole).toInt()) != FolderBookmark)
 	{
@@ -313,7 +317,7 @@ void BookmarksModel::restoreBookmark(BookmarksItem *bookmark)
 
 	readdBookmarkUrl(bookmark);
 
-	BookmarksItem *trashItem = getTrashItem();
+	BookmarksItem *trashItem(getTrashItem());
 
 	trashItem->setEnabled(trashItem->rowCount() > 0);
 
@@ -331,7 +335,7 @@ void BookmarksModel::removeBookmark(BookmarksItem *bookmark)
 
 	removeBookmarkUrl(bookmark);
 
-	const quint64 identifier = bookmark->data(IdentifierRole).toULongLong();
+	const quint64 identifier(bookmark->data(IdentifierRole).toULongLong());
 
 	if (identifier > 0 && m_identifiers.contains(identifier))
 	{
@@ -352,7 +356,7 @@ void BookmarksModel::removeBookmark(BookmarksItem *bookmark)
 
 void BookmarksModel::readBookmark(QXmlStreamReader *reader, BookmarksItem *parent)
 {
-	BookmarksItem *bookmark = NULL;
+	BookmarksItem *bookmark(nullptr);
 
 	if (reader->name() == QLatin1String("folder"))
 	{
@@ -382,7 +386,7 @@ void BookmarksModel::readBookmark(QXmlStreamReader *reader, BookmarksItem *paren
 					{
 						if (reader->isStartElement())
 						{
-							if (reader->name() == QLatin1String("metadata") && reader->attributes().value(QLatin1String("owner")).toString().startsWith("http://otter-browser.org/"))
+							if (reader->name() == QLatin1String("metadata") && reader->attributes().value(QLatin1String("owner")).toString().startsWith(QLatin1String("http://otter-browser.org/")))
 							{
 								while (reader->readNext())
 								{
@@ -641,7 +645,7 @@ void BookmarksModel::removeBookmarkUrl(BookmarksItem *bookmark)
 	}
 	else if (!bookmark->data(UrlRole).toUrl().isEmpty())
 	{
-		const QUrl url = Utils::normalizeUrl(bookmark->data(UrlRole).toUrl());
+		const QUrl url(Utils::normalizeUrl(bookmark->data(UrlRole).toUrl()));
 
 		if (m_urls.contains(url))
 		{
@@ -671,7 +675,7 @@ void BookmarksModel::readdBookmarkUrl(BookmarksItem *bookmark)
 	}
 	else if (!bookmark->data(UrlRole).toUrl().isEmpty())
 	{
-		const QUrl url = Utils::normalizeUrl(bookmark->data(UrlRole).toUrl());
+		const QUrl url(Utils::normalizeUrl(bookmark->data(UrlRole).toUrl()));
 
 		if (!m_urls.contains(url))
 		{
@@ -684,7 +688,7 @@ void BookmarksModel::readdBookmarkUrl(BookmarksItem *bookmark)
 
 void BookmarksModel::emptyTrash()
 {
-	BookmarksItem *trashItem = getTrashItem();
+	BookmarksItem *trashItem(getTrashItem());
 	trashItem->removeRows(0, trashItem->rowCount());
 	trashItem->setEnabled(false);
 
@@ -693,9 +697,19 @@ void BookmarksModel::emptyTrash()
 	emit modelModified();
 }
 
+void BookmarksModel::notifyBookmarkModified(const QModelIndex &index)
+{
+	BookmarksItem *bookmark(getBookmark(index));
+
+	if (bookmark)
+	{
+		emit bookmarkModified(bookmark);
+	}
+}
+
 BookmarksItem* BookmarksModel::addBookmark(BookmarkType type, quint64 identifier, const QUrl &url, const QString &title, BookmarksItem *parent, int index)
 {
-	BookmarksItem *bookmark = new BookmarksItem();
+	BookmarksItem *bookmark(new BookmarksItem());
 
 	if (parent)
 	{
@@ -745,7 +759,7 @@ BookmarksItem* BookmarksModel::getBookmark(const QString &keyword) const
 		return m_keywords[keyword];
 	}
 
-	return NULL;
+	return nullptr;
 }
 
 BookmarksItem* BookmarksModel::getBookmark(const QModelIndex &index) const
@@ -765,17 +779,17 @@ BookmarksItem* BookmarksModel::getBookmark(quint64 identifier) const
 		return m_identifiers[identifier];
 	}
 
-	return NULL;
+	return nullptr;
 }
 
 BookmarksItem* BookmarksModel::getRootItem() const
 {
-	return dynamic_cast<BookmarksItem*>(item(0, 0));
+	return m_rootItem;
 }
 
 BookmarksItem* BookmarksModel::getTrashItem() const
 {
-	return dynamic_cast<BookmarksItem*>(item(1, 0));
+	return m_trashItem;
 }
 
 BookmarksItem* BookmarksModel::getItem(const QString &path) const
@@ -790,12 +804,12 @@ BookmarksItem* BookmarksModel::getItem(const QString &path) const
 		return getBookmark(path.mid(1).toULongLong());
 	}
 
-	QStandardItem *item = getRootItem();
-	const QStringList directories = path.split(QLatin1Char('/'), QString::SkipEmptyParts);
+	QStandardItem *item(getRootItem());
+	const QStringList directories(path.split(QLatin1Char('/'), QString::SkipEmptyParts));
 
 	for (int i = 0; i < directories.count(); ++i)
 	{
-		bool found = false;
+		bool hasFound(false);
 
 		for (int j = 0; j < item->rowCount(); ++j)
 		{
@@ -803,15 +817,15 @@ BookmarksItem* BookmarksModel::getItem(const QString &path) const
 			{
 				item = item->child(j);
 
-				found = true;
+				hasFound = true;
 
 				break;
 			}
 		}
 
-		if (!found)
+		if (!hasFound)
 		{
-			return NULL;
+			return nullptr;
 		}
 	}
 
@@ -820,7 +834,7 @@ BookmarksItem* BookmarksModel::getItem(const QString &path) const
 
 QMimeData* BookmarksModel::mimeData(const QModelIndexList &indexes) const
 {
-	QMimeData *mimeData = new QMimeData();
+	QMimeData *mimeData(new QMimeData());
 	QStringList texts;
 	QList<QUrl> urls;
 
@@ -894,7 +908,7 @@ QList<BookmarksModel::BookmarkMatch> BookmarksModel::findBookmarks(const QString
 			continue;
 		}
 
-		const QString result = Utils::matchUrl(urlsIterator.key(), prefix);
+		const QString result(Utils::matchUrl(urlsIterator.key(), prefix));
 
 		if (!result.isEmpty())
 		{
@@ -931,11 +945,11 @@ QList<BookmarksItem*> BookmarksModel::findUrls(const QUrl &url, QStandardItem *b
 
 	for (int i = 0; i < branch->rowCount(); ++i)
 	{
-		BookmarksItem *item = dynamic_cast<BookmarksItem*>(branch->child(i));
+		BookmarksItem *item(dynamic_cast<BookmarksItem*>(branch->child(i)));
 
 		if (item)
 		{
-			const BookmarkType type = static_cast<BookmarkType>(item->data(TypeRole).toInt());
+			const BookmarkType type(static_cast<BookmarkType>(item->data(TypeRole).toInt()));
 
 			if (type == FolderBookmark)
 			{
@@ -953,7 +967,7 @@ QList<BookmarksItem*> BookmarksModel::findUrls(const QUrl &url, QStandardItem *b
 
 QList<BookmarksItem*> BookmarksModel::getBookmarks(const QUrl &url) const
 {
-	const QUrl adjustedUrl = Utils::normalizeUrl(url);
+	const QUrl adjustedUrl(Utils::normalizeUrl(url));
 
 	if (m_urls.contains(adjustedUrl))
 	{
@@ -975,7 +989,7 @@ bool BookmarksModel::moveBookmark(BookmarksItem *bookmark, BookmarksItem *newPar
 		return false;
 	}
 
-	BookmarksItem *previousParent = dynamic_cast<BookmarksItem*>(bookmark->parent());
+	BookmarksItem *previousParent(dynamic_cast<BookmarksItem*>(bookmark->parent()));
 
 	if (!previousParent)
 	{
@@ -993,7 +1007,7 @@ bool BookmarksModel::moveBookmark(BookmarksItem *bookmark, BookmarksItem *newPar
 		return true;
 	}
 
-	const int previousRow = bookmark->row();
+	const int previousRow(bookmark->row());
 
 	if (newRow < 0)
 	{
@@ -1005,7 +1019,7 @@ bool BookmarksModel::moveBookmark(BookmarksItem *bookmark, BookmarksItem *newPar
 		return true;
 	}
 
-	int targetRow = newRow;
+	int targetRow(newRow);
 
 	if (bookmark->parent() == newParent && bookmark->row() < newRow)
 	{
@@ -1022,15 +1036,27 @@ bool BookmarksModel::moveBookmark(BookmarksItem *bookmark, BookmarksItem *newPar
 
 bool BookmarksModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent)
 {
-	const BookmarkType type = static_cast<BookmarkType>(parent.data(TypeRole).toInt());
+	const BookmarkType type(static_cast<BookmarkType>(parent.data(TypeRole).toInt()));
 
 	if (type == FolderBookmark || type == RootBookmark || type == TrashBookmark)
 	{
-		const QModelIndex index = data->property("x-item-index").toModelIndex();
+		const QModelIndex index(data->property("x-item-index").toModelIndex());
 
 		if (index.isValid())
 		{
-			return moveBookmark(dynamic_cast<BookmarksItem*>(itemFromIndex(index)), dynamic_cast<BookmarksItem*>(itemFromIndex(parent)), row);
+			return moveBookmark(getBookmark(index), getBookmark(parent), row);
+		}
+
+		if (data->hasUrls())
+		{
+			const QList<QUrl> urls(data->urls());
+
+			for (int i = 0; i < urls.count(); ++i)
+			{
+				addBookmark(UrlBookmark, 0, urls.at(i), (data->property("x-url-title").toString().isEmpty() ? urls.at(i).toString() : data->property("x-url-title").toString()), getBookmark(parent), row);
+			}
+
+			return true;
 		}
 
 		return QStandardItemModel::dropMimeData(data, action, row, column, parent);
@@ -1041,9 +1067,14 @@ bool BookmarksModel::dropMimeData(const QMimeData *data, Qt::DropAction action,
 
 bool BookmarksModel::save(const QString &path) const
 {
-	QFile file(path);
+	if (SessionsManager::isReadOnly())
+	{
+		return false;
+	}
 
-	if (!file.open(QFile::WriteOnly))
+	QSaveFile file(path);
+
+	if (!file.open(QIODevice::WriteOnly))
 	{
 		return false;
 	}
@@ -1056,7 +1087,7 @@ bool BookmarksModel::save(const QString &path) const
 	writer.writeStartElement(QLatin1String("xbel"));
 	writer.writeAttribute(QLatin1String("version"), QLatin1String("1.0"));
 
-	QStandardItem *rootItem = item(0, 0);
+	QStandardItem *rootItem(item(0, 0));
 
 	for (int i = 0; i < rootItem->rowCount(); ++i)
 	{
@@ -1065,12 +1096,12 @@ bool BookmarksModel::save(const QString &path) const
 
 	writer.writeEndDocument();
 
-	return true;
+	return file.commit();
 }
 
 bool BookmarksModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
-	BookmarksItem *bookmark = dynamic_cast<BookmarksItem*>(itemFromIndex(index));
+	BookmarksItem *bookmark(dynamic_cast<BookmarksItem*>(itemFromIndex(index)));
 
 	if (!bookmark)
 	{
@@ -1079,8 +1110,8 @@ bool BookmarksModel::setData(const QModelIndex &index, const QVariant &value, in
 
 	if (role == UrlRole && value.toUrl() != index.data(UrlRole).toUrl())
 	{
-		const QUrl oldUrl = Utils::normalizeUrl(index.data(UrlRole).toUrl());
-		const QUrl newUrl = Utils::normalizeUrl(value.toUrl());
+		const QUrl oldUrl(Utils::normalizeUrl(index.data(UrlRole).toUrl()));
+		const QUrl newUrl(Utils::normalizeUrl(value.toUrl()));
 
 		if (!oldUrl.isEmpty() && m_urls.contains(oldUrl))
 		{
@@ -1104,8 +1135,8 @@ bool BookmarksModel::setData(const QModelIndex &index, const QVariant &value, in
 	}
 	else if (role == KeywordRole && value.toString() != index.data(KeywordRole).toString())
 	{
-		const QString oldKeyword = index.data(KeywordRole).toString();
-		const QString newKeyword = value.toString();
+		const QString oldKeyword(index.data(KeywordRole).toString());
+		const QString newKeyword(value.toString());
 
 		if (!oldKeyword.isEmpty() && m_keywords.contains(oldKeyword))
 		{
@@ -1119,7 +1150,7 @@ bool BookmarksModel::setData(const QModelIndex &index, const QVariant &value, in
 	}
 	else if (m_mode == NotesMode && role == DescriptionRole)
 	{
-		const QString title = value.toString().section(QLatin1Char('\n'), 0, 0).left(100);
+		const QString title(value.toString().section(QLatin1Char('\n'), 0, 0).left(100));
 
 		setData(index, ((title == value.toString().trimmed()) ? title : title + QStringLiteral("…")), TitleRole);
 	}
