@@ -29,7 +29,7 @@ class FileSystemCompleterModel : public QFileSystemModel
 {
 
 public:
-	explicit FileSystemCompleterModel(QObject *parent = NULL);
+	explicit FileSystemCompleterModel(QObject *parent = nullptr);
 
 	QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;
 };
