@@ -1,7 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
-* Copyright (C) 2015 Piotr Wójcik <chocimier@tlen.pl>
+* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2016 Piotr Wójcik <chocimier@tlen.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -68,7 +68,7 @@ GesturesManager::GestureStep::GestureStep(const QInputEvent *event) : type(event
 		case QEvent::MouseButtonRelease:
 		case QEvent::MouseButtonDblClick:
 			{
-				const QMouseEvent *mouseEvent = static_cast<const QMouseEvent*>(event);
+				const QMouseEvent *mouseEvent(static_cast<const QMouseEvent*>(event));
 
 				if (mouseEvent)
 				{
@@ -79,19 +79,19 @@ GesturesManager::GestureStep::GestureStep(const QInputEvent *event) : type(event
 			break;
 		case QEvent::Wheel:
 			{
-				const QWheelEvent *wheelEvent = static_cast<const QWheelEvent*>(event);
+				const QWheelEvent *wheelEvent(static_cast<const QWheelEvent*>(event));
 
 				if (wheelEvent)
 				{
-					const QPoint move = wheelEvent->angleDelta();
+					const QPoint delta(wheelEvent->angleDelta());
 
-					if (qAbs(move.x()) > qAbs(move.y()))
+					if (qAbs(delta.x()) > qAbs(delta.y()))
 					{
-						direction = (move.x() > 0) ? MouseGestures::MoveRightMouseAction : MouseGestures::MoveLeftMouseAction;
+						direction = (delta.x() > 0) ? MouseGestures::MoveRightMouseAction : MouseGestures::MoveLeftMouseAction;
 					}
-					else if (qAbs(move.y()) > 0)
+					else if (qAbs(delta.y()) > 0)
 					{
-						direction = (move.y() > 0) ? MouseGestures::MoveUpMouseAction : MouseGestures::MoveDownMouseAction;
+						direction = (delta.y() > 0) ? MouseGestures::MoveUpMouseAction : MouseGestures::MoveDownMouseAction;
 					}
 				}
 			}
@@ -102,19 +102,19 @@ GesturesManager::GestureStep::GestureStep(const QInputEvent *event) : type(event
 	}
 }
 
-bool GesturesManager::GestureStep::operator ==(const GestureStep &other)
+bool GesturesManager::GestureStep::operator ==(const GestureStep &other) const
 {
 	return (type == other.type) && (button == other.button) && (direction == other.direction) && (modifiers == other.modifiers || type == QEvent::MouseMove);
 }
 
-bool GesturesManager::GestureStep::operator !=(const GestureStep &other)
+bool GesturesManager::GestureStep::operator !=(const GestureStep &other) const
 {
 	return !((*this) == other);
 }
 
-GesturesManager* GesturesManager::m_instance = NULL;
-MouseGestures::Recognizer* GesturesManager::m_recognizer = NULL;
-QPointer<QObject> GesturesManager::m_trackedObject = NULL;
+GesturesManager* GesturesManager::m_instance(nullptr);
+MouseGestures::Recognizer* GesturesManager::m_recognizer(nullptr);
+QPointer<QObject> GesturesManager::m_trackedObject(nullptr);
 QPoint GesturesManager::m_lastClick;
 QPoint GesturesManager::m_lastPosition;
 QVariantMap GesturesManager::m_paramaters;
@@ -123,17 +123,13 @@ QHash<GesturesManager::GesturesContext, QList<QList<GesturesManager::GestureStep
 QList<QInputEvent*> GesturesManager::m_events;
 QList<GesturesManager::GestureStep> GesturesManager::m_steps;
 QList<GesturesManager::GesturesContext> GesturesManager::m_contexts;
-bool GesturesManager::m_isReleasing = false;
-bool GesturesManager::m_afterScroll = false;
+bool GesturesManager::m_isReleasing(false);
+bool GesturesManager::m_afterScroll(false);
 
 GesturesManager::GesturesManager(QObject *parent) : QObject(parent),
 	m_reloadTimer(0)
 {
-	m_instance = this;
-
-	loadProfiles();
-
-	connect(SettingsManager::getInstance(), SIGNAL(valueChanged(QString,QVariant)), m_instance, SLOT(optionChanged(QString)));
+	connect(SettingsManager::getInstance(), SIGNAL(valueChanged(int,QVariant)), this, SLOT(optionChanged(int)));
 }
 
 void GesturesManager::createInstance(QObject *parent)
@@ -141,19 +137,19 @@ void GesturesManager::createInstance(QObject *parent)
 	if (!m_instance)
 	{
 		QList<QList<GestureStep> > generic;
-		generic.append(QList<GestureStep>() << GestureStep(QEvent::MouseButtonDblClick, Qt::LeftButton));
-		generic.append(QList<GestureStep>() << GestureStep(QEvent::MouseButtonPress, Qt::LeftButton) << GestureStep(QEvent::MouseButtonRelease, Qt::LeftButton));
-		generic.append(QList<GestureStep>() << GestureStep(QEvent::MouseButtonPress, Qt::LeftButton) << GestureStep(QEvent::MouseMove, MouseGestures::UnknownMouseAction));
+		generic.append(QList<GestureStep>({GestureStep(QEvent::MouseButtonDblClick, Qt::LeftButton)}));
+		generic.append(QList<GestureStep>({GestureStep(QEvent::MouseButtonPress, Qt::LeftButton), GestureStep(QEvent::MouseButtonRelease, Qt::LeftButton)}));
+		generic.append(QList<GestureStep>({GestureStep(QEvent::MouseButtonPress, Qt::LeftButton), GestureStep(QEvent::MouseMove, MouseGestures::UnknownMouseAction)}));
 
 		QList<QList<GestureStep> > link;
-		link.append(QList<GestureStep>() << GestureStep(QEvent::MouseButtonPress, Qt::LeftButton) << GestureStep(QEvent::MouseButtonRelease, Qt::LeftButton));
-		link.append(QList<GestureStep>() << GestureStep(QEvent::MouseButtonPress, Qt::LeftButton) << GestureStep(QEvent::MouseMove, MouseGestures::UnknownMouseAction));
+		link.append(QList<GestureStep>({GestureStep(QEvent::MouseButtonPress, Qt::LeftButton), GestureStep(QEvent::MouseButtonRelease, Qt::LeftButton)}));
+		link.append(QList<GestureStep>({GestureStep(QEvent::MouseButtonPress, Qt::LeftButton), GestureStep(QEvent::MouseMove, MouseGestures::UnknownMouseAction)}));
 
 		QList<QList<GestureStep> > contentEditable;
-		contentEditable.append(QList<GestureStep>() << GestureStep(QEvent::MouseButtonPress, Qt::MiddleButton));
+		contentEditable.append(QList<GestureStep>({GestureStep(QEvent::MouseButtonPress, Qt::MiddleButton)}));
 
 		QList<QList<GestureStep> > tabHandle;
-		tabHandle.append(QList<GestureStep>() << GestureStep(QEvent::MouseButtonPress, Qt::LeftButton) << GestureStep(QEvent::MouseMove, MouseGestures::UnknownMouseAction));
+		tabHandle.append(QList<GestureStep>({GestureStep(QEvent::MouseButtonPress, Qt::LeftButton), GestureStep(QEvent::MouseMove, MouseGestures::UnknownMouseAction)}));
 
 		m_nativeGestures[GesturesManager::GenericGesturesContext] = generic;
 		m_nativeGestures[GesturesManager::LinkGesturesContext] = link;
@@ -161,6 +157,8 @@ void GesturesManager::createInstance(QObject *parent)
 		m_nativeGestures[GesturesManager::TabHandleGesturesContext] = tabHandle;
 
 		m_instance = new GesturesManager(parent);
+
+		loadProfiles();
 	}
 }
 
@@ -176,9 +174,9 @@ void GesturesManager::timerEvent(QTimerEvent *event)
 	}
 }
 
-void GesturesManager::optionChanged(const QString &option)
+void GesturesManager::optionChanged(int identifier)
 {
-	if ((option == QLatin1String("Browser/MouseProfilesOrder") || option == QLatin1String("Browser/EnableMouseGestures")) && m_reloadTimer == 0)
+	if ((identifier == SettingsManager::Browser_MouseProfilesOrderOption || identifier == SettingsManager::Browser_EnableMouseGesturesOption) && m_reloadTimer == 0)
 	{
 		m_reloadTimer = startTimer(250);
 	}
@@ -189,28 +187,28 @@ void GesturesManager::loadProfiles()
 	m_gestures.clear();
 
 	MouseGesture contextMenuGestureDefinition;
-	contextMenuGestureDefinition.steps << GestureStep(QEvent::MouseButtonPress, Qt::RightButton) << GestureStep(QEvent::MouseButtonRelease, Qt::RightButton);
+	contextMenuGestureDefinition.steps = QList<GestureStep>({GestureStep(QEvent::MouseButtonPress, Qt::RightButton), GestureStep(QEvent::MouseButtonRelease, Qt::RightButton)});
 	contextMenuGestureDefinition.action = ActionsManager::ContextMenuAction;
 
 	for (int i = (UnknownGesturesContext + 1); i < OtherGesturesContext; ++i)
 	{
-		GesturesContext context = static_cast<GesturesContext>(i);
+		GesturesContext context(static_cast<GesturesContext>(i));
 
 		m_gestures[context] = QVector<MouseGesture>();
 		m_gestures[context].append(contextMenuGestureDefinition);
 	}
 
-	const QStringList gestureProfiles = SettingsManager::getValue(QLatin1String("Browser/MouseProfilesOrder")).toStringList();
-	const bool enableMoves = SettingsManager::getValue(QLatin1String("Browser/EnableMouseGestures")).toBool();
+	const QStringList gestureProfiles(SettingsManager::getValue(SettingsManager::Browser_MouseProfilesOrderOption).toStringList());
+	const bool areMouseGesturesEnabled(SettingsManager::getValue(SettingsManager::Browser_EnableMouseGesturesOption).toBool());
 
 	for (int i = 0; i < gestureProfiles.count(); ++i)
 	{
 		Settings profile(SessionsManager::getReadableDataPath(QLatin1String("mouse/") + gestureProfiles.at(i) + QLatin1String(".ini")));
-		const QStringList contexts = profile.getGroups();
+		const QStringList contexts(profile.getGroups());
 
 		for (int j = 0; j < contexts.count(); ++j)
 		{
-			GesturesContext context = UnknownGesturesContext;
+			GesturesContext context(UnknownGesturesContext);
 
 			if (contexts.at(j) == QLatin1String("Generic"))
 			{
@@ -224,6 +222,10 @@ void GesturesManager::loadProfiles()
 			{
 				context = ContentEditableGesturesContext;
 			}
+			else if (contexts.at(j) == QLatin1String("ToolBar"))
+			{
+				context = ToolBarGesturesContext;
+			}
 			else if (contexts.at(j) == QLatin1String("TabHandle"))
 			{
 				context = TabHandleGesturesContext;
@@ -245,20 +247,27 @@ void GesturesManager::loadProfiles()
 
 			profile.beginGroup(contexts.at(j));
 
-			const QStringList gestures = profile.getKeys();
+			const QStringList gestures(profile.getKeys());
 
 			for (int k = 0; k < gestures.count(); ++k)
 			{
-				const QStringList rawMouseActions = gestures.at(k).split(QLatin1Char(','));
-				const int action = ActionsManager::getActionIdentifier(profile.getValue(gestures.at(k), QString()).toString());
+				const QStringList rawMouseActions(gestures.at(k).split(QLatin1Char(',')));
+				int action(ActionsManager::getActionIdentifier(profile.getValue(gestures.at(k), QString()).toString()));
 
 				if (action < 0 || rawMouseActions.isEmpty())
 				{
-					continue;
+					if (profile.getValue(gestures.at(k)) == QLatin1String("NoAction"))
+					{
+						action = NATIVE_GESTURE;
+					}
+					else
+					{
+						continue;
+					}
 				}
 
 				QList<GestureStep> steps;
-				bool hasMove = false;
+				bool hasMove(false);
 
 				for (int l = 0; l < rawMouseActions.count(); ++l)
 				{
@@ -270,7 +279,7 @@ void GesturesManager::loadProfiles()
 					}
 				}
 
-				if (!steps.empty() && !(!enableMoves && hasMove))
+				if (!steps.empty() && (!hasMove || areMouseGesturesEnabled))
 				{
 					MouseGesture definition;
 					definition.steps = steps;
@@ -294,7 +303,7 @@ void GesturesManager::releaseObject()
 		disconnect(m_trackedObject, SIGNAL(destroyed(QObject*)), m_instance, SLOT(endGesture()));
 	}
 
-	m_trackedObject = NULL;
+	m_trackedObject = nullptr;
 }
 
 void GesturesManager::endGesture()
@@ -316,8 +325,8 @@ GesturesManager* GesturesManager::getInstance()
 GesturesManager::GestureStep GesturesManager::deserializeStep(const QString &string)
 {
 	GestureStep step;
-	const QStringList parts = string.split(QLatin1Char('+'));
-	const QString &event = parts.first();
+	const QStringList parts(string.split(QLatin1Char('+')));
+	const QString event(parts.first());
 
 	if (event.startsWith(QLatin1String("press")))
 	{
@@ -395,7 +404,7 @@ GesturesManager::GestureStep GesturesManager::deserializeStep(const QString &str
 		}
 		else
 		{
-			int number = QRegularExpression(QLatin1String("Extra(\\d{1,2})$")).match(event).captured(1).toInt();
+			const int number(QRegularExpression(QLatin1String("Extra(\\d{1,2})$")).match(event).captured(1).toInt());
 
 			if (1 <= number && number <= 24)
 			{
@@ -442,16 +451,15 @@ QList<GesturesManager::GestureStep> GesturesManager::recognizeMoveStep(QInputEve
 	{
 		for (int j = 0; j < m_gestures[m_contexts[i]].count(); ++j)
 		{
-			QList<GestureStep> &steps = m_gestures[m_contexts[i]][j].steps;
+			QList<GestureStep> steps(m_gestures[m_contexts[i]][j].steps);
 
 			if (steps.count() > m_steps.count() && steps[m_steps.count()].type == QEvent::MouseMove && steps.mid(0, m_steps.count()) == m_steps)
 			{
-
 				MouseGestures::ActionList moves;
 
-				for (int j = m_steps.count(); j < steps.count() && steps[j].type == QEvent::MouseMove; ++j)
+				for (int k = m_steps.count(); k < steps.count() && steps[k].type == QEvent::MouseMove; ++k)
 				{
-					moves.push_back(steps[j].direction);
+					moves.push_back(steps[k].direction);
 				}
 
 				if (!moves.empty())
@@ -462,15 +470,15 @@ QList<GesturesManager::GestureStep> GesturesManager::recognizeMoveStep(QInputEve
 		}
 	}
 
-	const QMouseEvent *mouseEvent = static_cast<const QMouseEvent*>(event);
+	const QMouseEvent *mouseEvent(static_cast<const QMouseEvent*>(event));
 
 	if (mouseEvent)
 	{
 		m_recognizer->addPosition(mouseEvent->pos().x(), mouseEvent->pos().y());
 	}
 
-	const int gesture = m_recognizer->endGesture();
-	const MouseGestures::ActionList moves = possibleMoves.value(gesture);
+	const int gesture(m_recognizer->endGesture());
+	const MouseGestures::ActionList moves(possibleMoves.value(gesture));
 	MouseGestures::ActionList::const_iterator iterator;
 
 	for (iterator = moves.begin(); iterator != moves.end(); ++iterator)
@@ -488,9 +496,9 @@ QList<GesturesManager::GestureStep> GesturesManager::recognizeMoveStep(QInputEve
 
 int GesturesManager::matchGesture()
 {
-	int bestGesture = 0;
-	int lowestDifference = std::numeric_limits<int>::max();
-	int difference = 0;
+	int bestGesture(0);
+	int lowestDifference(std::numeric_limits<int>::max());
+	int difference(0);
 
 	for (int i = 0; i < m_contexts.count(); ++i)
 	{
@@ -532,8 +540,8 @@ int GesturesManager::matchGesture()
 
 int GesturesManager::getLastMoveDistance(bool measureFinished)
 {
-	int result = 0;
-	int index = m_events.count() - 1;
+	int result(0);
+	int index(m_events.count() - 1);
 
 	if (!measureFinished && (index < 0 || m_events[index]->type() != QEvent::MouseMove))
 	{
@@ -547,8 +555,8 @@ int GesturesManager::getLastMoveDistance(bool measureFinished)
 
 	for (; index > 0 && m_events[index - 1]->type() == QEvent::MouseMove; --index)
 	{
-		QMouseEvent *current = static_cast<QMouseEvent*>(m_events[index]);
-		QMouseEvent *previous = static_cast<QMouseEvent*>(m_events[index - 1]);
+		QMouseEvent *current(static_cast<QMouseEvent*>(m_events[index]));
+		QMouseEvent *previous(static_cast<QMouseEvent*>(m_events[index - 1]));
 
 		if (current && previous)
 		{
@@ -570,11 +578,11 @@ int GesturesManager::gesturesDifference(QList<GestureStep> defined)
 		return std::numeric_limits<int>::max();
 	}
 
-	int difference = 0;
+	int difference(0);
 
 	for (int j = 0; j < defined.count(); ++j)
 	{
-		int stepDifference = 0;
+		int stepDifference(0);
 
 		if (j == (defined.count() - 1) && defined[j].type == QEvent::MouseButtonPress && m_steps[j].type == QEvent::MouseButtonDblClick && defined[j].button == m_steps[j].button && defined[j].modifiers == m_steps[j].modifiers)
 		{
@@ -617,7 +625,7 @@ int GesturesManager::gesturesDifference(QList<GestureStep> defined)
 
 bool GesturesManager::startGesture(QObject *object, QEvent *event, QList<GesturesContext> contexts, const QVariantMap &parameters)
 {
-	QInputEvent *inputEvent = static_cast<QInputEvent*>(event);
+	QInputEvent *inputEvent(static_cast<QInputEvent*>(event));
 
 	if (!object || !inputEvent || m_gestures.keys().toSet().intersect(contexts.toSet()).isEmpty() || m_events.contains(inputEvent))
 	{
@@ -690,20 +698,20 @@ bool GesturesManager::triggerAction(int gestureIdentifier)
 
 bool GesturesManager::isTracking()
 {
-	return (m_trackedObject != NULL);
+	return (m_trackedObject != nullptr);
 }
 
 bool GesturesManager::eventFilter(QObject *object, QEvent *event)
 {
-	QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
-	QWheelEvent *wheelEvent = static_cast<QWheelEvent*>(event);
+	QMouseEvent *mouseEvent(static_cast<QMouseEvent*>(event));
+	QWheelEvent *wheelEvent(static_cast<QWheelEvent*>(event));
 
 	if (!mouseEvent && !wheelEvent)
 	{
 		return QObject::eventFilter(object, event);
 	}
 
-	int gesture = UNKNOWN_GESTURE;
+	int gesture(UNKNOWN_GESTURE);
 
 	switch (event->type())
 	{
@@ -712,7 +720,7 @@ bool GesturesManager::eventFilter(QObject *object, QEvent *event)
 		case QEvent::MouseButtonDblClick:
 			if (mouseEvent && !m_events.isEmpty() && m_events.last()->type() == event->type())
 			{
-				QMouseEvent *previousMouseEvent = static_cast<QMouseEvent*>(m_events.last());
+				QMouseEvent *previousMouseEvent(static_cast<QMouseEvent*>(m_events.last()));
 
 				if (previousMouseEvent && previousMouseEvent->button() == mouseEvent->button() && previousMouseEvent->modifiers() == mouseEvent->modifiers())
 				{
@@ -752,7 +760,7 @@ bool GesturesManager::eventFilter(QObject *object, QEvent *event)
 			{
 				delete m_recognizer;
 
-				m_recognizer = NULL;
+				m_recognizer = nullptr;
 			}
 
 			gesture = matchGesture();
@@ -810,18 +818,23 @@ bool GesturesManager::eventFilter(QObject *object, QEvent *event)
 			{
 				delete m_recognizer;
 
-				m_recognizer = NULL;
+				m_recognizer = nullptr;
 			}
 
 			gesture = matchGesture();
 
 			triggerAction(gesture);
 
-			while (m_steps.count() && m_steps[m_steps.count() - 1].type == QEvent::Wheel)
+			while (!m_steps.isEmpty() && m_steps[m_steps.count() - 1].type == QEvent::Wheel)
 			{
 				m_steps.removeAt(m_steps.count() - 1);
 			}
 
+			while (!m_events.isEmpty() && m_events[m_events.count() - 1]->type() == QEvent::Wheel)
+			{
+				m_events.removeAt(m_events.count() - 1);
+			}
+
 			m_afterScroll = true;
 
 			break;
