@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -18,8 +18,11 @@
 **************************************************************************/
 
 #include "SearchEnginesManager.h"
+#include "NetworkManager.h"
+#include "NetworkManagerFactory.h"
 #include "SessionsManager.h"
 #include "SettingsManager.h"
+#include "ThemesManager.h"
 #include "Utils.h"
 
 #include <QtCore/QBuffer>
@@ -32,12 +35,12 @@
 namespace Otter
 {
 
-SearchEnginesManager* SearchEnginesManager::m_instance = NULL;
-QStandardItemModel* SearchEnginesManager::m_searchEnginesModel = NULL;
+SearchEnginesManager* SearchEnginesManager::m_instance(nullptr);
+QStandardItemModel* SearchEnginesManager::m_searchEnginesModel(nullptr);
 QStringList SearchEnginesManager::m_searchEnginesOrder;
 QStringList SearchEnginesManager::m_searchKeywords;
 QHash<QString, SearchEnginesManager::SearchEngineDefinition> SearchEnginesManager::m_searchEngines;
-bool SearchEnginesManager::m_isInitialized = false;
+bool SearchEnginesManager::m_isInitialized(false);
 
 SearchEnginesManager::SearchEnginesManager(QObject *parent) : QObject(parent)
 {
@@ -59,13 +62,13 @@ void SearchEnginesManager::initialize()
 
 		loadSearchEngines();
 
-		connect(SettingsManager::getInstance(), SIGNAL(valueChanged(QString,QVariant)), m_instance, SLOT(optionChanged(QString)));
+		connect(SettingsManager::getInstance(), SIGNAL(valueChanged(int,QVariant)), m_instance, SLOT(optionChanged(int)));
 	}
 }
 
-void SearchEnginesManager::optionChanged(const QString &key)
+void SearchEnginesManager::optionChanged(int identifier)
 {
-	if (key == QLatin1String("Search/SearchEnginesOrder"))
+	if (identifier == SettingsManager::Search_SearchEnginesOrderOption)
 	{
 		loadSearchEngines();
 	}
@@ -76,9 +79,9 @@ void SearchEnginesManager::loadSearchEngines()
 	m_searchEngines.clear();
 	m_searchKeywords.clear();
 
-	m_searchEnginesOrder = SettingsManager::getValue(QLatin1String("Search/SearchEnginesOrder")).toStringList();
+	m_searchEnginesOrder = SettingsManager::getValue(SettingsManager::Search_SearchEnginesOrderOption).toStringList();
 
-	const QStringList searchEnginesOrder = m_searchEnginesOrder;
+	const QStringList searchEnginesOrder(m_searchEnginesOrder);
 
 	for (int i = 0; i < searchEnginesOrder.count(); ++i)
 	{
@@ -91,7 +94,7 @@ void SearchEnginesManager::loadSearchEngines()
 			continue;
 		}
 
-		const SearchEngineDefinition searchEngine = loadSearchEngine(&file, searchEnginesOrder.at(i));
+		SearchEngineDefinition searchEngine(loadSearchEngine(&file, searchEnginesOrder.at(i), true));
 
 		file.close();
 
@@ -108,29 +111,33 @@ void SearchEnginesManager::loadSearchEngines()
 	emit m_instance->searchEnginesModified();
 
 	updateSearchEnginesModel();
+	updateSearchEnginesOptions();
 }
 
 void SearchEnginesManager::addSearchEngine(const SearchEngineDefinition &searchEngine, bool isDefault)
 {
-	if (saveSearchEngine(searchEngine))
+	if (!saveSearchEngine(searchEngine))
 	{
-		if (isDefault)
-		{
-			SettingsManager::setValue(QLatin1String("Search/DefaultSearchEngine"), searchEngine.identifier);
-		}
+		return;
+	}
 
-		if (m_searchEnginesOrder.contains(searchEngine.identifier))
-		{
-			emit m_instance->searchEnginesModified();
+	if (isDefault)
+	{
+		SettingsManager::setValue(SettingsManager::Search_DefaultSearchEngineOption, searchEngine.identifier);
+	}
 
-			updateSearchEnginesModel();
-		}
-		else
-		{
-			m_searchEnginesOrder.append(searchEngine.identifier);
+	if (m_searchEnginesOrder.contains(searchEngine.identifier))
+	{
+		emit m_instance->searchEnginesModified();
 
-			SettingsManager::setValue(QLatin1String("Search/SearchEnginesOrder"), m_searchEnginesOrder);
-		}
+		updateSearchEnginesModel();
+		updateSearchEnginesOptions();
+	}
+	else
+	{
+		m_searchEnginesOrder.append(searchEngine.identifier);
+
+		SettingsManager::setValue(SettingsManager::Search_SearchEnginesOrderOption, m_searchEnginesOrder);
 	}
 }
 
@@ -143,18 +150,19 @@ void SearchEnginesManager::updateSearchEnginesModel()
 
 	m_searchEnginesModel->clear();
 
-	const QStringList searchEngines = getSearchEngines();
+	const QStringList searchEngines(getSearchEngines());
 
 	for (int i = 0; i < searchEngines.count(); ++i)
 	{
-		const SearchEngineDefinition search = getSearchEngine(searchEngines.at(i));
+		const SearchEngineDefinition search(getSearchEngine(searchEngines.at(i)));
 
 		if (!search.identifier.isEmpty())
 		{
-			QStandardItem *item = new QStandardItem((search.icon.isNull() ? Utils::getIcon(QLatin1String("edit-find")) : search.icon), QString());
-			item->setData(search.title, Qt::UserRole);
-			item->setData(search.identifier, (Qt::UserRole + 1));
-			item->setData(search.keyword, (Qt::UserRole + 2));
+			QStandardItem *item(new QStandardItem((search.icon.isNull() ? ThemesManager::getIcon(QLatin1String("edit-find")) : search.icon), QString()));
+			item->setData(search.title, TitleRole);
+			item->setData(search.identifier, IdentifierRole);
+			item->setData(search.keyword, KeywordRole);
+			item->setFlags(item->flags() | Qt::ItemNeverHasChildren);
 
 			m_searchEnginesModel->appendRow(item);
 		}
@@ -162,12 +170,13 @@ void SearchEnginesManager::updateSearchEnginesModel()
 
 	if (searchEngines.count() > 0)
 	{
-		QStandardItem *separatorItem = new QStandardItem();
+		QStandardItem *separatorItem(new QStandardItem());
 		separatorItem->setData(QLatin1String("separator"), Qt::AccessibleDescriptionRole);
-		separatorItem->setData(QSize(-1, 10), Qt::SizeHintRole);
+		separatorItem->setFlags(Qt::ItemIsEnabled | Qt::ItemNeverHasChildren);
 
-		QStandardItem *manageItem = new QStandardItem(Utils::getIcon(QLatin1String("configure")), tr("Manage Search Engines…"));
+		QStandardItem *manageItem(new QStandardItem(ThemesManager::getIcon(QLatin1String("configure")), tr("Manage Search Engines…")));
 		manageItem->setData(QLatin1String("configure"), Qt::AccessibleDescriptionRole);
+		manageItem->setFlags(manageItem->flags() | Qt::ItemNeverHasChildren);
 
 		m_searchEnginesModel->appendRow(separatorItem);
 		m_searchEnginesModel->appendRow(manageItem);
@@ -176,6 +185,19 @@ void SearchEnginesManager::updateSearchEnginesModel()
 	emit m_instance->searchEnginesModelModified();
 }
 
+void SearchEnginesManager::updateSearchEnginesOptions()
+{
+	const QStringList searchEngines(m_searchEngines.keys());
+	SettingsManager::OptionDefinition defaultQuickSearchEngineOption(SettingsManager::getOptionDefinition(SettingsManager::Search_DefaultQuickSearchEngineOption));
+	defaultQuickSearchEngineOption.choices = searchEngines;
+
+	SettingsManager::OptionDefinition defaultSearchEngineOption(SettingsManager::getOptionDefinition(SettingsManager::Search_DefaultSearchEngineOption));
+	defaultSearchEngineOption.choices = searchEngines;
+
+	SettingsManager::updateOptionDefinition(SettingsManager::Search_DefaultQuickSearchEngineOption, defaultQuickSearchEngineOption);
+	SettingsManager::updateOptionDefinition(SettingsManager::Search_DefaultSearchEngineOption, defaultSearchEngineOption);
+}
+
 void SearchEnginesManager::setupQuery(const QString &query, const SearchUrl &searchUrl, QNetworkRequest *request, QNetworkAccessManager::Operation *method, QByteArray *body)
 {
 	if (searchUrl.url.isEmpty())
@@ -183,7 +205,7 @@ void SearchEnginesManager::setupQuery(const QString &query, const SearchUrl &sea
 		return;
 	}
 
-	QString urlString = searchUrl.url;
+	QString urlString(searchUrl.url);
 	QHash<QString, QString> values;
 	values[QLatin1String("searchTerms")] = query;
 	values[QLatin1String("count")] = QString();
@@ -205,11 +227,11 @@ void SearchEnginesManager::setupQuery(const QString &query, const SearchUrl &sea
 	QUrl url(urlString);
 	QUrlQuery getQuery(url);
 	QUrlQuery postQuery;
-	const QList<QPair<QString, QString> > parameters = searchUrl.parameters.queryItems(QUrl::FullyDecoded);
+	const QList<QPair<QString, QString> > parameters(searchUrl.parameters.queryItems(QUrl::FullyDecoded));
 
 	for (int i = 0; i < parameters.count(); ++i)
 	{
-		QString value = parameters.at(i).second;
+		QString value(parameters.at(i).second);
 
 		for (valuesIterator = values.begin(); valuesIterator != values.end(); ++valuesIterator)
 		{
@@ -229,12 +251,12 @@ void SearchEnginesManager::setupQuery(const QString &query, const SearchUrl &sea
 			else if (searchUrl.enctype == QLatin1String("multipart/form-data"))
 			{
 				QString encodedValue;
-				QByteArray plainValue = value.toUtf8();
+				QByteArray plainValue(value.toUtf8());
 				const char hex[] = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
 
 				for (int j = 0; j < plainValue.length(); ++j)
 				{
-					const char character = plainValue[j];
+					const char character(plainValue[j]);
 
 					if (character == 32 || (character >= 33 && character <= 126 && character != 61))
 					{
@@ -275,7 +297,7 @@ void SearchEnginesManager::setupQuery(const QString &query, const SearchUrl &sea
 	request->setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::AlwaysNetwork);
 }
 
-SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::loadSearchEngine(QIODevice *device, const QString &identifier)
+SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::loadSearchEngine(QIODevice *device, const QString &identifier, bool checkKeyword)
 {
 	SearchEngineDefinition searchEngine;
 	searchEngine.identifier = identifier;
@@ -284,7 +306,7 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::loadSearchEng
 
 	if (reader.readNextStartElement() && reader.name() == QLatin1String("OpenSearchDescription"))
 	{
-		SearchUrl *currentUrl = NULL;
+		SearchUrl *currentUrl(nullptr);
 
 		while (!reader.atEnd())
 		{
@@ -297,6 +319,8 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::loadSearchEng
 					if (reader.attributes().value(QLatin1String("rel")) == QLatin1String("self") || reader.attributes().value(QLatin1String("type")) == QLatin1String("application/opensearchdescription+xml"))
 					{
 						searchEngine.selfUrl = QUrl(reader.attributes().value(QLatin1String("template")).toString());
+
+						currentUrl = nullptr;
 					}
 					else if (reader.attributes().value(QLatin1String("rel")) == QLatin1String("suggestions") || reader.attributes().value(QLatin1String("type")) == QLatin1String("application/x-suggestions+json"))
 					{
@@ -308,7 +332,7 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::loadSearchEng
 					}
 					else
 					{
-						currentUrl = NULL;
+						currentUrl = nullptr;
 					}
 
 					if (currentUrl)
@@ -324,13 +348,20 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::loadSearchEng
 				}
 				else if (reader.name() == QLatin1String("Shortcut"))
 				{
-					const QString keyword = reader.readElementText();
+					const QString keyword(reader.readElementText());
 
-					if (!keyword.isEmpty() && !m_searchKeywords.contains(keyword))
+					if (!keyword.isEmpty())
 					{
-						searchEngine.keyword = keyword;
+						if (!m_searchKeywords.contains(keyword))
+						{
+							searchEngine.keyword = keyword;
 
-						m_searchKeywords.append(keyword);
+							m_searchKeywords.append(keyword);
+						}
+						else if (!checkKeyword)
+						{
+							searchEngine.keyword = keyword;
+						}
 					}
 				}
 				else if (reader.name() == QLatin1String("ShortName"))
@@ -347,9 +378,16 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::loadSearchEng
 				}
 				else if (reader.name() == QLatin1String("Image"))
 				{
-					const QString data = reader.readElementText();
+					const QString data(reader.readElementText());
 
-					searchEngine.icon = QIcon(QPixmap::fromImage(QImage::fromData(QByteArray::fromBase64(data.mid(data.indexOf(QLatin1String("base64,")) + 7).toUtf8()), "png")));
+					if (data.startsWith(QLatin1String("data:image")))
+					{
+						searchEngine.icon = QIcon(QPixmap::fromImage(QImage::fromData(QByteArray::fromBase64(data.mid(data.indexOf(QLatin1String("base64,")) + 7).toUtf8()), "png")));
+					}
+					else
+					{
+						searchEngine.iconUrl = QUrl(data);
+					}
 				}
 				else if (reader.name() == QLatin1String("SearchForm"))
 				{
@@ -415,7 +453,7 @@ SearchEnginesManager::SearchEngineDefinition SearchEnginesManager::getSearchEngi
 
 	if (identifier.isEmpty())
 	{
-		return m_searchEngines.value(SettingsManager::getValue(QLatin1String("Search/DefaultSearchEngine")).toString(), SearchEngineDefinition());
+		return m_searchEngines.value(SettingsManager::getValue(SettingsManager::Search_DefaultSearchEngineOption).toString(), SearchEngineDefinition());
 	}
 
 	return m_searchEngines.value(identifier, SearchEngineDefinition());
@@ -441,13 +479,40 @@ QStringList SearchEnginesManager::getSearchKeywords()
 	return m_searchKeywords;
 }
 
-bool SearchEnginesManager::saveSearchEngine(const SearchEngineDefinition &searchEngine)
+bool SearchEnginesManager::hasSearchEngine(const QUrl &url)
 {
-	if (searchEngine.identifier.isEmpty())
+	if (!url.isValid())
 	{
 		return false;
 	}
 
+	if (!m_isInitialized)
+	{
+		initialize();
+	}
+
+	QHash<QString, SearchEngineDefinition>::iterator iterator;
+
+	for (iterator = m_searchEngines.begin(); iterator != m_searchEngines.end(); ++iterator)
+	{
+		if (iterator.value().selfUrl == url)
+		{
+			return true;
+		}
+	}
+
+	return false;
+}
+
+bool SearchEnginesManager::saveSearchEngine(const SearchEngineDefinition &searchEngine)
+{
+	if (SessionsManager::isReadOnly() || searchEngine.identifier.isEmpty())
+	{
+		return false;
+	}
+
+	QDir().mkpath(SessionsManager::getWritableDataPath(QLatin1String("searches")));
+
 	QFile file(SessionsManager::getWritableDataPath(QLatin1String("searches/") + searchEngine.identifier + QLatin1String(".xml")));
 
 	if (!file.open(QFile::WriteOnly))
@@ -474,7 +539,7 @@ bool SearchEnginesManager::saveSearchEngine(const SearchEngineDefinition &search
 
 	if (!searchEngine.icon.isNull())
 	{
-		const QSize size = searchEngine.icon.availableSizes().value(0, QSize(16, 16));
+		const QSize size(searchEngine.icon.availableSizes().value(0, QSize(16, 16)));
 		QByteArray data;
 		QBuffer buffer(&data);
 		buffer.open(QIODevice::WriteOnly);
@@ -507,7 +572,7 @@ bool SearchEnginesManager::saveSearchEngine(const SearchEngineDefinition &search
 		writer.writeAttribute(QLatin1String("enctype"), searchEngine.resultsUrl.enctype.toLower());
 		writer.writeAttribute(QLatin1String("template"), searchEngine.resultsUrl.url);
 
-		const QList<QPair<QString, QString> > parameters = searchEngine.resultsUrl.parameters.queryItems();
+		const QList<QPair<QString, QString> > parameters(searchEngine.resultsUrl.parameters.queryItems());
 
 		for (int i = 0; i < parameters.count(); ++i)
 		{
@@ -529,7 +594,7 @@ bool SearchEnginesManager::saveSearchEngine(const SearchEngineDefinition &search
 		writer.writeAttribute(QLatin1String("enctype"), searchEngine.suggestionsUrl.enctype.toLower());
 		writer.writeAttribute(QLatin1String("template"), searchEngine.suggestionsUrl.url);
 
-		const QList<QPair<QString, QString> > parameters = searchEngine.suggestionsUrl.parameters.queryItems();
+		const QList<QPair<QString, QString> > parameters(searchEngine.suggestionsUrl.parameters.queryItems());
 
 		for (int i = 0; i < parameters.count(); ++i)
 		{
@@ -555,7 +620,7 @@ bool SearchEnginesManager::saveSearchEngine(const SearchEngineDefinition &search
 
 bool SearchEnginesManager::setupSearchQuery(const QString &query, const QString &searchEngine, QNetworkRequest *request, QNetworkAccessManager::Operation *method, QByteArray *body)
 {
-	const SearchEngineDefinition search = getSearchEngine(searchEngine);
+	const SearchEngineDefinition search(getSearchEngine(searchEngine));
 
 	if (search.identifier.isEmpty())
 	{
@@ -567,4 +632,118 @@ bool SearchEnginesManager::setupSearchQuery(const QString &query, const QString
 	return true;
 }
 
+SearchEngineFetchJob::SearchEngineFetchJob(const QUrl &url, const QString &identifier, bool saveSearchEngine, QObject *parent) : QObject(parent),
+	m_reply(nullptr),
+	m_isFetchingIcon(false),
+	m_needsToSaveSearchEngine(saveSearchEngine)
+{
+	m_searchEngine.identifier = (identifier.isEmpty() ? Utils::createIdentifier(QString(), SearchEnginesManager::getSearchEngines()) : identifier);
+
+	QNetworkRequest request(url);
+	request.setHeader(QNetworkRequest::UserAgentHeader, NetworkManagerFactory::getUserAgent());
+
+	m_reply = NetworkManagerFactory::getNetworkManager()->get(request);
+
+	connect(m_reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(handleRequestFailed()));
+	connect(m_reply, SIGNAL(finished()), this, SLOT(handleRequestFinished()));
+}
+
+SearchEngineFetchJob::~SearchEngineFetchJob()
+{
+	m_reply->deleteLater();
+}
+
+void SearchEngineFetchJob::cancel()
+{
+	if (m_reply)
+	{
+		m_reply->abort();
+	}
+
+	emit jobFinished(false);
+}
+
+void SearchEngineFetchJob::handleRequestFailed()
+{
+	deleteLater();
+
+	if (m_isFetchingIcon)
+	{
+		SearchEnginesManager::addSearchEngine(m_searchEngine);
+	}
+
+	emit jobFinished(m_isFetchingIcon);
+}
+
+void SearchEngineFetchJob::handleRequestFinished()
+{
+	if (m_isFetchingIcon)
+	{
+		QPixmap pixmap;
+
+		if (pixmap.loadFromData(m_reply->readAll()))
+		{
+			m_searchEngine.icon = QIcon(pixmap);
+		}
+
+		if (m_needsToSaveSearchEngine)
+		{
+			SearchEnginesManager::addSearchEngine(m_searchEngine);
+		}
+
+		deleteLater();
+
+		emit jobFinished(true);
+
+		return;
+	}
+
+	m_searchEngine = SearchEnginesManager::loadSearchEngine(m_reply, m_searchEngine.identifier);
+
+	if (m_searchEngine.identifier.isEmpty())
+	{
+		deleteLater();
+
+		emit jobFinished(false);
+
+		return;
+	}
+
+	if (m_searchEngine.selfUrl.isEmpty())
+	{
+		m_searchEngine.selfUrl = m_reply->request().url();
+	}
+
+	if (m_searchEngine.iconUrl.isValid())
+	{
+		m_isFetchingIcon = true;
+
+		m_reply->deleteLater();
+
+		QNetworkRequest request(m_searchEngine.iconUrl);
+		request.setHeader(QNetworkRequest::UserAgentHeader, NetworkManagerFactory::getUserAgent());
+
+		m_reply = NetworkManagerFactory::getNetworkManager()->get(request);
+
+		connect(m_reply, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(handleRequestFailed()));
+		connect(m_reply, SIGNAL(finished()), this, SLOT(handleRequestFinished()));
+	}
+	else
+	{
+		if (m_needsToSaveSearchEngine)
+		{
+			SearchEnginesManager::addSearchEngine(m_searchEngine);
+		}
+
+		deleteLater();
+
+		emit jobFinished(true);
+	}
+}
+
+SearchEnginesManager::SearchEngineDefinition SearchEngineFetchJob::getSearchEngine() const
+{
+	return m_searchEngine;
+}
+
 }
