@@ -1,7 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
-* Copyright (C) 2014 - 2015 Jan Bajer aka bajasoft <jbajer@gmail.com>
+* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2014 - 2016 Jan Bajer aka bajasoft <jbajer@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -25,6 +25,7 @@
 #include "../../../../core/ContentBlockingManager.h"
 #include "../../../../core/NetworkManagerFactory.h"
 #include "../../../../core/SettingsManager.h"
+#include "../../../../core/ThemesManager.h"
 #include "../../../../core/Utils.h"
 #include "../../../../ui/ContentsDialog.h"
 
@@ -32,7 +33,6 @@
 #include <QtGui/QDesktopServices>
 #include <QtGui/QGuiApplication>
 #include <QtGui/QWheelEvent>
-#include <QtWidgets/QFileDialog>
 #include <QtWidgets/QHBoxLayout>
 #include <QtWidgets/QInputDialog>
 #include <QtWidgets/QLineEdit>
@@ -53,15 +53,18 @@ QtWebKitPage::QtWebKitPage(QtWebKitNetworkManager *networkManager, QtWebKitWebWi
 	setNetworkAccessManager(m_networkManager);
 	setForwardUnsupportedContent(true);
 	updateStyleSheets();
-	optionChanged(QLatin1String("Interface/ShowScrollBars"), SettingsManager::getValue(QLatin1String("Interface/ShowScrollBars")));
+	optionChanged(SettingsManager::Interface_ShowScrollBarsOption);
 
-	connect(this, SIGNAL(loadFinished(bool)), this, SLOT(pageLoadFinished()));
-	connect(SettingsManager::getInstance(), SIGNAL(valueChanged(QString,QVariant)), this, SLOT(optionChanged(QString,QVariant)));
+	connect(this->mainFrame(), SIGNAL(loadFinished(bool)), this, SLOT(pageLoadFinished()));
+#ifndef OTTER_ENABLE_QTWEBKIT_LEGACY
+	connect(this, SIGNAL(consoleMessageReceived(MessageSource,MessageLevel,QString,int,QString)), this, SLOT(handleConsoleMessage(MessageSource,MessageLevel,QString,int,QString)));
+#endif
+	connect(SettingsManager::getInstance(), SIGNAL(valueChanged(int,QVariant)), this, SLOT(optionChanged(int)));
 }
 
 QtWebKitPage::QtWebKitPage() : QWebPage(),
-	m_widget(NULL),
-	m_networkManager(NULL),
+	m_widget(nullptr),
+	m_networkManager(nullptr),
 	m_ignoreJavaScriptPopups(false),
 	m_isPopup(false),
 	m_isViewingMedia(false)
@@ -75,11 +78,9 @@ QtWebKitPage::~QtWebKitPage()
 	m_popups.clear();
 }
 
-void QtWebKitPage::optionChanged(const QString &option, const QVariant &value)
+void QtWebKitPage::optionChanged(int identifier)
 {
-	Q_UNUSED(value)
-
-	if (option.startsWith(QLatin1String("Content/")) || option == QLatin1String("Interface/ShowScrollBars"))
+	if (SettingsManager::getOptionName(identifier).startsWith(QLatin1String("Content/")) || identifier == SettingsManager::Interface_ShowScrollBarsOption)
 	{
 		updateStyleSheets();
 	}
@@ -91,21 +92,61 @@ void QtWebKitPage::pageLoadFinished()
 
 	updateStyleSheets();
 
-	if (m_widget)
+	if (!m_widget || !m_widget->getOption(SettingsManager::ContentBlocking_EnableContentBlockingOption, m_widget->getUrl()).toBool())
 	{
-		const QStringList domainList = ContentBlockingManager::createSubdomainList(m_widget->getUrl().host());
+		return;
+	}
 
-		for (int i = 0; i < domainList.count(); ++i)
+	const QUrl url(m_widget->getUrl());
+	const QVector<int> profiles(ContentBlockingManager::getProfileList(m_widget->getOption(SettingsManager::ContentBlocking_ProfilesOption, url).toStringList()));
+
+	if (!profiles.isEmpty() && ContentBlockingManager::getCosmeticFiltersMode() != ContentBlockingManager::NoFiltersMode)
+	{
+		const ContentBlockingManager::CosmeticFiltersMode mode(ContentBlockingManager::checkUrl(profiles, url, url, NetworkManager::OtherType).comesticFiltersMode);
+
+		if (mode != ContentBlockingManager::NoFiltersMode)
 		{
-			applyContentBlockingRules(ContentBlockingManager::getStyleSheetBlackList(m_widget->getContentBlockingProfiles()).values(domainList.at(i)), true);
-			applyContentBlockingRules(ContentBlockingManager::getStyleSheetWhiteList(m_widget->getContentBlockingProfiles()).values(domainList.at(i)), false);
+			if (mode != ContentBlockingManager::DomainOnlyFiltersMode)
+			{
+				applyContentBlockingRules(ContentBlockingManager::getStyleSheet(profiles), true);
+			}
+
+			const QStringList domainList(ContentBlockingManager::createSubdomainList(url.host()));
+
+			for (int i = 0; i < domainList.count(); ++i)
+			{
+				applyContentBlockingRules(ContentBlockingManager::getStyleSheetBlackList(domainList.at(i), profiles), true);
+				applyContentBlockingRules(ContentBlockingManager::getStyleSheetWhiteList(domainList.at(i), profiles), false);
+			}
+		}
+	}
+
+	const QStringList blockedRequests(m_widget->getBlockedElements());
+
+	if (blockedRequests.count() > 0)
+	{
+		const QWebElementCollection elements(mainFrame()->documentElement().findAll(QLatin1String("[src]")));
+
+		for (int i = 0; i < blockedRequests.count(); ++i)
+		{
+			for (int j = 0; j < elements.count(); ++j)
+			{
+				QWebElement element(elements.at(j));
+
+				if (element.attribute(QLatin1String("src")) == blockedRequests[i]) ///NOTE: Consider comparing them as URLs
+				{
+					element.setStyleProperty(QLatin1String("display"), QLatin1String("none !important"));
+
+					break;
+				}
+			}
 		}
 	}
 }
 
 void QtWebKitPage::removePopup(const QUrl &url)
 {
-	QtWebKitPage *page = qobject_cast<QtWebKitPage*>(sender());
+	QtWebKitPage *page(qobject_cast<QtWebKitPage*>(sender()));
 
 	if (page)
 	{
@@ -124,37 +165,90 @@ void QtWebKitPage::markAsPopup()
 
 void QtWebKitPage::applyContentBlockingRules(const QStringList &rules, bool remove)
 {
+	const QWebElement document(mainFrame()->documentElement());
+	const QString value(remove ? QLatin1String("none !important") : QString());
+
 	for (int i = 0; i < rules.count(); ++i)
 	{
-		const QWebElementCollection elements = mainFrame()->documentElement().findAll(rules.at(i));
+		const QWebElementCollection elements(document.findAll(rules.at(i)));
 
 		for (int j = 0; j < elements.count(); ++j)
 		{
-			QWebElement element = elements.at(j);
+			QWebElement element(elements.at(j));
 
-			if (element.isNull())
+			if (!element.isNull())
 			{
-				continue;
-			}
-
-			if (remove)
-			{
-				element.removeFromDocument();
-			}
-			else
-			{
-				element.setStyleProperty(QLatin1String("display"), QLatin1String("block"));
+				element.setStyleProperty(QLatin1String("display"), value);
 			}
 		}
 	}
 }
 
+#ifndef OTTER_ENABLE_QTWEBKIT_LEGACY
+void QtWebKitPage::handleConsoleMessage(MessageSource category, MessageLevel level, const QString &message, int line, const QString &source)
+{
+	Console::MessageLevel mappedLevel(Console::UnknownLevel);
+
+	switch (level)
+	{
+		case LogMessageLevel:
+			mappedLevel = Console::LogLevel;
+
+			break;
+		case WarningMessageLevel:
+			mappedLevel = Console::WarningLevel;
+
+			break;
+		case ErrorMessageLevel:
+			mappedLevel = Console::ErrorLevel;
+
+			break;
+		default:
+			mappedLevel = Console::DebugLevel;
+
+			break;
+	}
+
+	Console::MessageCategory mappedCategory(Console::OtherCategory);
+
+	switch (category)
+	{
+		case NetworkMessageSource:
+			mappedCategory = Console::NetworkCategory;
+
+			break;
+		case ContentBlockerMessageSource:
+			mappedCategory = Console::ContentBlockingCategory;
+
+			break;
+		case SecurityMessageSource:
+			mappedCategory = Console::SecurityCategory;
+
+			break;
+		case CSSMessageSource:
+			mappedCategory = Console::CssCategory;
+
+			break;
+		case JSMessageSource:
+			mappedCategory = Console::JavaScriptCategory;
+
+			break;
+		default:
+			mappedCategory = Console::OtherCategory;
+
+			break;
+	}
+
+	Console::addMessage(message, mappedCategory, mappedLevel, source, line, (m_widget ? m_widget->getWindowIdentifier() : 0));
+}
+#endif
+
 void QtWebKitPage::updateStyleSheets(const QUrl &url)
 {
-	const QUrl currentUrl = (url.isEmpty() ? mainFrame()->url() : url);
-	QString styleSheet = QString(QStringLiteral("html {color: %1;} a {color: %2;} a:visited {color: %3;}")).arg(SettingsManager::getValue(QLatin1String("Content/TextColor")).toString()).arg(SettingsManager::getValue(QLatin1String("Content/LinkColor")).toString()).arg(SettingsManager::getValue(QLatin1String("Content/VisitedLinkColor")).toString()).toUtf8() + (m_widget ? ContentBlockingManager::getStyleSheet(m_widget->getContentBlockingProfiles()) : QByteArray());
-	QWebElement media = mainFrame()->findFirstElement(QLatin1String("img, audio source, video source"));
-	const bool isViewingMedia = (!media.isNull() && QUrl(media.attribute(QLatin1String("src"))) == currentUrl);
+	const QUrl currentUrl(url.isEmpty() ? mainFrame()->url() : url);
+	QString styleSheet((QStringLiteral("html {color: %1;} a {color: %2;} a:visited {color: %3;}")).arg(SettingsManager::getValue(SettingsManager::Content_TextColorOption).toString()).arg(SettingsManager::getValue(SettingsManager::Content_LinkColorOption).toString()).arg(SettingsManager::getValue(SettingsManager::Content_VisitedLinkColorOption).toString()).toUtf8());
+	QWebElement media(mainFrame()->findFirstElement(QLatin1String("img, audio source, video source")));
+	const bool isViewingMedia(!media.isNull() && QUrl(media.attribute(QLatin1String("src"))) == currentUrl);
 
 	if (isViewingMedia && media.tagName().toLower() == QLatin1String("img"))
 	{
@@ -173,12 +267,12 @@ void QtWebKitPage::updateStyleSheets(const QUrl &url)
 		emit viewingMediaChanged(m_isViewingMedia);
 	}
 
-	if (!SettingsManager::getValue(QLatin1String("Interface/ShowScrollBars")).toBool())
+	if (!SettingsManager::getValue(SettingsManager::Interface_ShowScrollBarsOption).toBool())
 	{
 		styleSheet.append(QLatin1String("body::-webkit-scrollbar {display:none;}"));
 	}
 
-	const QString userSyleSheet = (m_widget ? m_widget->getOption(QLatin1String("Content/UserStyleSheet"), currentUrl).toString() : QString());
+	const QString userSyleSheet(m_widget ? m_widget->getOption(SettingsManager::Content_UserStyleSheetOption, currentUrl).toString() : QString());
 
 	if (!userSyleSheet.isEmpty())
 	{
@@ -205,7 +299,9 @@ void QtWebKitPage::javaScriptAlert(QWebFrame *frame, const QString &message)
 		return;
 	}
 
-	ContentsDialog dialog(Utils::getIcon(QLatin1String("dialog-information")), tr("JavaScript"), message, QString(), QDialogButtonBox::Ok, NULL, m_widget);
+	emit m_widget->needsAttention();
+
+	ContentsDialog dialog(ThemesManager::getIcon(QLatin1String("dialog-information")), tr("JavaScript"), message, QString(), QDialogButtonBox::Ok, nullptr, m_widget);
 	dialog.setCheckBox(tr("Disable JavaScript popups"), false);
 
 	connect(m_widget, SIGNAL(aboutToReload()), &dialog, SLOT(close()));
@@ -218,18 +314,22 @@ void QtWebKitPage::javaScriptAlert(QWebFrame *frame, const QString &message)
 	}
 }
 
+#ifdef OTTER_ENABLE_QTWEBKIT_LEGACY
 void QtWebKitPage::javaScriptConsoleMessage(const QString &note, int line, const QString &source)
 {
-	Console::addMessage(note, JavaScriptMessageCategory, ErrorMessageLevel, source, line);
+	Console::addMessage(note, Console::JavaScriptCategory, Console::ErrorLevel, source, line, (m_widget ? m_widget->getWindowIdentifier() : 0));
 }
+#endif
 
 void QtWebKitPage::triggerAction(QWebPage::WebAction action, bool checked)
 {
-	if (action == InspectElement && m_widget)
+	if (action == InspectElement && m_widget && !m_widget->isInspecting())
 	{
 		QVariantMap parameters;
 		parameters[QLatin1String("isChecked")] = true;
 
+		settings()->setAttribute(QWebSettings::DeveloperExtrasEnabled, true);
+
 		m_widget->triggerAction(ActionsManager::InspectPageAction, parameters);
 	}
 
@@ -238,10 +338,15 @@ void QtWebKitPage::triggerAction(QWebPage::WebAction action, bool checked)
 
 QVariant QtWebKitPage::runScript(const QString &path, QWebElement element)
 {
+	if (element.isNull())
+	{
+		element = mainFrame()->documentElement();
+	}
+
 	QFile file(QString(":/modules/backends/web/qtwebkit/resources/%1.js").arg(path));
 	file.open(QIODevice::ReadOnly);
 
-	const QVariant result = (element.isNull() ? mainFrame()->evaluateJavaScript(file.readAll()) : element.evaluateJavaScript(file.readAll()));
+	const QVariant result(element.evaluateJavaScript(file.readAll()));
 
 	file.close();
 
@@ -252,13 +357,13 @@ QWebPage* QtWebKitPage::createWindow(QWebPage::WebWindowType type)
 {
 	if (type == QWebPage::WebBrowserWindow)
 	{
-		QtWebKitWebWidget *widget = NULL;
-		QString popupsPolicy = SettingsManager::getValue(QLatin1String("Content/PopupsPolicy")).toString();
-		bool isPopup = true;
+		QtWebKitWebWidget *widget(nullptr);
+		QString popupsPolicy(SettingsManager::getValue(SettingsManager::Content_PopupsPolicyOption).toString());
+		bool isPopup(true);
 
 		if (m_widget)
 		{
-			popupsPolicy = m_widget->getOption(QLatin1String("Content/PopupsPolicy"), (m_widget ? m_widget->getRequestedUrl() : QUrl())).toString();
+			popupsPolicy = m_widget->getOption(SettingsManager::Content_PopupsPolicyOption, (m_widget ? m_widget->getRequestedUrl() : QUrl())).toString();
 			isPopup = currentFrame()->hitTestContent(m_widget->getClickPosition()).linkUrl().isEmpty();
 		}
 
@@ -266,12 +371,12 @@ QWebPage* QtWebKitPage::createWindow(QWebPage::WebWindowType type)
 		{
 			if (popupsPolicy == QLatin1String("blockAll"))
 			{
-				return NULL;
+				return nullptr;
 			}
 
 			if (popupsPolicy == QLatin1String("ask"))
 			{
-				QtWebKitPage *page = new QtWebKitPage();
+				QtWebKitPage *page(new QtWebKitPage());
 				page->markAsPopup();
 
 				connect(page, SIGNAL(aboutToNavigate(QUrl,QWebFrame*,QWebPage::NavigationType)), this, SLOT(removePopup(QUrl)));
@@ -283,14 +388,15 @@ QWebPage* QtWebKitPage::createWindow(QWebPage::WebWindowType type)
 		if (m_widget)
 		{
 			widget = qobject_cast<QtWebKitWebWidget*>(m_widget->clone(false));
-			widget->setRequestedUrl(m_widget->getRequestedUrl(), false, true);
 		}
 		else
 		{
-			widget = new QtWebKitWebWidget(settings()->testAttribute(QWebSettings::PrivateBrowsingEnabled), NULL, NULL);
+			widget = new QtWebKitWebWidget(settings()->testAttribute(QWebSettings::PrivateBrowsingEnabled), nullptr, nullptr);
 		}
 
-		emit requestedNewWindow(widget, (popupsPolicy == QLatin1String("openAllInBackground") ? (WindowsManager::NewTabOpen | WindowsManager::BackgroundOpen) : WindowsManager::DefaultOpen));
+		widget->pageLoadStarted();
+
+		emit requestedNewWindow(widget, WindowsManager::calculateOpenHints(popupsPolicy == QLatin1String("openAllInBackground") ? (WindowsManager::NewTabOpen | WindowsManager::BackgroundOpen) : WindowsManager::NewTabOpen));
 
 		return widget->getPage();
 	}
@@ -298,9 +404,26 @@ QWebPage* QtWebKitPage::createWindow(QWebPage::WebWindowType type)
 	return QWebPage::createWindow(type);
 }
 
+QString QtWebKitPage::chooseFile(QWebFrame *frame, const QString &suggestedFile)
+{
+	Q_UNUSED(frame)
+
+	return Utils::getOpenPaths(QStringList(suggestedFile)).value(0);
+}
+
+QString QtWebKitPage::userAgentForUrl(const QUrl &url) const
+{
+	if (m_networkManager)
+	{
+		return m_networkManager->getUserAgent();
+	}
+
+	return QWebPage::userAgentForUrl(url);
+}
+
 QString QtWebKitPage::getDefaultUserAgent() const
 {
-	return userAgentForUrl(QUrl());
+	return QWebPage::userAgentForUrl(QUrl());
 }
 
 bool QtWebKitPage::acceptNavigationRequest(QWebFrame *frame, const QNetworkRequest &request, QWebPage::NavigationType type)
@@ -314,7 +437,7 @@ bool QtWebKitPage::acceptNavigationRequest(QWebFrame *frame, const QNetworkReque
 
 	if (frame && request.url().scheme() == QLatin1String("javascript"))
 	{
-		frame->evaluateJavaScript(request.url().path());
+		frame->documentElement().evaluateJavaScript(request.url().path());
 
 		return false;
 	}
@@ -331,14 +454,14 @@ bool QtWebKitPage::acceptNavigationRequest(QWebFrame *frame, const QNetworkReque
 		m_networkManager->setFormRequest(request.url());
 	}
 
-	if (type == QWebPage::NavigationTypeFormResubmitted && SettingsManager::getValue(QLatin1String("Choices/WarnFormResend")).toBool())
+	if (type == QWebPage::NavigationTypeFormResubmitted && SettingsManager::getValue(SettingsManager::Choices_WarnFormResendOption).toBool())
 	{
-		bool cancel = false;
-		bool warn = true;
+		bool cancel(false);
+		bool warn(true);
 
 		if (m_widget)
 		{
-			ContentsDialog dialog(Utils::getIcon(QLatin1String("dialog-warning")), tr("Question"), tr("Are you sure that you want to send form data again?"), tr("Do you want to resend data?"), (QDialogButtonBox::Yes | QDialogButtonBox::Cancel), NULL, m_widget);
+			ContentsDialog dialog(ThemesManager::getIcon(QLatin1String("dialog-warning")), tr("Question"), tr("Are you sure that you want to send form data again?"), tr("Do you want to resend data?"), (QDialogButtonBox::Yes | QDialogButtonBox::Cancel), nullptr, m_widget);
 			dialog.setCheckBox(tr("Do not show this message again"), false);
 
 			connect(m_widget, SIGNAL(aboutToReload()), &dialog, SLOT(close()));
@@ -363,7 +486,7 @@ bool QtWebKitPage::acceptNavigationRequest(QWebFrame *frame, const QNetworkReque
 			warn = !dialog.checkBox()->isChecked();
 		}
 
-		SettingsManager::setValue(QLatin1String("Choices/WarnFormResend"), warn);
+		SettingsManager::setValue(SettingsManager::Choices_WarnFormResendOption, warn);
 
 		if (cancel)
 		{
@@ -388,7 +511,9 @@ bool QtWebKitPage::javaScriptConfirm(QWebFrame *frame, const QString &message)
 		return QWebPage::javaScriptConfirm(frame, message);
 	}
 
-	ContentsDialog dialog(Utils::getIcon(QLatin1String("dialog-information")), tr("JavaScript"), message, QString(), (QDialogButtonBox::Ok | QDialogButtonBox::Cancel), NULL, m_widget);
+	emit m_widget->needsAttention();
+
+	ContentsDialog dialog(ThemesManager::getIcon(QLatin1String("dialog-information")), tr("JavaScript"), message, QString(), (QDialogButtonBox::Ok | QDialogButtonBox::Cancel), nullptr, m_widget);
 	dialog.setCheckBox(tr("Disable JavaScript popups"), false);
 
 	connect(m_widget, SIGNAL(aboutToReload()), &dialog, SLOT(close()));
@@ -415,17 +540,20 @@ bool QtWebKitPage::javaScriptPrompt(QWebFrame *frame, const QString &message, co
 		return QWebPage::javaScriptPrompt(frame, message, defaultValue, result);
 	}
 
-	QWidget *widget = new QWidget(m_widget);
-	QLineEdit *lineEdit = new QLineEdit(defaultValue, widget);
-	QLabel *label = new QLabel(message, widget);
+	emit m_widget->needsAttention();
+
+	QWidget *widget(new QWidget(m_widget));
+	QLineEdit *lineEdit(new QLineEdit(defaultValue, widget));
+	QLabel *label(new QLabel(message, widget));
 	label->setBuddy(lineEdit);
 	label->setTextFormat(Qt::PlainText);
 
-	QVBoxLayout *layout = new QVBoxLayout(widget);
+	QVBoxLayout *layout(new QVBoxLayout(widget));
+	layout->setContentsMargins(0, 0, 0, 0);
 	layout->addWidget(label);
 	layout->addWidget(lineEdit);
 
-	ContentsDialog dialog(Utils::getIcon(QLatin1String("dialog-information")), tr("JavaScript"), QString(), QString(), (QDialogButtonBox::Ok | QDialogButtonBox::Cancel), widget, m_widget);
+	ContentsDialog dialog(ThemesManager::getIcon(QLatin1String("dialog-information")), tr("JavaScript"), QString(), QString(), (QDialogButtonBox::Ok | QDialogButtonBox::Cancel), widget, m_widget);
 	dialog.setCheckBox(tr("Disable JavaScript popups"), false);
 
 	connect(m_widget, SIGNAL(aboutToReload()), &dialog, SLOT(close()));
@@ -449,7 +577,7 @@ bool QtWebKitPage::event(QEvent *event)
 {
 	if (event->type() == QEvent::Wheel)
 	{
-		QWheelEvent *wheelEvent = static_cast<QWheelEvent*>(event);
+		QWheelEvent *wheelEvent(static_cast<QWheelEvent*>(event));
 
 		if (wheelEvent->buttons() == Qt::RightButton)
 		{
@@ -464,27 +592,24 @@ bool QtWebKitPage::extension(QWebPage::Extension extension, const QWebPage::Exte
 {
 	if (extension == QWebPage::ChooseMultipleFilesExtension && m_widget)
 	{
-		const QWebPage::ChooseMultipleFilesExtensionOption *filesOption = static_cast<const QWebPage::ChooseMultipleFilesExtensionOption*>(option);
-		QWebPage::ChooseMultipleFilesExtensionReturn *filesOutput = static_cast<QWebPage::ChooseMultipleFilesExtensionReturn*>(output);
+		const QWebPage::ChooseMultipleFilesExtensionOption *filesOption(static_cast<const QWebPage::ChooseMultipleFilesExtensionOption*>(option));
+		QWebPage::ChooseMultipleFilesExtensionReturn *filesOutput(static_cast<QWebPage::ChooseMultipleFilesExtensionReturn*>(output));
 
-		filesOutput->fileNames = QFileDialog::getOpenFileNames(m_widget, tr("Open File"), QString(), filesOption->suggestedFileNames.join(QLatin1Char(';')));
+		filesOutput->fileNames = Utils::getOpenPaths(filesOption->suggestedFileNames, QStringList(), true);
 
 		return true;
 	}
 
 	if (extension == QWebPage::ErrorPageExtension)
 	{
-		const QWebPage::ErrorPageExtensionOption *errorOption = static_cast<const QWebPage::ErrorPageExtensionOption*>(option);
-		QWebPage::ErrorPageExtensionReturn *errorOutput = static_cast<QWebPage::ErrorPageExtensionReturn*>(output);
+		const QWebPage::ErrorPageExtensionOption *errorOption(static_cast<const QWebPage::ErrorPageExtensionOption*>(option));
+		QWebPage::ErrorPageExtensionReturn *errorOutput(static_cast<QWebPage::ErrorPageExtensionReturn*>(output));
 
-		if (!errorOption || !errorOutput || (errorOption->error == 203 && errorOption->domain == QWebPage::WebKit))
+		if (!errorOption || !errorOutput)
 		{
 			return false;
 		}
 
-		errorOutput->baseUrl = errorOption->url;
-		errorOutput->content = Utils::createErrorPage(errorOption->url, QString::number(errorOption->error), errorOption->errorString).toUtf8();
-
 		QString domain;
 
 		if (errorOption->domain == QWebPage::QtNetwork)
@@ -500,7 +625,41 @@ bool QtWebKitPage::extension(QWebPage::Extension extension, const QWebPage::Exte
 			domain = QLatin1String("HTTP");
 		}
 
-		Console::addMessage(tr("%1 error #%2: %3").arg(domain).arg(errorOption->error).arg(errorOption->errorString), NetworkMessageCategory, ErrorMessageLevel, errorOption->url.toString());
+		Console::addMessage(tr("%1 error #%2: %3").arg(domain).arg(errorOption->error).arg(errorOption->errorString), Console::NetworkCategory, Console::ErrorLevel, errorOption->url.toString(), -1, (m_widget ? m_widget->getWindowIdentifier() : 0));
+
+		if (errorOption->domain == QWebPage::WebKit && (errorOption->error == 102 || errorOption->error == 203))
+		{
+			return false;
+		}
+
+		QString title;
+
+		if ((errorOption->domain == QWebPage::QtNetwork && (errorOption->error == QNetworkReply::HostNotFoundError || errorOption->error == QNetworkReply::ContentNotFoundError)) || (errorOption->domain == QWebPage::Http && errorOption->error == 404))
+		{
+			if (errorOption->url.isLocalFile())
+			{
+				title = tr("File not found");
+			}
+			else
+			{
+				title = tr("Server not found");
+			}
+		}
+		else if (errorOption->domain == QWebPage::QtNetwork && errorOption->error == QNetworkReply::ConnectionRefusedError)
+		{
+			title = tr("Connection refused");
+		}
+		else if (errorOption->domain == QWebPage::WebKit)
+		{
+			title = tr("WebKit error %1").arg(errorOption->error);
+		}
+		else
+		{
+			title = tr("Network error %1").arg(errorOption->error);
+		}
+
+		errorOutput->baseUrl = errorOption->url;
+		errorOutput->content = Utils::createErrorPage(errorOption->url, title, errorOption->errorString).toUtf8();
 
 		return true;
 	}
@@ -512,7 +671,7 @@ bool QtWebKitPage::shouldInterruptJavaScript()
 {
 	if (m_widget)
 	{
-		ContentsDialog dialog(Utils::getIcon(QLatin1String("dialog-warning")), tr("Question"), tr("The script on this page appears to have a problem."), tr("Do you want to stop the script?"), (QDialogButtonBox::Yes | QDialogButtonBox::No), NULL, m_widget);
+		ContentsDialog dialog(ThemesManager::getIcon(QLatin1String("dialog-warning")), tr("Question"), tr("The script on this page appears to have a problem."), tr("Do you want to stop the script?"), (QDialogButtonBox::Yes | QDialogButtonBox::No), nullptr, m_widget);
 
 		connect(m_widget, SIGNAL(aboutToReload()), &dialog, SLOT(close()));
 
