@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2014 - 2015 Jan Bajer aka bajasoft <jbajer@gmail.com>
+* Copyright (C) 2014 - 2016 Jan Bajer aka bajasoft <jbajer@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #include "../../../core/Application.h"
 #include "../../../core/Console.h"
 #include "../../../core/NotificationsManager.h"
-#include "../../../core/Transfer.h"
+#include "../../../core/ThemesManager.h"
 #include "../../../core/TransfersManager.h"
 #include "../../../core/Utils.h"
 #include "../../../ui/MainWindow.h"
@@ -35,6 +35,8 @@
 #include <QtCore/QtMath>
 #include <QtGui/QDesktopServices>
 #include <QtWidgets/QFileIconProvider>
+#include <QtWinExtras/QWinJumpList>
+#include <QtWinExtras/QWinJumpListCategory>
 
 namespace Otter
 {
@@ -44,7 +46,7 @@ QProcessEnvironment WindowsPlatformIntegration::m_environment;
 WindowsPlatformIntegration::WindowsPlatformIntegration(Application *parent) : PlatformIntegration(parent),
 	m_registrationIdentifier(QLatin1String("OtterBrowser")),
 	m_applicationFilePath(QDir::toNativeSeparators(QCoreApplication::applicationFilePath())),
-	m_applicationRegistration(QLatin1String("HKEY_LOCAL_MACHINE\\Software\\RegisteredApplications"), QSettings::NativeFormat),
+	m_applicationRegistration(QLatin1String("HKEY_CURRENT_USER\\Software\\RegisteredApplications"), QSettings::NativeFormat),
 	m_propertiesRegistration(QLatin1String("HKEY_CURRENT_USER\\Software\\Classes\\") + m_registrationIdentifier, QSettings::NativeFormat),
 	m_cleanupTimer(0)
 {
@@ -60,6 +62,18 @@ WindowsPlatformIntegration::WindowsPlatformIntegration(Application *parent) : Pl
 		connect(TransfersManager::getInstance(), SIGNAL(transferRemoved(Transfer*)), this, SLOT(updateTaskbarButtons()));
 		connect(TransfersManager::getInstance(), SIGNAL(transferStopped(Transfer*)), this, SLOT(updateTaskbarButtons()));
 	}
+
+	if (QSysInfo::windowsVersion() >= QSysInfo::WV_VISTA)
+	{
+		const QString applicationFilePath(QCoreApplication::applicationFilePath());
+		QWinJumpList jumpLists;
+		QWinJumpListCategory* tasks(jumpLists.tasks());
+		tasks->addLink(ThemesManager::getIcon(QLatin1String("tab-new")), tr("New tab"), applicationFilePath, QStringList(QLatin1String("--new-tab")));
+		tasks->addLink(ThemesManager::getIcon(QLatin1String("tab-new-private")), tr("New private tab"), applicationFilePath, QStringList(QLatin1String("--new-private-tab")));
+		tasks->addLink(ThemesManager::getIcon(QLatin1String("window-new")), tr("New window"), applicationFilePath, QStringList(QLatin1String("--new-window")));
+		tasks->addLink(ThemesManager::getIcon(QLatin1String("window-new-private")), tr("New private window"), applicationFilePath, QStringList(QLatin1String("--new-private-window")));
+		tasks->setVisible(true);
+	}
 }
 
 void WindowsPlatformIntegration::timerEvent(QTimerEvent *event)
@@ -83,10 +97,10 @@ void WindowsPlatformIntegration::removeWindow(MainWindow *window)
 
 void WindowsPlatformIntegration::updateTaskbarButtons()
 {
-	const QList<Transfer*> transfers = TransfersManager::getInstance()->getTransfers();
-	qint64 bytesTotal = 0;
-	qint64 bytesReceived = 0;
-	bool hasActiveTransfers = false;
+	const QList<Transfer*> transfers(TransfersManager::getInstance()->getTransfers());
+	qint64 bytesTotal(0);
+	qint64 bytesReceived(0);
+	bool hasActiveTransfers(false);
 
 	for (int i = 0; i < transfers.count(); ++i)
 	{
@@ -98,11 +112,11 @@ void WindowsPlatformIntegration::updateTaskbarButtons()
 		}
 	}
 
-	const QList<MainWindow*> windows = Application::getInstance()->getWindows();
+	const QList<MainWindow*> windows(Application::getWindows());
 
 	for (int i = 0; i < windows.count(); ++i)
 	{
-		MainWindow *window = windows.at(i);
+		MainWindow *window(windows.at(i));
 
 		if (hasActiveTransfers)
 		{
@@ -127,7 +141,7 @@ void WindowsPlatformIntegration::updateTaskbarButtons()
 
 void WindowsPlatformIntegration::showNotification(Notification *notification)
 {
-	TrayIcon *trayIcon = Application::getInstance()->getTrayIcon();
+	TrayIcon *trayIcon(Application::getTrayIcon());
 
 	if (trayIcon && QSystemTrayIcon::supportsMessages())
 	{
@@ -135,7 +149,7 @@ void WindowsPlatformIntegration::showNotification(Notification *notification)
 	}
 	else
 	{
-		NotificationDialog *dialog = new NotificationDialog(notification);
+		NotificationDialog *dialog(new NotificationDialog(notification));
 		dialog->show();
 	}
 }
@@ -149,18 +163,18 @@ void WindowsPlatformIntegration::runApplication(const QString &command, const QU
 		return;
 	}
 
-	const int indexOfExecutable = command.indexOf(QLatin1String(".exe"), 0, Qt::CaseInsensitive);
+	const int indexOfExecutable(command.indexOf(QLatin1String(".exe"), 0, Qt::CaseInsensitive));
 
 	if (indexOfExecutable == -1)
 	{
-		Console::addMessage(tr("Failed to run command \"%1\", file is not executable").arg(command), OtherMessageCategory, ErrorMessageLevel);
+		Console::addMessage(tr("Failed to run command \"%1\", file is not executable").arg(command), Console::OtherCategory, Console::ErrorLevel);
 
 		return;
 	}
 
-	const QString application = command.left(indexOfExecutable + 4);
-	QStringList arguments = command.mid(indexOfExecutable + 5).split(QLatin1Char(' '), QString::SkipEmptyParts);
-	const int indexOfPlaceholder = arguments.indexOf(QLatin1String("%1"));
+	const QString application(command.left(indexOfExecutable + 4));
+	QStringList arguments(command.mid(indexOfExecutable + 5).split(QLatin1Char(' '), QString::SkipEmptyParts));
+	const int indexOfPlaceholder(arguments.indexOf(QLatin1String("%1")));
 
 	if (url.isValid())
 	{
@@ -180,31 +194,28 @@ void WindowsPlatformIntegration::runApplication(const QString &command, const QU
 
 	if (!QProcess::startDetached(application, arguments))
 	{
-		Console::addMessage(tr("Failed to run command \"%1\" (arguments: \"%2\")").arg(command).arg(arguments.join(QLatin1Char(' '))), OtherMessageCategory, ErrorMessageLevel);
+		Console::addMessage(tr("Failed to run command \"%1\" (arguments: \"%2\")").arg(command).arg(arguments.join(QLatin1Char(' '))), Console::OtherCategory, Console::ErrorLevel);
 	}
 }
 
 void WindowsPlatformIntegration::getApplicationInformation(ApplicationInformation &information)
 {
-	const QString rootPath = information.command.left(information.command.indexOf(QLatin1String("\\"))).remove(QLatin1Char('%'));
+	const QString rootPath(information.command.left(information.command.indexOf(QLatin1String("\\"))).remove(QLatin1Char('%')));
 
 	if (m_environment.contains(rootPath))
 	{
 		information.command.replace(QLatin1Char('%') + rootPath + QLatin1Char('%'), m_environment.value(rootPath));
 	}
 
-	const QString fullApplicationPath = information.command.left(information.command.indexOf(QLatin1String(".exe"), 0, Qt::CaseInsensitive) + 4);
-	const QFileInfo fileInfo(fullApplicationPath);
-	const QFileIconProvider fileIconProvider;
-	information.icon = fileIconProvider.icon(fileInfo);
-
-	HKEY key = NULL;
+	const QString fullApplicationPath(information.command.left(information.command.indexOf(QLatin1String(".exe"), 0, Qt::CaseInsensitive) + 4));
+	const QFileInfo fileInformation(fullApplicationPath);
+	HKEY key = nullptr;
 	TCHAR readBuffer[128];
 	DWORD bufferSize = sizeof(readBuffer);
 
 	if (RegOpenKeyEx(HKEY_CURRENT_USER, TEXT("Software\\Classes\\Local Settings\\Software\\Microsoft\\Windows\\Shell\\MuiCache"), 0, KEY_QUERY_VALUE, &key) == ERROR_SUCCESS)
 	{
-		if (RegQueryValueEx(key, fullApplicationPath.toStdWString().c_str(), NULL, NULL, (LPBYTE)readBuffer, &bufferSize) == ERROR_SUCCESS)
+		if (RegQueryValueEx(key, fullApplicationPath.toStdWString().c_str(), nullptr, nullptr, (LPBYTE)readBuffer, &bufferSize) == ERROR_SUCCESS)
 		{
 			information.name = QString::fromWCharArray(readBuffer);
 		}
@@ -214,18 +225,20 @@ void WindowsPlatformIntegration::getApplicationInformation(ApplicationInformatio
 
 	if (information.name.isEmpty())
 	{
-		information.name = fileInfo.baseName();
+		information.name = fileInformation.baseName();
 	}
+
+	information.icon = QFileIconProvider().icon(fileInformation);
 }
 
 QList<ApplicationInformation> WindowsPlatformIntegration::getApplicationsForMimeType(const QMimeType &mimeType)
 {
 	QList<ApplicationInformation> applications;
-	const QString suffix = mimeType.preferredSuffix();
+	const QString suffix(mimeType.preferredSuffix());
 
 	if (suffix.isEmpty())
 	{
-		Console::addMessage(tr("No valid suffix for given MIME type: %1").arg(mimeType.name()), OtherMessageCategory, ErrorMessageLevel);
+		Console::addMessage(tr("No valid suffix for given MIME type: %1").arg(mimeType.name()), Console::OtherCategory, Console::ErrorLevel);
 
 		return QList<ApplicationInformation>();
 	}
@@ -249,7 +262,7 @@ QList<ApplicationInformation> WindowsPlatformIntegration::getApplicationsForMime
 
 	// Vista+ applications
 	const QSettings defaultAssociation(QLatin1String("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FileExts\\.") + suffix, QSettings::NativeFormat);
-	QString defaultApplication = defaultAssociation.value(QLatin1String("."), QString()).toString();
+	QString defaultApplication(defaultAssociation.value(QLatin1String("."), QString()).toString());
 	QStringList associations;
 
 	if (defaultApplication.isEmpty())
@@ -272,7 +285,7 @@ QList<ApplicationInformation> WindowsPlatformIntegration::getApplicationsForMime
 
 	for (int i = 0; i < associations.count(); ++i)
 	{
-		const QString value = associations.at(i);
+		const QString value(associations.at(i));
 
 		if (m_registrationIdentifier == value)
 		{
@@ -292,7 +305,7 @@ QList<ApplicationInformation> WindowsPlatformIntegration::getApplicationsForMime
 
 		if (information.command.isEmpty())
 		{
-			Console::addMessage(tr("Failed to load a valid application path for MIME type %1: %2").arg(suffix).arg(value), OtherMessageCategory, ErrorMessageLevel);
+			Console::addMessage(tr("Failed to load a valid application path for MIME type %1: %2").arg(suffix).arg(value), Console::OtherCategory, Console::ErrorLevel);
 
 			continue;
 		}
@@ -304,8 +317,8 @@ QList<ApplicationInformation> WindowsPlatformIntegration::getApplicationsForMime
 
 	// Win XP applications
 	const QSettings legacyAssociations(QLatin1String("HKEY_CURRENT_USER\\Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\FileExts\\.") + suffix + QLatin1String("\\OpenWithList"), QSettings::NativeFormat);
-	const QString order = legacyAssociations.value(QLatin1String("MRUList"), QString()).toString();
-	const QString applicationFileName = QFileInfo(QCoreApplication::applicationFilePath()).fileName();
+	const QString order(legacyAssociations.value(QLatin1String("MRUList"), QString()).toString());
+	const QString applicationFileName(QFileInfo(QCoreApplication::applicationFilePath()).fileName());
 
 	associations = legacyAssociations.childKeys();
 	associations.removeAt(associations.indexOf(QLatin1String("MRUList")));
@@ -313,7 +326,7 @@ QList<ApplicationInformation> WindowsPlatformIntegration::getApplicationsForMime
 	for (int i = 0; i < associations.count(); ++i)
 	{
 		ApplicationInformation information;
-		const QString value = legacyAssociations.value(order.at(i)).toString();
+		const QString value(legacyAssociations.value(order.at(i)).toString());
 
 		if (applicationFileName == value)
 		{
@@ -342,7 +355,7 @@ QList<ApplicationInformation> WindowsPlatformIntegration::getApplicationsForMime
 				}
 				else
 				{
-					Console::addMessage(tr("Failed to load a valid application path for MIME type %1: %2").arg(suffix).arg(value), OtherMessageCategory, ErrorMessageLevel);
+					Console::addMessage(tr("Failed to load a valid application path for MIME type %1: %2").arg(suffix).arg(value), Console::OtherCategory, Console::ErrorLevel);
 
 					continue;
 				}
@@ -351,7 +364,7 @@ QList<ApplicationInformation> WindowsPlatformIntegration::getApplicationsForMime
 
 		getApplicationInformation(information);
 
-		bool exclude = false;
+		bool exclude(false);
 
 		for (int j = 0; j < applications.count(); ++j)
 		{
@@ -388,7 +401,6 @@ QString WindowsPlatformIntegration::getPlatform() const
 	}
 #endif
 	return QLatin1String("win32");
-
 }
 
 bool WindowsPlatformIntegration::canShowNotifications() const
@@ -424,8 +436,8 @@ bool WindowsPlatformIntegration::setAsDefaultBrowser()
 	if (QSysInfo::windowsVersion() >= QSysInfo::WV_10_0)
 	{
 		DWORD pid = 0;
-		IApplicationActivationManager *activationManager = NULL;
-		HRESULT result = CoCreateInstance(CLSID_ApplicationActivationManager, NULL, CLSCTX_INPROC_SERVER, IID_IApplicationActivationManager, (LPVOID*)&activationManager);
+		IApplicationActivationManager *activationManager = nullptr;
+		HRESULT result = CoCreateInstance(CLSID_ApplicationActivationManager, nullptr, CLSCTX_INPROC_SERVER, IID_IApplicationActivationManager, (LPVOID*)&activationManager);
 
 		if (result == S_OK)
 		{
@@ -439,12 +451,12 @@ bool WindowsPlatformIntegration::setAsDefaultBrowser()
 			}
 		}
 
-		Console::addMessage(QCoreApplication::translate("main", "Failed to run File Associations Manager, error code: %1\nApplication ID: %2").arg(result).arg(pid), Otter::OtherMessageCategory, ErrorMessageLevel);
+		Console::addMessage(QCoreApplication::translate("main", "Failed to run File Associations Manager, error code: %1\nApplication ID: %2").arg(result).arg(pid), Otter::Console::OtherCategory, Console::ErrorLevel);
 	}
 	else if (QSysInfo::windowsVersion() >= QSysInfo::WV_VISTA)
 	{
-		IApplicationAssociationRegistrationUI *applicationAssociationRegistrationUI = NULL;
-		HRESULT result = CoCreateInstance(CLSID_ApplicationAssociationRegistrationUI, NULL, CLSCTX_INPROC_SERVER, IID_IApplicationAssociationRegistrationUI, (LPVOID*)&applicationAssociationRegistrationUI);
+		IApplicationAssociationRegistrationUI *applicationAssociationRegistrationUI = nullptr;
+		HRESULT result = CoCreateInstance(CLSID_ApplicationAssociationRegistrationUI, nullptr, CLSCTX_INPROC_SERVER, IID_IApplicationAssociationRegistrationUI, (LPVOID*)&applicationAssociationRegistrationUI);
 
 		if (result == S_OK && applicationAssociationRegistrationUI)
 		{
@@ -458,11 +470,11 @@ bool WindowsPlatformIntegration::setAsDefaultBrowser()
 			}
 		}
 
-		Console::addMessage(QCoreApplication::translate("main", "Failed to run File Associations Manager, error code: %1").arg(result), Otter::OtherMessageCategory, ErrorMessageLevel);
+		Console::addMessage(QCoreApplication::translate("main", "Failed to run File Associations Manager, error code: %1").arg(result), Otter::Console::OtherCategory, Console::ErrorLevel);
 	}
 	else
 	{
-		SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_DWORD | SHCNF_FLUSH, NULL, NULL);
+		SHChangeNotify(SHCNE_ASSOCCHANGED, SHCNF_DWORD | SHCNF_FLUSH, nullptr, nullptr);
 		Sleep(1000);
 	}
 
@@ -482,7 +494,7 @@ bool WindowsPlatformIntegration::registerToSystem()
 	m_propertiesRegistration.setValue(QLatin1String("shell/open/command/."), QLatin1String("\"") + m_applicationFilePath + QLatin1String("\" \"%1\""));
 	m_propertiesRegistration.sync();
 
-	QSettings capabilities(QLatin1String("HKEY_LOCAL_MACHINE\\Software\\Clients\\StartMenuInternet\\") + m_registrationIdentifier, QSettings::NativeFormat);
+	QSettings capabilities(QLatin1String("HKEY_CURRENT_USER\\Software\\Clients\\StartMenuInternet\\") + m_registrationIdentifier, QSettings::NativeFormat);
 	capabilities.setValue(QLatin1String("./"), QLatin1String("Otter Browser"));
 	capabilities.setValue(QLatin1String("Capabilities/ApplicationDescription"), QLatin1String("Web browser controlled by the user, not vice-versa"));
 	capabilities.setValue(QLatin1String("Capabilities/ApplicationIcon"), m_applicationFilePath + QLatin1String(",0"));
@@ -513,7 +525,7 @@ bool WindowsPlatformIntegration::registerToSystem()
 
 	if (m_applicationRegistration.status() != QSettings::NoError || capabilities.status() != QSettings::NoError)
 	{
-		Console::addMessage(QCoreApplication::translate("main", "Failed to register application to system registry: %1, %2").arg(m_applicationRegistration.status(), capabilities.status()), Otter::OtherMessageCategory, ErrorMessageLevel);
+		Console::addMessage(QCoreApplication::translate("main", "Failed to register application to system registry: %1, %2").arg(m_applicationRegistration.status(), capabilities.status()), Otter::Console::OtherCategory, Console::ErrorLevel);
 
 		return false;
 	}
@@ -538,7 +550,7 @@ bool WindowsPlatformIntegration::canSetAsDefaultBrowser() const
 
 bool WindowsPlatformIntegration::isDefaultBrowser() const
 {
-	bool isDefault = true;
+	bool isDefault(true);
 
 	if (m_applicationRegistration.value(m_registrationIdentifier).isNull())
 	{
@@ -564,7 +576,7 @@ bool WindowsPlatformIntegration::isDefaultBrowser() const
 		}
 	}
 
-	isDefault &= (registry.value("Clients/StartmenuInternet/.", QString()).toString() == m_registrationIdentifier);
+	isDefault &= (registry.value(QLatin1String("Clients/StartmenuInternet/."), QString()).toString() == m_registrationIdentifier);
 
 	return isDefault;
 }
