@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -18,9 +18,9 @@
 **************************************************************************/
 
 #include "TransfersContentsWidget.h"
-#include "ProgressBarDelegate.h"
 #include "../../../core/ActionsManager.h"
-#include "../../../core/Transfer.h"
+#include "../../../core/ThemesManager.h"
+#include "../../../core/TransfersManager.h"
 #include "../../../core/Utils.h"
 
 #include "ui_TransfersContentsWidget.h"
@@ -38,6 +38,27 @@
 namespace Otter
 {
 
+ProgressBarDelegate::ProgressBarDelegate(QObject *parent) : ItemDelegate(parent)
+{
+}
+
+void ProgressBarDelegate::paint(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const
+{
+	QStyleOptionProgressBar progressBarOption;
+	progressBarOption.fontMetrics = option.fontMetrics;
+	progressBarOption.palette = option.palette;
+	progressBarOption.rect = option.rect;
+	progressBarOption.state = option.state;
+	progressBarOption.minimum = 0;
+	progressBarOption.maximum = 100;
+	progressBarOption.textAlignment = Qt::AlignCenter;
+	progressBarOption.textVisible = true;
+	progressBarOption.progress = index.data(Qt::DisplayRole).toInt();
+	progressBarOption.text = QStringLiteral("%1%").arg(progressBarOption.progress);
+
+	QApplication::style()->drawControl(QStyle::CE_ProgressBar, &progressBarOption, painter, 0);
+}
+
 TransfersContentsWidget::TransfersContentsWidget(Window *window) : ContentsWidget(window),
 	m_model(new QStandardItemModel(this)),
 	m_isLoading(false),
@@ -45,10 +66,7 @@ TransfersContentsWidget::TransfersContentsWidget(Window *window) : ContentsWidge
 {
 	m_ui->setupUi(this);
 
-	QStringList labels;
-	labels << QString() << tr("Filename") << tr("Size") << tr("Progress") << tr("Time") << tr("Speed") << tr("Started") << tr("Finished");
-
-	m_model->setHorizontalHeaderLabels(labels);
+	m_model->setHorizontalHeaderLabels(QStringList({QString(), tr("Filename"), tr("Size"), tr("Progress"), tr("Time"), tr("Speed"), tr("Started"), tr("Finished")}));
 
 	m_ui->transfersViewWidget->setModel(m_model);
 	m_ui->transfersViewWidget->header()->resizeSection(0, 30);
@@ -56,11 +74,11 @@ TransfersContentsWidget::TransfersContentsWidget(Window *window) : ContentsWidge
 	m_ui->transfersViewWidget->header()->setSectionResizeMode(1, QHeaderView::Stretch);
 	m_ui->transfersViewWidget->setItemDelegateForColumn(3, new ProgressBarDelegate(this));
 	m_ui->transfersViewWidget->installEventFilter(this);
-	m_ui->stopResumeButton->setIcon(Utils::getIcon(QLatin1String("task-ongoing")));
-	m_ui->redownloadButton->setIcon(Utils::getIcon(QLatin1String("view-refresh")));
+	m_ui->stopResumeButton->setIcon(ThemesManager::getIcon(QLatin1String("task-ongoing")));
+	m_ui->redownloadButton->setIcon(ThemesManager::getIcon(QLatin1String("view-refresh")));
 	m_ui->downloadLineEdit->installEventFilter(this);
 
-	const QList<Transfer*> transfers = TransfersManager::getTransfers();
+	const QList<Transfer*> transfers(TransfersManager::getTransfers());
 
 	for (int i = 0; i < transfers.count(); ++i)
 	{
@@ -101,21 +119,14 @@ void TransfersContentsWidget::changeEvent(QEvent *event)
 
 void TransfersContentsWidget::addTransfer(Transfer *transfer)
 {
-	QList<QStandardItem*> items;
-	QStandardItem *item = new QStandardItem();
-	item->setData(qVariantFromValue(static_cast<void*>(transfer)), Qt::UserRole);
-	item->setFlags(item->flags() | Qt::ItemNeverHasChildren);
-
-	items.append(item);
-
-	item = new QStandardItem(QFileInfo(transfer->getTarget()).fileName());
-	item->setFlags(item->flags() | Qt::ItemNeverHasChildren);
-
-	items.append(item);
+	QList<QStandardItem*> items({new QStandardItem(), new QStandardItem(QFileInfo(transfer->getTarget()).fileName())});
+	items[0]->setData(qVariantFromValue(static_cast<void*>(transfer)), Qt::UserRole);
+	items[0]->setFlags(items[0]->flags() | Qt::ItemNeverHasChildren);
+	items[1]->setFlags(items[1]->flags() | Qt::ItemNeverHasChildren);
 
 	for (int i = 2; i < m_model->columnCount(); ++i)
 	{
-		item = new QStandardItem();
+		QStandardItem *item(new QStandardItem());
 		item->setFlags(item->flags() | Qt::ItemNeverHasChildren);
 
 		items.append(item);
@@ -133,7 +144,7 @@ void TransfersContentsWidget::addTransfer(Transfer *transfer)
 
 void TransfersContentsWidget::removeTransfer(Transfer *transfer)
 {
-	const int row = findTransfer(transfer);
+	const int row(findTransfer(transfer));
 
 	if (row >= 0)
 	{
@@ -145,7 +156,7 @@ void TransfersContentsWidget::removeTransfer(Transfer *transfer)
 
 void TransfersContentsWidget::removeTransfer()
 {
-	Transfer *transfer = getTransfer(m_ui->transfersViewWidget->currentIndex());
+	Transfer *transfer(getTransfer(m_ui->transfersViewWidget->currentIndex()));
 
 	if (transfer)
 	{
@@ -164,7 +175,7 @@ void TransfersContentsWidget::removeTransfer()
 
 void TransfersContentsWidget::updateTransfer(Transfer *transfer)
 {
-	const int row = findTransfer(transfer);
+	const int row(findTransfer(transfer));
 
 	if (row < 0)
 	{
@@ -189,8 +200,8 @@ void TransfersContentsWidget::updateTransfer(Transfer *transfer)
 
 		if (transfer->getBytesTotal() > 0)
 		{
-			qint64 speedSum = 0;
-			const QList<qint64> speeds = m_speeds[transfer];
+			qint64 speedSum(0);
+			const QList<qint64> speeds(m_speeds[transfer]);
 
 			for (int i = 0; i < speeds.count(); ++i)
 			{
@@ -212,15 +223,15 @@ void TransfersContentsWidget::updateTransfer(Transfer *transfer)
 	switch (transfer->getState())
 	{
 		case Transfer::RunningState:
-			icon = Utils::getIcon(QLatin1String("task-ongoing"));
+			icon = ThemesManager::getIcon(QLatin1String("task-ongoing"));
 
 			break;
 		case Transfer::FinishedState:
-			icon = Utils::getIcon(QLatin1String("task-complete"));
+			icon = ThemesManager::getIcon(QLatin1String("task-complete"));
 
 			break;
 		default:
-			icon = Utils::getIcon(QLatin1String("task-reject"));
+			icon = ThemesManager::getIcon(QLatin1String("task-reject"));
 
 			break;
 	}
@@ -234,7 +245,7 @@ void TransfersContentsWidget::updateTransfer(Transfer *transfer)
 	m_model->item(row, 6)->setText(transfer->getTimeStarted().toString(QLatin1String("yyyy-MM-dd HH:mm:ss")));
 	m_model->item(row, 7)->setText(transfer->getTimeFinished().toString(QLatin1String("yyyy-MM-dd HH:mm:ss")));
 
-	const QString tooltip = tr("<div style=\"white-space:pre;\">Source: %1\nTarget: %2\nSize: %3\nDownloaded: %4\nProgress: %5</div>").arg(transfer->getSource().toDisplayString().toHtmlEscaped()).arg(transfer->getTarget().toHtmlEscaped()).arg(Utils::formatUnit(transfer->getBytesTotal(), false, 1, true)).arg(Utils::formatUnit(transfer->getBytesReceived(), false, 1, true)).arg(QStringLiteral("%1%").arg(((transfer->getBytesTotal() > 0) ? ((static_cast<qreal>(transfer->getBytesReceived()) / transfer->getBytesTotal()) * 100) : 0.0), 0, 'f', 1));
+	const QString tooltip(tr("<div style=\"white-space:pre;\">Source: %1\nTarget: %2\nSize: %3\nDownloaded: %4\nProgress: %5</div>").arg(transfer->getSource().toDisplayString().toHtmlEscaped()).arg(transfer->getTarget().toHtmlEscaped()).arg(Utils::formatUnit(transfer->getBytesTotal(), false, 1, true)).arg(Utils::formatUnit(transfer->getBytesReceived(), false, 1, true)).arg(QStringLiteral("%1%").arg(((transfer->getBytesTotal() > 0) ? ((static_cast<qreal>(transfer->getBytesReceived()) / transfer->getBytesTotal()) * 100) : 0.0), 0, 'f', 1)));
 
 	for (int i = 0; i < m_model->columnCount(); ++i)
 	{
@@ -246,7 +257,7 @@ void TransfersContentsWidget::updateTransfer(Transfer *transfer)
 		updateActions();
 	}
 
-	const bool isRunning = (transfer && transfer->getState() == Transfer::RunningState);
+	const bool isRunning(transfer && transfer->getState() == Transfer::RunningState);
 
 	if (isRunning != m_isLoading)
 	{
@@ -254,12 +265,12 @@ void TransfersContentsWidget::updateTransfer(Transfer *transfer)
 		{
 			m_isLoading = true;
 
-			emit loadingChanged(true);
+			emit loadingStateChanged(WindowsManager::OngoingLoadingState);
 		}
 		else
 		{
-			const QList<Transfer*> transfers = TransfersManager::getTransfers();
-			bool hasRunning = false;
+			const QList<Transfer*> transfers(TransfersManager::getTransfers());
+			bool hasRunning(false);
 
 			for (int i = 0; i < transfers.count(); ++i)
 			{
@@ -275,7 +286,7 @@ void TransfersContentsWidget::updateTransfer(Transfer *transfer)
 			{
 				m_isLoading = false;
 
-				emit loadingChanged(false);
+				emit loadingStateChanged(WindowsManager::FinishedLoadingState);
 			}
 		}
 	}
@@ -283,7 +294,7 @@ void TransfersContentsWidget::updateTransfer(Transfer *transfer)
 
 void TransfersContentsWidget::openTransfer(const QModelIndex &index)
 {
-	Transfer *transfer = getTransfer(index.isValid() ? index : m_ui->transfersViewWidget->currentIndex());
+	Transfer *transfer(getTransfer(index.isValid() ? index : m_ui->transfersViewWidget->currentIndex()));
 
 	if (transfer)
 	{
@@ -293,7 +304,7 @@ void TransfersContentsWidget::openTransfer(const QModelIndex &index)
 
 void TransfersContentsWidget::openTransfer(QAction *action)
 {
-	Transfer *transfer = getTransfer(m_ui->transfersViewWidget->currentIndex());
+	Transfer *transfer(getTransfer(m_ui->transfersViewWidget->currentIndex()));
 
 	if (transfer && action && !action->data().isNull())
 	{
@@ -303,7 +314,7 @@ void TransfersContentsWidget::openTransfer(QAction *action)
 
 void TransfersContentsWidget::openTransferFolder(const QModelIndex &index)
 {
-	Transfer *transfer = getTransfer(index.isValid() ? index : m_ui->transfersViewWidget->currentIndex());
+	Transfer *transfer(getTransfer(index.isValid() ? index : m_ui->transfersViewWidget->currentIndex()));
 
 	if (transfer)
 	{
@@ -313,7 +324,7 @@ void TransfersContentsWidget::openTransferFolder(const QModelIndex &index)
 
 void TransfersContentsWidget::copyTransferInformation()
 {
-	QStandardItem *item = m_model->itemFromIndex(m_ui->transfersViewWidget->currentIndex());
+	QStandardItem *item(m_model->itemFromIndex(m_ui->transfersViewWidget->currentIndex()));
 
 	if (item)
 	{
@@ -323,7 +334,7 @@ void TransfersContentsWidget::copyTransferInformation()
 
 void TransfersContentsWidget::stopResumeTransfer()
 {
-	Transfer *transfer = getTransfer(m_ui->transfersViewWidget->selectionModel()->hasSelection() ? m_ui->transfersViewWidget->selectionModel()->currentIndex() : QModelIndex());
+	Transfer *transfer(getTransfer(m_ui->transfersViewWidget->selectionModel()->hasSelection() ? m_ui->transfersViewWidget->selectionModel()->currentIndex() : QModelIndex()));
 
 	if (transfer)
 	{
@@ -342,7 +353,7 @@ void TransfersContentsWidget::stopResumeTransfer()
 
 void TransfersContentsWidget::redownloadTransfer()
 {
-	Transfer *transfer = getTransfer(m_ui->transfersViewWidget->selectionModel()->hasSelection() ? m_ui->transfersViewWidget->selectionModel()->currentIndex() : QModelIndex());
+	Transfer *transfer(getTransfer(m_ui->transfersViewWidget->selectionModel()->hasSelection() ? m_ui->transfersViewWidget->selectionModel()->currentIndex() : QModelIndex()));
 
 	if (transfer)
 	{
@@ -352,7 +363,7 @@ void TransfersContentsWidget::redownloadTransfer()
 
 void TransfersContentsWidget::startQuickTransfer()
 {
-	TransfersManager::startTransfer(m_ui->downloadLineEdit->text(), QString(), (Transfer::IsQuickTransferOption | (SessionsManager::isPrivate() ? Transfer::IsPrivateOption : Transfer::NoOption)));
+	TransfersManager::startTransfer(m_ui->downloadLineEdit->text(), QString(), (Transfer::CanNotifyOption | Transfer::IsQuickTransferOption | (SessionsManager::isPrivate() ? Transfer::IsPrivateOption : Transfer::NoOption)));
 
 	m_ui->downloadLineEdit->clear();
 }
@@ -364,18 +375,18 @@ void TransfersContentsWidget::clearFinishedTransfers()
 
 void TransfersContentsWidget::showContextMenu(const QPoint &point)
 {
-	Transfer *transfer = getTransfer(m_ui->transfersViewWidget->indexAt(point));
+	Transfer *transfer(getTransfer(m_ui->transfersViewWidget->indexAt(point)));
 	QMenu menu(this);
 
 	if (transfer)
 	{
 		menu.addAction(tr("Open"), this, SLOT(openTransfer()));
 
-		const QList<ApplicationInformation> applications = Utils::getApplicationsForMimeType(transfer->getMimeType());
+		const QList<ApplicationInformation> applications(Utils::getApplicationsForMimeType(transfer->getMimeType()));
 
 		if (applications.count() > 1)
 		{
-			QMenu *applicationsMenu = menu.addMenu(tr("Open With"));
+			QMenu *applicationsMenu(menu.addMenu(tr("Open With")));
 
 			for (int i = 0; i < applications.count(); ++i)
 			{
@@ -400,8 +411,8 @@ void TransfersContentsWidget::showContextMenu(const QPoint &point)
 		menu.addAction(tr("Remove"), this, SLOT(removeTransfer()));
 	}
 
-	const QList<Transfer*> transfers = TransfersManager::getTransfers();
-	int finishedTransfers = 0;
+	const QList<Transfer*> transfers(TransfersManager::getTransfers());
+	int finishedTransfers(0);
 
 	for (int i = 0; i < transfers.count(); ++i)
 	{
@@ -417,17 +428,17 @@ void TransfersContentsWidget::showContextMenu(const QPoint &point)
 
 void TransfersContentsWidget::updateActions()
 {
-	Transfer *transfer = getTransfer(m_ui->transfersViewWidget->selectionModel()->hasSelection() ? m_ui->transfersViewWidget->selectionModel()->currentIndex() : QModelIndex());
+	Transfer *transfer(getTransfer(m_ui->transfersViewWidget->selectionModel()->hasSelection() ? m_ui->transfersViewWidget->selectionModel()->currentIndex() : QModelIndex()));
 
 	if (transfer && transfer->getState() == Transfer::ErrorState)
 	{
 		m_ui->stopResumeButton->setText(tr("Resume"));
-		m_ui->stopResumeButton->setIcon(Utils::getIcon(QLatin1String("task-ongoing")));
+		m_ui->stopResumeButton->setIcon(ThemesManager::getIcon(QLatin1String("task-ongoing")));
 	}
 	else
 	{
 		m_ui->stopResumeButton->setText(tr("Stop"));
-		m_ui->stopResumeButton->setIcon(Utils::getIcon(QLatin1String("task-reject")));
+		m_ui->stopResumeButton->setIcon(ThemesManager::getIcon(QLatin1String("task-reject")));
 	}
 
 	m_ui->stopResumeButton->setEnabled(transfer && (transfer->getState() == Transfer::RunningState || transfer->getState() == Transfer::ErrorState));
@@ -439,7 +450,9 @@ void TransfersContentsWidget::updateActions()
 	if (transfer)
 	{
 		m_ui->sourceLabelWidget->setText(transfer->getSource().toDisplayString());
+		m_ui->sourceLabelWidget->setUrl(transfer->getSource());
 		m_ui->targetLabelWidget->setText(transfer->getTarget());
+		m_ui->targetLabelWidget->setUrl(QUrl(transfer->getTarget()));
 		m_ui->sizeLabelWidget->setText(Utils::formatUnit(transfer->getBytesTotal(), false, 1, true));
 		m_ui->downloadedLabelWidget->setText(Utils::formatUnit(transfer->getBytesReceived(), false, 1, true));
 		m_ui->progressLabelWidget->setText(QStringLiteral("%1%").arg(((transfer->getBytesTotal() > 0) ? ((static_cast<qreal>(transfer->getBytesReceived()) / transfer->getBytesTotal()) * 100) : 0.0), 0, 'f', 1));
@@ -472,11 +485,11 @@ void TransfersContentsWidget::triggerAction(int identifier, const QVariantMap &p
 			}
 			else
 			{
-				QWidget *widget = focusWidget();
+				QWidget *widget(focusWidget());
 
 				if (widget->metaObject()->className() == QLatin1String("Otter::TextLabelWidget"))
 				{
-					TextLabelWidget *label = qobject_cast<TextLabelWidget*>(widget);
+					TextLabelWidget *label(qobject_cast<TextLabelWidget*>(widget));
 
 					if (label)
 					{
@@ -513,7 +526,7 @@ Transfer* TransfersContentsWidget::getTransfer(const QModelIndex &index)
 		return static_cast<Transfer*>(m_model->item(index.row(), 0)->data(Qt::UserRole).value<void*>());
 	}
 
-	return NULL;
+	return nullptr;
 }
 
 Action* TransfersContentsWidget::getAction(int identifier)
@@ -525,10 +538,10 @@ Action* TransfersContentsWidget::getAction(int identifier)
 
 	if (identifier != ActionsManager::CopyAction && identifier != ActionsManager::DeleteAction)
 	{
-		return NULL;
+		return nullptr;
 	}
 
-	Action *action = new Action(identifier, this);
+	Action *action(new Action(identifier, this));
 
 	m_actions[identifier] = action;
 
@@ -554,7 +567,12 @@ QUrl TransfersContentsWidget::getUrl() const
 
 QIcon TransfersContentsWidget::getIcon() const
 {
-	return Utils::getIcon(QLatin1String("transfers"), false);
+	return ThemesManager::getIcon(QLatin1String("transfers"), false);
+}
+
+WindowsManager::LoadingState TransfersContentsWidget::getLoadingState() const
+{
+	return (m_isLoading ? WindowsManager::OngoingLoadingState : WindowsManager::FinishedLoadingState);
 }
 
 int TransfersContentsWidget::findTransfer(Transfer *transfer) const
@@ -575,16 +593,11 @@ int TransfersContentsWidget::findTransfer(Transfer *transfer) const
 	return -1;
 }
 
-bool TransfersContentsWidget::isLoading() const
-{
-	return m_isLoading;
-}
-
 bool TransfersContentsWidget::eventFilter(QObject *object, QEvent *event)
 {
 	if (object == m_ui->transfersViewWidget && event->type() == QEvent::KeyPress)
 	{
-		QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
+		QKeyEvent *keyEvent(static_cast<QKeyEvent*>(event));
 
 		if (keyEvent && (keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return))
 		{
@@ -602,7 +615,7 @@ bool TransfersContentsWidget::eventFilter(QObject *object, QEvent *event)
 	}
 	else if (object == m_ui->downloadLineEdit && event->type() == QEvent::KeyPress)
 	{
-		QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
+		QKeyEvent *keyEvent(static_cast<QKeyEvent*>(event));
 
 		if (keyEvent->key() == Qt::Key_Escape)
 		{
