@@ -1,6 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2016 Piotr Wójcik <chocimier@tlen.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -26,11 +27,12 @@
 #include "../../../core/GesturesManager.h"
 #include "../../../core/SettingsManager.h"
 #include "../../../core/Utils.h"
+#include "../../../core/WindowsManager.h"
+#include "../../../modules/widgets/search/SearchWidget.h"
 #include "../../../ui/BookmarkPropertiesDialog.h"
 #include "../../../ui/MainWindow.h"
 #include "../../../ui/Menu.h"
 #include "../../../ui/OpenAddressDialog.h"
-#include "../../../ui/toolbars/SearchWidget.h"
 
 #include <QtGui/QGuiApplication>
 #include <QtGui/QMouseEvent>
@@ -43,7 +45,7 @@
 namespace Otter
 {
 
-StartPageModel* StartPageWidget::m_model = NULL;
+StartPageModel* StartPageWidget::m_model(nullptr);
 
 StartPageContentsWidget::StartPageContentsWidget(QWidget *parent) : QWidget(parent),
 	m_color(Qt::transparent),
@@ -76,8 +78,8 @@ void StartPageContentsWidget::paintEvent(QPaintEvent *event)
 	{
 		case BestFitBackground:
 			{
-				const QString key = QLatin1String("start-page-best-fit-") + QString::number(width()) + QLatin1Char('-') + QString::number(height());
-				QPixmap *cachedBackground = QPixmapCache::find(key);
+				const QString key(QLatin1String("start-page-best-fit-") + QString::number(width()) + QLatin1Char('-') + QString::number(height()));
+				QPixmap *cachedBackground(QPixmapCache::find(key));
 
 				if (cachedBackground)
 				{
@@ -85,8 +87,8 @@ void StartPageContentsWidget::paintEvent(QPaintEvent *event)
 				}
 				else
 				{
-					const qreal pixmapAscpectRatio = (pixmap.width() / qreal(pixmap.height()));
-					const qreal backgroundAscpectRatio = (width() / qreal(height()));
+					const qreal pixmapAscpectRatio(pixmap.width() / qreal(pixmap.height()));
+					const qreal backgroundAscpectRatio(width() / qreal(height()));
 					QPixmap newBackground(size());
 
 					if (pixmapAscpectRatio > backgroundAscpectRatio)
@@ -115,8 +117,8 @@ void StartPageContentsWidget::paintEvent(QPaintEvent *event)
 			break;
 		case StretchBackground:
 			{
-				const QString key = QLatin1String("start-page-stretch-") + QString::number(width()) + QLatin1Char('-') + QString::number(height());
-				QPixmap *cachedBackground = QPixmapCache::find(key);
+				const QString key(QLatin1String("start-page-stretch-") + QString::number(width()) + QLatin1Char('-') + QString::number(height()));
+				QPixmap *cachedBackground(QPixmapCache::find(key));
 
 				if (cachedBackground)
 				{
@@ -124,7 +126,7 @@ void StartPageContentsWidget::paintEvent(QPaintEvent *event)
 				}
 				else
 				{
-					const QPixmap newBackground = pixmap.scaled(size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+					const QPixmap newBackground(pixmap.scaled(size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation));
 
 					painter.drawPixmap(contentsRect(), newBackground, contentsRect());
 
@@ -144,10 +146,10 @@ void StartPageContentsWidget::paintEvent(QPaintEvent *event)
 
 void StartPageContentsWidget::setBackgroundMode(StartPageContentsWidget::BackgroundMode mode)
 {
-	const QString color = SettingsManager::getValue(QLatin1String("StartPage/BackgroundColor")).toString();
+	const QString color(SettingsManager::getValue(SettingsManager::StartPage_BackgroundColorOption).toString());
 
-	m_path = ((mode == NoCustomBackground) ? QString() : SettingsManager::getValue(QLatin1String("StartPage/BackgroundPath")).toString());
-	m_color = (color.isEmpty() ? QColor(Qt::transparent) : QColor(color));
+	m_path = ((mode == NoCustomBackground) ? QString() : SettingsManager::getValue(SettingsManager::StartPage_BackgroundPathOption).toString());
+	m_color = ((mode == NoCustomBackground || color.isEmpty()) ? QColor(Qt::transparent) : QColor(color));
 	m_mode = mode;
 
 	update();
@@ -157,7 +159,7 @@ StartPageWidget::StartPageWidget(Window *window, QWidget *parent) : QScrollArea(
 	m_window(window),
 	m_contentsWidget(new StartPageContentsWidget(this)),
 	m_listView(new QListView(this)),
-	m_searchWidget(NULL),
+	m_searchWidget(nullptr),
 	m_ignoreEnter(false)
 {
 	if (!m_model)
@@ -188,12 +190,12 @@ StartPageWidget::StartPageWidget(Window *window, QWidget *parent) : QScrollArea(
 	setWidgetResizable(true);
 	setAlignment(Qt::AlignHCenter);
 	updateTiles();
-	optionChanged(QLatin1String("StartPage/BackgroundPath"), SettingsManager::getValue(QLatin1String("StartPage/BackgroundPath")));
-	optionChanged(QLatin1String("StartPage/ShowSearchField"), SettingsManager::getValue(QLatin1String("StartPage/ShowSearchField")));
+	optionChanged(SettingsManager::StartPage_BackgroundPathOption, SettingsManager::getValue(SettingsManager::StartPage_BackgroundPathOption));
+	optionChanged(SettingsManager::StartPage_ShowSearchFieldOption, SettingsManager::getValue(SettingsManager::StartPage_ShowSearchFieldOption));
 
 	connect(m_model, SIGNAL(modelModified()), this, SLOT(updateTiles()));
 	connect(m_model, SIGNAL(isReloadingTileChanged(QModelIndex)), this, SLOT(updateTile(QModelIndex)));
-	connect(SettingsManager::getInstance(), SIGNAL(valueChanged(QString,QVariant)), this, SLOT(optionChanged(QString,QVariant)));
+	connect(SettingsManager::getInstance(), SIGNAL(valueChanged(int,QVariant)), this, SLOT(optionChanged(int,QVariant)));
 }
 
 void StartPageWidget::resizeEvent(QResizeEvent *event)
@@ -226,98 +228,114 @@ void StartPageWidget::wheelEvent(QWheelEvent *event)
 	}
 }
 
-void StartPageWidget::optionChanged(const QString &option, const QVariant &value)
+void StartPageWidget::optionChanged(int identifier, const QVariant &value)
 {
-	if (option == QLatin1String("StartPage/TilesPerRow") || option == QLatin1String("StartPage/TileHeight") || option == QLatin1String("StartPage/TileWidth") || option == QLatin1String("StartPage/ZoomLevel"))
+	switch (identifier)
 	{
-		updateSize();
-	}
-	else if (option == QLatin1String("StartPage/BackgroundColor") || option == QLatin1String("StartPage/BackgroundMode") || option == QLatin1String("StartPage/BackgroundPath"))
-	{
-		const QString backgroundMode = SettingsManager::getValue(QLatin1String("StartPage/BackgroundMode")).toString();
-
-		if (backgroundMode == QLatin1String("bestFit"))
-		{
-			m_contentsWidget->setBackgroundMode(StartPageContentsWidget::BestFitBackground);
-		}
-		else if (backgroundMode == QLatin1String("center"))
-		{
-			m_contentsWidget->setBackgroundMode(StartPageContentsWidget::CenterBackground);
-		}
-		else if (backgroundMode == QLatin1String("stretch"))
-		{
-			m_contentsWidget->setBackgroundMode(StartPageContentsWidget::StretchBackground);
-		}
-		else if (backgroundMode == QLatin1String("tile"))
-		{
-			m_contentsWidget->setBackgroundMode(StartPageContentsWidget::TileBackground);
-		}
-		else
-		{
-			m_contentsWidget->setBackgroundMode(StartPageContentsWidget::NoCustomBackground);
-		}
-	}
-	else if (option == QLatin1String("StartPage/ShowSearchField"))
-	{
-		QGridLayout *layout = NULL;
-		const bool needsInitialization = (m_contentsWidget->layout() == NULL);
-
-		if (needsInitialization)
-		{
-			layout = new QGridLayout(m_contentsWidget);
-			layout->setContentsMargins(0, 0, 0, 0);
-			layout->setSpacing(0);
-		}
-		else if ((m_searchWidget && !value.toBool()) || (!m_searchWidget && value.toBool()))
-		{
-			layout = qobject_cast<QGridLayout*>(m_contentsWidget->layout());
-
-			for (int i = (layout->count() - 1); i >=0; --i)
+		case SettingsManager::StartPage_BackgroundColorOption:
+		case SettingsManager::StartPage_BackgroundModeOption:
+		case SettingsManager::StartPage_BackgroundPathOption:
 			{
-				QLayoutItem *item = layout->takeAt(i);
+				const QString backgroundMode(SettingsManager::getValue(SettingsManager::StartPage_BackgroundModeOption).toString());
 
-				if (item)
+				if (backgroundMode == QLatin1String("bestFit"))
 				{
-					if (item->widget())
+					m_contentsWidget->setBackgroundMode(StartPageContentsWidget::BestFitBackground);
+				}
+				else if (backgroundMode == QLatin1String("center"))
+				{
+					m_contentsWidget->setBackgroundMode(StartPageContentsWidget::CenterBackground);
+				}
+				else if (backgroundMode == QLatin1String("stretch"))
+				{
+					m_contentsWidget->setBackgroundMode(StartPageContentsWidget::StretchBackground);
+				}
+				else if (backgroundMode == QLatin1String("tile"))
+				{
+					m_contentsWidget->setBackgroundMode(StartPageContentsWidget::TileBackground);
+				}
+				else
+				{
+					m_contentsWidget->setBackgroundMode(StartPageContentsWidget::NoCustomBackground);
+				}
+
+				update();
+			}
+
+			break;
+		case SettingsManager::StartPage_ShowSearchFieldOption:
+			{
+				QGridLayout *layout(nullptr);
+				const bool needsInitialization(m_contentsWidget->layout() == nullptr);
+
+				if (needsInitialization)
+				{
+					layout = new QGridLayout(m_contentsWidget);
+					layout->setContentsMargins(0, 0, 0, 0);
+					layout->setSpacing(0);
+				}
+				else if ((m_searchWidget && !value.toBool()) || (!m_searchWidget && value.toBool()))
+				{
+					layout = qobject_cast<QGridLayout*>(m_contentsWidget->layout());
+
+					for (int i = (layout->count() - 1); i >=0; --i)
 					{
-						item->widget()->setParent(m_contentsWidget);
+						QLayoutItem *item(layout->takeAt(i));
+
+						if (item)
+						{
+							if (item->widget())
+							{
+								item->widget()->setParent(m_contentsWidget);
+							}
+
+							delete item;
+						}
+					}
+				}
+
+				if (value.toBool() && (needsInitialization || !m_searchWidget))
+				{
+					if (!m_searchWidget)
+					{
+						m_searchWidget = new SearchWidget(m_window, this);
+						m_searchWidget->setFixedWidth(300);
 					}
 
-					delete item;
+					layout->addItem(new QSpacerItem(1, 50, QSizePolicy::Fixed, QSizePolicy::Fixed), 0, 1);
+					layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding), 1, 0);
+					layout->addWidget(m_searchWidget, 1, 1, 1, 1, Qt::AlignCenter);
+					layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding), 1, 2);
+					layout->addItem(new QSpacerItem(1, 50, QSizePolicy::Fixed, QSizePolicy::Fixed), 2, 1);
+					layout->addWidget(m_listView, 3, 0, 1, 3, Qt::AlignCenter);
+					layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding, QSizePolicy::Expanding), 4, 1);
+				}
+				else if (!value.toBool() && (needsInitialization || m_searchWidget))
+				{
+					if (m_searchWidget)
+					{
+						m_searchWidget->deleteLater();
+						m_searchWidget = nullptr;
+					}
+
+					layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding, QSizePolicy::Expanding), 0, 1);
+					layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding, QSizePolicy::Expanding), 1, 0);
+					layout->addWidget(m_listView, 1, 1, 1, 1, Qt::AlignCenter);
+					layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding, QSizePolicy::Expanding), 1, 2);
+					layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding, QSizePolicy::Expanding), 2, 1);
 				}
 			}
-		}
 
-		if (value.toBool() && (needsInitialization || !m_searchWidget))
-		{
-			if (!m_searchWidget)
-			{
-				m_searchWidget = new SearchWidget(m_window, this);
-				m_searchWidget->setFixedWidth(300);
-			}
+			break;
+		case SettingsManager::StartPage_TilesPerRowOption:
+		case SettingsManager::StartPage_TileHeightOption:
+		case SettingsManager::StartPage_TileWidthOption:
+		case SettingsManager::StartPage_ZoomLevelOption:
+			updateSize();
 
-			layout->addItem(new QSpacerItem(1, 50, QSizePolicy::Fixed, QSizePolicy::Fixed), 0, 1);
-			layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding), 1, 0);
-			layout->addWidget(m_searchWidget, 1, 1, 1, 1, Qt::AlignCenter);
-			layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding), 1, 2);
-			layout->addItem(new QSpacerItem(1, 50, QSizePolicy::Fixed, QSizePolicy::Fixed), 2, 1);
-			layout->addWidget(m_listView, 3, 0, 1, 3, Qt::AlignCenter);
-			layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding, QSizePolicy::Expanding), 4, 1);
-		}
-		else if (!value.toBool() && (needsInitialization || m_searchWidget))
-		{
-			if (m_searchWidget)
-			{
-				m_searchWidget->deleteLater();
-				m_searchWidget = NULL;
-			}
-
-			layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding, QSizePolicy::Expanding), 0, 1);
-			layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding, QSizePolicy::Expanding), 1, 0);
-			layout->addWidget(m_listView, 1, 1, 1, 1, Qt::AlignCenter);
-			layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding, QSizePolicy::Expanding), 1, 2);
-			layout->addItem(new QSpacerItem(1, 1, QSizePolicy::Expanding, QSizePolicy::Expanding), 2, 1);
-		}
+			break;
+		default:
+			break;
 	}
 }
 
@@ -377,8 +395,8 @@ void StartPageWidget::triggerAction(int identifier, const QVariantMap &parameter
 			return;
 	}
 
-	const QUrl url = m_currentIndex.data(BookmarksModel::UrlRole).toUrl();
-	MainWindow *mainWindow = MainWindow::findMainWindow(this);
+	const QUrl url(m_currentIndex.data(BookmarksModel::UrlRole).toUrl());
+	MainWindow *mainWindow(MainWindow::findMainWindow(this));
 
 	if (url.isValid() && mainWindow)
 	{
@@ -412,7 +430,7 @@ void StartPageWidget::addTile()
 
 void StartPageWidget::addTile(const QUrl &url)
 {
-	BookmarksItem *bookmark = BookmarksManager::getModel()->addBookmark(BookmarksModel::UrlBookmark, 0, url, QString(), BookmarksManager::getModel()->getItem(SettingsManager::getValue(QLatin1String("StartPage/BookmarksFolder")).toString()));
+	BookmarksItem *bookmark(BookmarksManager::getModel()->addBookmark(BookmarksModel::UrlBookmark, 0, url, QString(), BookmarksManager::getModel()->getItem(SettingsManager::getValue(SettingsManager::StartPage_BookmarksFolderOption).toString())));
 
 	if (bookmark)
 	{
@@ -429,12 +447,12 @@ void StartPageWidget::openTile()
 		return;
 	}
 
-	const BookmarksModel::BookmarkType type = static_cast<BookmarksModel::BookmarkType>(m_currentIndex.data(BookmarksModel::TypeRole).toInt());
+	const BookmarksModel::BookmarkType type(static_cast<BookmarksModel::BookmarkType>(m_currentIndex.data(BookmarksModel::TypeRole).toInt()));
 
 	if (type == BookmarksModel::FolderBookmark)
 	{
-		MainWindow *mainWindow = MainWindow::findMainWindow(this);
-		BookmarksItem *bookmark = BookmarksManager::getModel()->getBookmark(m_currentIndex);
+		MainWindow *mainWindow(MainWindow::findMainWindow(this));
+		BookmarksItem *bookmark(BookmarksManager::getModel()->getBookmark(m_currentIndex));
 
 		if (mainWindow && bookmark && bookmark->rowCount() > 0)
 		{
@@ -450,21 +468,17 @@ void StartPageWidget::openTile()
 	}
 
 	const QUrl url(m_currentIndex.data(BookmarksModel::UrlRole).toUrl());
+	WindowsManager *manager(SessionsManager::getWindowsManager());
 
-	if (url.isValid() && parentWidget() && parentWidget()->parentWidget())
+	if (manager && url.isValid())
 	{
-		WebContentsWidget *contentsWidget = qobject_cast<WebContentsWidget*>(parentWidget()->parentWidget());
-
-		if (contentsWidget)
-		{
-			contentsWidget->setUrl(url);
-		}
+		manager->open(url, WindowsManager::CurrentTabOpen);
 	}
 }
 
 void StartPageWidget::editTile()
 {
-	BookmarksItem *bookmark = BookmarksManager::getModel()->getBookmark(m_currentIndex);
+	BookmarksItem *bookmark(BookmarksManager::getModel()->getBookmark(m_currentIndex));
 
 	if (bookmark)
 	{
@@ -477,18 +491,18 @@ void StartPageWidget::editTile()
 
 void StartPageWidget::reloadTile()
 {
-	m_model->reloadTile(m_currentIndex);
-
 	m_listView->openPersistentEditor(m_currentIndex);
+
+	m_model->reloadTile(m_currentIndex);
 }
 
 void StartPageWidget::removeTile()
 {
-	BookmarksItem *bookmark = BookmarksManager::getModel()->getBookmark(m_currentIndex);
+	BookmarksItem *bookmark(BookmarksManager::getModel()->getBookmark(m_currentIndex));
 
 	if (bookmark)
 	{
-		const QString path = SessionsManager::getWritableDataPath(QLatin1String("thumbnails/")) + QString::number(bookmark->data(BookmarksModel::IdentifierRole).toULongLong()) + QLatin1String(".png");
+		const QString path(SessionsManager::getWritableDataPath(QLatin1String("thumbnails/")) + QString::number(bookmark->data(BookmarksModel::IdentifierRole).toULongLong()) + QLatin1String(".png"));
 
 		if (QFile::exists(path))
 		{
@@ -507,12 +521,12 @@ void StartPageWidget::updateTile(const QModelIndex &index)
 
 void StartPageWidget::updateSize()
 {
-	const qreal zoom = (SettingsManager::getValue(QLatin1String("StartPage/ZoomLevel")).toInt() / qreal(100));
-	const int tileHeight = (SettingsManager::getValue(QLatin1String("StartPage/TileHeight")).toInt() * zoom);
-	const int tileWidth = (SettingsManager::getValue(QLatin1String("StartPage/TileWidth")).toInt() * zoom);
-	const int amount = m_model->rowCount();
-	const int columns = getTilesPerRow();
-	const int rows = qCeil(amount / qreal(columns));
+	const qreal zoom(SettingsManager::getValue(SettingsManager::StartPage_ZoomLevelOption).toInt() / qreal(100));
+	const int tileHeight(SettingsManager::getValue(SettingsManager::StartPage_TileHeightOption).toInt() * zoom);
+	const int tileWidth(SettingsManager::getValue(SettingsManager::StartPage_TileWidthOption).toInt() * zoom);
+	const int amount(m_model->rowCount());
+	const int columns(getTilesPerRow());
+	const int rows(qCeil(amount / qreal(columns)));
 
 	m_listView->setGridSize(QSize(tileWidth, tileHeight));
 	m_listView->setFixedSize(((qMin(amount, columns) * tileWidth) + 2), ((rows * tileHeight) + 20));
@@ -522,7 +536,7 @@ void StartPageWidget::updateTiles()
 {
 	for (int i = 0; i < m_model->rowCount(); ++i)
 	{
-		if (m_model->item(i) && m_model->isReloadingTile(m_model->index(i, 0)))
+		if (m_model->index(i, 0).data(StartPageModel::IsReloadingRole).toBool())
 		{
 			m_listView->openPersistentEditor(m_model->index(i, 0));
 		}
@@ -533,11 +547,11 @@ void StartPageWidget::updateTiles()
 
 void StartPageWidget::showContextMenu(const QPoint &position)
 {
-	QPoint hitPosition = position;
+	QPoint hitPosition(position);
 
 	if (hitPosition.isNull())
 	{
-		WebWidget *webWidget = qobject_cast<WebWidget*>(parentWidget());
+		WebWidget *webWidget(qobject_cast<WebWidget*>(parentWidget()));
 
 		if (webWidget && !webWidget->getClickPosition().isNull())
 		{
@@ -551,7 +565,7 @@ void StartPageWidget::showContextMenu(const QPoint &position)
 	}
 
 	const QModelIndex index(m_listView->indexAt(m_listView->mapFromGlobal(hitPosition)));
-	QMenu menu(this);
+	QMenu menu;
 
 	if (index.isValid() && index.data(Qt::AccessibleDescriptionRole).toString() != QLatin1String("add"))
 	{
@@ -559,7 +573,7 @@ void StartPageWidget::showContextMenu(const QPoint &position)
 		menu.addSeparator();
 		menu.addAction(tr("Edit…"), this, SLOT(editTile()));
 
-		if (SettingsManager::getValue(QLatin1String("StartPage/TileBackgroundMode")) == QLatin1String("thumbnail"))
+		if (SettingsManager::getValue(SettingsManager::StartPage_TileBackgroundModeOption) == QLatin1String("thumbnail"))
 		{
 			menu.addAction(tr("Reload"), this, SLOT(reloadTile()))->setEnabled(static_cast<BookmarksModel::BookmarkType>(index.data(BookmarksModel::TypeRole).toInt()) == BookmarksModel::UrlBookmark);
 		}
@@ -578,23 +592,23 @@ void StartPageWidget::showContextMenu(const QPoint &position)
 
 int StartPageWidget::getTilesPerRow() const
 {
-	const int tilesPerRow = SettingsManager::getValue(QLatin1String("StartPage/TilesPerRow")).toInt();
+	const int tilesPerRow(SettingsManager::getValue(SettingsManager::StartPage_TilesPerRowOption).toInt());
 
 	if (tilesPerRow > 0)
 	{
 		return tilesPerRow;
 	}
 
-	return qMax(1, int((width() - 50) / (SettingsManager::getValue(QLatin1String("StartPage/TileWidth")).toInt() * (SettingsManager::getValue(QLatin1String("StartPage/ZoomLevel")).toInt() / qreal(100)))));
+	return qMax(1, int((width() - 50) / (SettingsManager::getValue(SettingsManager::StartPage_TileWidthOption).toInt() * (SettingsManager::getValue(SettingsManager::StartPage_ZoomLevelOption).toInt() / qreal(100)))));
 }
 
 bool StartPageWidget::eventFilter(QObject *object, QEvent *event)
 {
-	if (event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonDblClick || event->type() == QEvent::Wheel)
+	if ((object == this || object == m_listView || object == m_listView->viewport()) && (event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonDblClick || event->type() == QEvent::Wheel))
 	{
 		if (event->type() == QEvent::Wheel)
 		{
-			QWheelEvent *wheelEvent = static_cast<QWheelEvent*>(event);
+			QWheelEvent *wheelEvent(static_cast<QWheelEvent*>(event));
 
 			if (wheelEvent)
 			{
@@ -603,7 +617,7 @@ bool StartPageWidget::eventFilter(QObject *object, QEvent *event)
 		}
 		else
 		{
-			QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
+			QMouseEvent *mouseEvent(static_cast<QMouseEvent*>(event));
 
 			if (mouseEvent)
 			{
@@ -615,10 +629,10 @@ bool StartPageWidget::eventFilter(QObject *object, QEvent *event)
 
 		if (m_currentIndex.isValid())
 		{
-			contexts << GesturesManager::LinkGesturesContext;
+			contexts.append(GesturesManager::LinkGesturesContext);
 		}
 
-		contexts << GesturesManager::GenericGesturesContext;
+		contexts.append(GesturesManager::GenericGesturesContext);
 
 		if (GesturesManager::startGesture(object, event, contexts))
 		{
@@ -626,9 +640,9 @@ bool StartPageWidget::eventFilter(QObject *object, QEvent *event)
 		}
 	}
 
-	if (event->type() == QEvent::KeyRelease && object == m_listView)
+	if (object == m_listView && event->type() == QEvent::KeyRelease)
 	{
-		QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
+		QKeyEvent *keyEvent(static_cast<QKeyEvent*>(event));
 
 		if (keyEvent)
 		{
@@ -646,11 +660,11 @@ bool StartPageWidget::eventFilter(QObject *object, QEvent *event)
 
 			if (keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return)
 			{
-				const BookmarksModel::BookmarkType type = static_cast<BookmarksModel::BookmarkType>(m_currentIndex.data(BookmarksModel::TypeRole).toInt());
+				const BookmarksModel::BookmarkType type(static_cast<BookmarksModel::BookmarkType>(m_currentIndex.data(BookmarksModel::TypeRole).toInt()));
 
 				if (type == BookmarksModel::FolderBookmark)
 				{
-					BookmarksItem *bookmark = BookmarksManager::getModel()->getBookmark(m_currentIndex);
+					BookmarksItem *bookmark(BookmarksManager::getModel()->getBookmark(m_currentIndex));
 
 					if (bookmark && bookmark->rowCount() > 0)
 					{
@@ -667,16 +681,16 @@ bool StartPageWidget::eventFilter(QObject *object, QEvent *event)
 
 					if (keyEvent->modifiers() != Qt::NoModifier)
 					{
-						MainWindow *mainWindow = MainWindow::findMainWindow(this);
+						MainWindow *mainWindow(MainWindow::findMainWindow(this));
 
 						if (mainWindow)
 						{
-							mainWindow->getWindowsManager()->open(url, WindowsManager::calculateOpenHints(keyEvent->modifiers(), Qt::LeftButton));
+							mainWindow->getWindowsManager()->open(url, WindowsManager::calculateOpenHints(WindowsManager::DefaultOpen, Qt::LeftButton, keyEvent->modifiers()));
 						}
 					}
 					else if (parentWidget() && parentWidget()->parentWidget())
 					{
-						WebContentsWidget *contentsWidget = qobject_cast<WebContentsWidget*>(parentWidget()->parentWidget());
+						WebContentsWidget *contentsWidget(qobject_cast<WebContentsWidget*>(parentWidget()->parentWidget()));
 
 						if (contentsWidget)
 						{
@@ -693,9 +707,9 @@ bool StartPageWidget::eventFilter(QObject *object, QEvent *event)
 			}
 		}
 	}
-	else if (event->type() == QEvent::MouseButtonRelease && object == m_listView->viewport())
+	else if (object == m_listView->viewport() && event->type() == QEvent::MouseButtonRelease)
 	{
-		QMouseEvent *mouseEvent = static_cast<QMouseEvent*>(event);
+		QMouseEvent *mouseEvent(static_cast<QMouseEvent*>(event));
 
 		if (mouseEvent)
 		{
@@ -708,11 +722,11 @@ bool StartPageWidget::eventFilter(QObject *object, QEvent *event)
 
 			if (mouseEvent->button() == Qt::LeftButton || mouseEvent->button() == Qt::MiddleButton)
 			{
-				const BookmarksModel::BookmarkType type = static_cast<BookmarksModel::BookmarkType>(m_currentIndex.data(BookmarksModel::TypeRole).toInt());
+				const BookmarksModel::BookmarkType type(static_cast<BookmarksModel::BookmarkType>(m_currentIndex.data(BookmarksModel::TypeRole).toInt()));
 
 				if (type == BookmarksModel::FolderBookmark)
 				{
-					BookmarksItem *bookmark = BookmarksManager::getModel()->getBookmark(m_currentIndex);
+					BookmarksItem *bookmark(BookmarksManager::getModel()->getBookmark(m_currentIndex));
 
 					if (bookmark && bookmark->rowCount() > 0)
 					{
@@ -746,20 +760,20 @@ bool StartPageWidget::eventFilter(QObject *object, QEvent *event)
 				{
 					if ((mouseEvent->button() == Qt::LeftButton && mouseEvent->modifiers() != Qt::NoModifier))
 					{
-						MainWindow *mainWindow = MainWindow::findMainWindow(this);
+						MainWindow *mainWindow(MainWindow::findMainWindow(this));
 
 						if (mainWindow)
 						{
-							mainWindow->getWindowsManager()->open(url, WindowsManager::calculateOpenHints(mouseEvent->modifiers(), mouseEvent->button()));
+							mainWindow->getWindowsManager()->open(url, WindowsManager::calculateOpenHints(WindowsManager::DefaultOpen, mouseEvent->button(), mouseEvent->modifiers()));
 						}
 					}
 					else if (parentWidget() && parentWidget()->parentWidget() && mouseEvent->button() != Qt::MiddleButton)
 					{
-						WebContentsWidget *contentsWidget = qobject_cast<WebContentsWidget*>(parentWidget()->parentWidget());
+						WindowsManager *manager(SessionsManager::getWindowsManager());
 
-						if (contentsWidget)
+						if (manager && url.isValid())
 						{
-							contentsWidget->setUrl(url);
+							manager->open(url, WindowsManager::CurrentTabOpen);
 						}
 					}
 				}
