@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -35,7 +35,7 @@ class LocaleDialog : public Dialog
 	Q_OBJECT
 
 public:
-	explicit LocaleDialog(QWidget *parent = NULL);
+	explicit LocaleDialog(QWidget *parent = nullptr);
 	~LocaleDialog();
 
 protected:
