@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 * Copyright (C) 2014 - 2015 Piotr Wójcik <chocimier@tlen.pl>
 * Copyright (C) 2015 Jan Bajer aka bajasoft <jbajer@gmail.com>
 *
@@ -41,119 +41,137 @@
 #include "UpdateCheckerDialog.h"
 #include "Window.h"
 #include "preferences/ContentBlockingDialog.h"
-#include "toolbars/ActionWidget.h"
 #include "../core/ActionsManager.h"
 #include "../core/AddonsManager.h"
-#include "../core/Application.h"
 #include "../core/BookmarksManager.h"
 #include "../core/HistoryManager.h"
 #include "../core/InputInterpreter.h"
 #include "../core/SettingsManager.h"
+#include "../core/ThemesManager.h"
 #include "../core/TransfersManager.h"
 #include "../core/Utils.h"
 #include "../core/WebBackend.h"
+#include "../modules/widgets/action/ActionWidget.h"
 
 #include "ui_MainWindow.h"
 
 #include <QtGui/QCloseEvent>
-#include <QtWidgets/QFileDialog>
 #include <QtWidgets/QMessageBox>
 #include <QtWidgets/QVBoxLayout>
 
 namespace Otter
 {
 
-MainWindow::MainWindow(bool isPrivate, const SessionMainWindow &session, QWidget *parent) : QMainWindow(parent),
-	m_windowsManager(NULL),
-	m_tabSwitcher(NULL),
+MainWindow::MainWindow(Application::MainWindowFlags flags, const SessionMainWindow &session, QWidget *parent) : QMainWindow(parent),
+	m_windowsManager(nullptr),
+	m_tabSwitcher(nullptr),
 	m_workspace(new WorkspaceWidget(this)),
-	m_topToolBarArea(NULL),
-	m_bottomToolBarArea(NULL),
-	m_leftToolBarArea(NULL),
-	m_rightToolBarArea(NULL),
-	m_tabBar(NULL),
-	m_menuBar(NULL),
-	m_statusBar(NULL),
-	m_sidebarToggle(NULL),
-	m_sidebar(NULL),
+	m_topToolBarArea(nullptr),
+	m_bottomToolBarArea(nullptr),
+	m_leftToolBarArea(nullptr),
+	m_rightToolBarArea(nullptr),
+	m_tabBar(nullptr),
+	m_menuBar(nullptr),
+	m_statusBar(nullptr),
+	m_sidebarToggle(nullptr),
+	m_sidebar(nullptr),
 	m_splitter(new QSplitter(this)),
-	m_currentWindow(NULL),
+	m_currentWindow(nullptr),
+	m_mouseTrackerTimer(0),
 	m_tabSwitcherTimer(0),
+	m_hasToolBars(!flags.testFlag(Application::NoToolBarsFlag)),
 	m_ui(new Ui::MainWindow)
 {
 	m_ui->setupUi(this);
 
-	m_standardActions.fill(NULL, ActionsManager::getActionDefinitions().count());
+	setUnifiedTitleAndToolBarOnMac(true);
+
+	m_standardActions.fill(nullptr, ActionsManager::getActionDefinitions().count());
 
 	updateShortcuts();
 
 	SessionsManager::setActiveWindow(this);
 
-	m_windowsManager = new WindowsManager((isPrivate || SessionsManager::isPrivate() || SettingsManager::getValue(QLatin1String("Browser/PrivateMode")).toBool()), this);
-	m_topToolBarArea = new ToolBarAreaWidget(Qt::TopToolBarArea, this);
-	m_bottomToolBarArea = new ToolBarAreaWidget(Qt::BottomToolBarArea, this);
-	m_leftToolBarArea = new ToolBarAreaWidget(Qt::LeftToolBarArea, this);
-	m_rightToolBarArea = new ToolBarAreaWidget(Qt::RightToolBarArea, this);
+	m_windowsManager = new WindowsManager((flags.testFlag(Application::PrivateFlag) || SessionsManager::isPrivate() || SettingsManager::getValue(SettingsManager::Browser_PrivateModeOption).toBool()), this);
 
 	m_workspace->updateActions();
 
-	m_splitter->setChildrenCollapsible(false);
-	m_splitter->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
-	m_splitter->addWidget(m_workspace);
+	if (m_hasToolBars)
+	{
+		m_topToolBarArea = new ToolBarAreaWidget(Qt::TopToolBarArea, this);
+		m_bottomToolBarArea = new ToolBarAreaWidget(Qt::BottomToolBarArea, this);
+		m_leftToolBarArea = new ToolBarAreaWidget(Qt::LeftToolBarArea, this);
+		m_rightToolBarArea = new ToolBarAreaWidget(Qt::RightToolBarArea, this);
 
-	QWidget *verticalWidget = new QWidget(this);
-	QWidget *horizontalWidget = new QWidget(verticalWidget);
-	QHBoxLayout *horizontalLayout = new QHBoxLayout(horizontalWidget);
-	horizontalLayout->setContentsMargins(0, 0, 0, 0);
-	horizontalLayout->setSpacing(0);
-	horizontalLayout->addWidget(m_leftToolBarArea);
-	horizontalLayout->addWidget(m_splitter);
-	horizontalLayout->addWidget(m_rightToolBarArea);
+		m_splitter->setChildrenCollapsible(false);
+		m_splitter->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+		m_splitter->addWidget(m_workspace);
 
-	QVBoxLayout *verticalLayout = new QVBoxLayout(verticalWidget);
-	verticalLayout->setContentsMargins(0, 0, 0, 0);
-	verticalLayout->setSpacing(0);
-	verticalLayout->addWidget(m_topToolBarArea);
-	verticalLayout->addWidget(horizontalWidget);
-	verticalLayout->addWidget(m_bottomToolBarArea);
+		QWidget *verticalWidget(new QWidget(this));
+		QWidget *horizontalWidget(new QWidget(verticalWidget));
+		QHBoxLayout *horizontalLayout(new QHBoxLayout(horizontalWidget));
+		horizontalLayout->setContentsMargins(0, 0, 0, 0);
+		horizontalLayout->setSpacing(0);
 
-	setCentralWidget(verticalWidget);
+		horizontalLayout->addWidget(m_leftToolBarArea);
+		horizontalLayout->addWidget(m_splitter);
+		horizontalLayout->addWidget(m_rightToolBarArea);
+
+		QVBoxLayout *verticalLayout(new QVBoxLayout(verticalWidget));
+		verticalLayout->setContentsMargins(0, 0, 0, 0);
+		verticalLayout->setSpacing(0);
+		verticalLayout->addWidget(m_topToolBarArea);
+		verticalLayout->addWidget(horizontalWidget);
+		verticalLayout->addWidget(m_bottomToolBarArea);
+
+		setCentralWidget(verticalWidget);
+	}
+	else
+	{
+		m_tabBar = new TabBarWidget(this);
+		m_tabBar->hide();
+
+		m_splitter->setChildrenCollapsible(false);
+		m_splitter->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
+		m_splitter->addWidget(m_workspace);
+
+		setCentralWidget(m_splitter);
+	}
 
 	SessionsManager::registerWindow(this);
 
-	getAction(ActionsManager::WorkOfflineAction)->setChecked(SettingsManager::getValue(QLatin1String("Network/WorkOffline")).toBool());
-	getAction(ActionsManager::ShowMenuBarAction)->setChecked(ToolBarsManager::getToolBarDefinition(ToolBarsManager::MenuBar).visibility != ToolBarsManager::AlwaysHiddenToolBar);
-	getAction(ActionsManager::ShowSidebarAction)->setChecked(SettingsManager::getValue(QLatin1String("Sidebar/Visible")).toBool());
+	getAction(ActionsManager::WorkOfflineAction)->setChecked(SettingsManager::getValue(SettingsManager::Network_WorkOfflineOption).toBool());
+	getAction(ActionsManager::ShowMenuBarAction)->setChecked(ToolBarsManager::getToolBarDefinition(ToolBarsManager::MenuBar).normalVisibility != ToolBarsManager::AlwaysHiddenToolBar);
+	getAction(ActionsManager::ShowSidebarAction)->setChecked(SettingsManager::getValue(SettingsManager::Sidebar_VisibleOption).toBool());
 	getAction(ActionsManager::LockToolBarsAction)->setChecked(ToolBarsManager::areToolBarsLocked());
 	getAction(ActionsManager::ExitAction)->setMenuRole(QAction::QuitRole);
 	getAction(ActionsManager::PreferencesAction)->setMenuRole(QAction::PreferencesRole);
 	getAction(ActionsManager::AboutQtAction)->setMenuRole(QAction::AboutQtRole);
 	getAction(ActionsManager::AboutApplicationAction)->setMenuRole(QAction::AboutRole);
 
-	if (ToolBarsManager::getToolBarDefinition(ToolBarsManager::MenuBar).visibility != ToolBarsManager::AlwaysHiddenToolBar)
+	if (ToolBarsManager::getToolBarDefinition(ToolBarsManager::MenuBar).normalVisibility != ToolBarsManager::AlwaysHiddenToolBar)
 	{
 		m_menuBar = new MenuBarWidget(this);
 
 		setMenuBar(m_menuBar);
 	}
 
-	if (ToolBarsManager::getToolBarDefinition(ToolBarsManager::StatusBar).visibility != ToolBarsManager::AlwaysHiddenToolBar)
+	if (ToolBarsManager::getToolBarDefinition(ToolBarsManager::StatusBar).normalVisibility != ToolBarsManager::AlwaysHiddenToolBar)
 	{
 		m_statusBar = new StatusBarWidget(this);
 
 		setStatusBar(m_statusBar);
 	}
 
-	optionChanged(QLatin1String("Sidebar/ShowToggleEdge"), SettingsManager::getValue(QLatin1String("Sidebar/ShowToggleEdge")));
-	optionChanged(QLatin1String("Sidebar/Visible"), SettingsManager::getValue(QLatin1String("Sidebar/Visible")));
+	optionChanged(SettingsManager::Sidebar_ShowToggleEdgeOption, SettingsManager::getValue(SettingsManager::Sidebar_ShowToggleEdgeOption));
+	optionChanged(SettingsManager::Sidebar_VisibleOption, SettingsManager::getValue(SettingsManager::Sidebar_VisibleOption));
 
 	connect(ActionsManager::getInstance(), SIGNAL(shortcutsChanged()), this, SLOT(updateShortcuts()));
-	connect(SettingsManager::getInstance(), SIGNAL(valueChanged(QString,QVariant)), this, SLOT(optionChanged(QString,QVariant)));
+	connect(SettingsManager::getInstance(), SIGNAL(valueChanged(int,QVariant)), this, SLOT(optionChanged(int,QVariant)));
 	connect(ToolBarsManager::getInstance(), SIGNAL(toolBarModified(int)), this, SLOT(toolBarModified(int)));
 	connect(TransfersManager::getInstance(), SIGNAL(transferStarted(Transfer*)), this, SLOT(transferStarted()));
 	connect(m_windowsManager, SIGNAL(requestedAddBookmark(QUrl,QString,QString)), this, SLOT(addBookmark(QUrl,QString,QString)));
 	connect(m_windowsManager, SIGNAL(requestedEditBookmark(QUrl)), this, SLOT(editBookmark(QUrl)));
-	connect(m_windowsManager, SIGNAL(requestedNewWindow(bool,bool,QUrl)), this, SIGNAL(requestedNewWindow(bool,bool,QUrl)));
 	connect(m_windowsManager, SIGNAL(windowTitleChanged(QString)), this, SLOT(updateWindowTitle(QString)));
 	connect(m_ui->consoleDockWidget, SIGNAL(visibilityChanged(bool)), getAction(ActionsManager::ShowErrorConsoleAction), SLOT(setChecked(bool)));
 
@@ -167,7 +185,7 @@ MainWindow::MainWindow(bool isPrivate, const SessionMainWindow &session, QWidget
 	{
 		restoreGeometry(session.geometry);
 	}
-	else if (SettingsManager::getValue(QLatin1String("Interface/MaximizeNewWindows")).toBool())
+	else if (SettingsManager::getValue(SettingsManager::Interface_MaximizeNewWindowsOption).toBool())
 	{
 		showMaximized();
 	}
@@ -180,7 +198,35 @@ MainWindow::~MainWindow()
 
 void MainWindow::timerEvent(QTimerEvent *event)
 {
-	if (event->timerId() == m_tabSwitcherTimer)
+	if (event->timerId() == m_mouseTrackerTimer)
+	{
+		Qt::ToolBarAreas areas(Qt::NoToolBarArea);
+		const QPoint position(mapFromGlobal(QCursor::pos()));
+
+		if (position.x() < 10)
+		{
+			areas |= (isLeftToRight() ? Qt::LeftToolBarArea : Qt::RightToolBarArea);
+		}
+		else if (position.x() > (contentsRect().width() - 10))
+		{
+			areas |= (isLeftToRight() ? Qt::RightToolBarArea : Qt::LeftToolBarArea);
+		}
+
+		if (position.y() < 10)
+		{
+			areas |= Qt::TopToolBarArea;
+		}
+		else if (position.y() > (contentsRect().height() - 10))
+		{
+			areas |= Qt::BottomToolBarArea;
+		}
+
+		if (areas != Qt::NoToolBarArea)
+		{
+			emit requestedToolBarsActivation(areas);
+		}
+	}
+	else if (event->timerId() == m_tabSwitcherTimer)
 	{
 		killTimer(m_tabSwitcherTimer);
 
@@ -209,9 +255,7 @@ void MainWindow::closeEvent(QCloseEvent *event)
 		return;
 	}
 
-	Application *application = Application::getInstance();
-
-	if (application->getWindows().count() == 1)
+	if (Application::getInstance()->getWindows().count() == 1)
 	{
 		if (SessionsManager::getCurrentSession() == QLatin1String("default"))
 		{
@@ -225,7 +269,7 @@ void MainWindow::closeEvent(QCloseEvent *event)
 
 	m_windowsManager->closeAll();
 
-	application->removeWindow(this);
+	Application::removeWindow(this);
 
 	event->accept();
 }
@@ -308,39 +352,40 @@ void MainWindow::mouseReleaseEvent(QMouseEvent *event)
 	QMainWindow::mouseReleaseEvent(event);
 }
 
-void MainWindow::optionChanged(const QString &option, const QVariant &value)
+void MainWindow::optionChanged(int identifier, const QVariant &value)
 {
-	if (option == QLatin1String("Network/WorkOffline"))
+	if (identifier == SettingsManager::Network_WorkOfflineOption)
 	{
 		getAction(ActionsManager::WorkOfflineAction)->setChecked(value.toBool());
 	}
-	else if (option == QLatin1String("Interface/LockToolBars"))
+	else if (identifier == SettingsManager::Interface_LockToolBarsOption)
 	{
 		getAction(ActionsManager::LockToolBarsAction)->setChecked(value.toBool());
 	}
-	else if (option == QLatin1String("Sidebar/CurrentPanel") || option == QLatin1String("Sidebar/Reverse"))
+	else if (identifier == SettingsManager::Sidebar_CurrentPanelOption || identifier == SettingsManager::Sidebar_ReverseOption)
 	{
 		placeSidebars();
 	}
-	else if (option == QLatin1String("Sidebar/ShowToggleEdge"))
+	else if (identifier == SettingsManager::Sidebar_ShowToggleEdgeOption)
 	{
 		if (m_sidebarToggle && !value.toBool())
 		{
 			m_sidebarToggle->deleteLater();
-			m_sidebarToggle = NULL;
+			m_sidebarToggle = nullptr;
 
 			placeSidebars();
 		}
 		else if (!m_sidebarToggle && value.toBool())
 		{
-			m_sidebarToggle = new ActionWidget(ActionsManager::ShowSidebarAction, NULL, ActionsManager::ActionEntryDefinition(), this);
+			m_sidebarToggle = new ActionWidget(ActionsManager::ShowSidebarAction, nullptr, ActionsManager::ActionEntryDefinition(), this);
 			m_sidebarToggle->setFixedWidth(6);
 			m_sidebarToggle->setText(QString());
+			m_sidebarToggle->setVisible(!areControlsHidden());
 
 			placeSidebars();
 		}
 	}
-	else if (option == QLatin1String("Sidebar/Visible"))
+	else if (identifier == SettingsManager::Sidebar_VisibleOption)
 	{
 		if (m_sidebar && !value.toBool())
 		{
@@ -370,20 +415,20 @@ void MainWindow::triggerAction(int identifier, const QVariantMap &parameters)
 	switch (identifier)
 	{
 		case ActionsManager::NewWindowAction:
-			emit requestedNewWindow(false, false, QUrl());
+			emit Application::openWindow();
 
 			break;
 		case ActionsManager::NewWindowPrivateAction:
-			emit requestedNewWindow(true, false, QUrl());
+			emit Application::openWindow(true);
 
 			break;
 		case ActionsManager::OpenAction:
 			{
-				const QUrl url = QFileDialog::getOpenFileUrl(this, tr("Open File"));
+				const QString path(Utils::getOpenPaths().value(0));
 
-				if (url.isValid())
+				if (!path.isEmpty())
 				{
-					m_windowsManager->open(url);
+					m_windowsManager->open(QUrl::fromLocalFile(path));
 				}
 			}
 
@@ -395,7 +440,7 @@ void MainWindow::triggerAction(int identifier, const QVariantMap &parameters)
 			}
 			else
 			{
-				const QList<MainWindow*> windows = SessionsManager::getWindows();
+				const QList<MainWindow*> windows(SessionsManager::getWindows());
 
 				for (int i = 0; i < windows.count(); ++i)
 				{
@@ -455,7 +500,7 @@ void MainWindow::triggerAction(int identifier, const QVariantMap &parameters)
 			break;
 		case ActionsManager::GoToHomePageAction:
 			{
-				const QString homePage = SettingsManager::getValue(QLatin1String("Browser/HomePage")).toString();
+				const QString homePage(SettingsManager::getValue(SettingsManager::Browser_HomePageOption).toString());
 
 				if (!homePage.isEmpty())
 				{
@@ -490,7 +535,7 @@ void MainWindow::triggerAction(int identifier, const QVariantMap &parameters)
 
 			break;
 		case ActionsManager::WorkOfflineAction:
-			SettingsManager::setValue(QLatin1String("Network/WorkOffline"), parameters.value(QLatin1String("isChecked")).toBool());
+			SettingsManager::setValue(SettingsManager::Network_WorkOfflineOption, Action::calculateCheckedState(parameters, getAction(ActionsManager::WorkOfflineAction)));
 
 			break;
 		case ActionsManager::FullScreenAction:
@@ -504,6 +549,8 @@ void MainWindow::triggerAction(int identifier, const QVariantMap &parameters)
 				showFullScreen();
 			}
 
+			m_windowsManager->triggerAction(identifier, parameters);
+
 			break;
 		case ActionsManager::ShowTabSwitcherAction:
 			if (!m_tabSwitcher)
@@ -518,8 +565,8 @@ void MainWindow::triggerAction(int identifier, const QVariantMap &parameters)
 			break;
 		case ActionsManager::ShowMenuBarAction:
 			{
-				ToolBarsManager::ToolBarDefinition definition = ToolBarsManager::getToolBarDefinition(ToolBarsManager::MenuBar);
-				definition.visibility = (parameters.value(QLatin1String("isChecked")).toBool() ? ToolBarsManager::AlwaysVisibleToolBar : ToolBarsManager::AlwaysHiddenToolBar);
+				ToolBarsManager::ToolBarDefinition definition(ToolBarsManager::getToolBarDefinition(ToolBarsManager::MenuBar));
+				definition.normalVisibility = (Action::calculateCheckedState(parameters, getAction(ActionsManager::ShowMenuBarAction)) ? ToolBarsManager::AlwaysVisibleToolBar : ToolBarsManager::AlwaysHiddenToolBar);
 
 				ToolBarsManager::setToolBar(definition);
 			}
@@ -527,34 +574,43 @@ void MainWindow::triggerAction(int identifier, const QVariantMap &parameters)
 			break;
 		case ActionsManager::ShowTabBarAction:
 			{
-				ToolBarsManager::ToolBarDefinition definition = ToolBarsManager::getToolBarDefinition(ToolBarsManager::TabBar);
-				definition.visibility = (parameters.value(QLatin1String("isChecked")).toBool() ? ToolBarsManager::AlwaysVisibleToolBar : ToolBarsManager::AlwaysHiddenToolBar);
+				ToolBarsManager::ToolBarDefinition definition(ToolBarsManager::getToolBarDefinition(ToolBarsManager::TabBar));
+				definition.normalVisibility = (Action::calculateCheckedState(parameters, getAction(ActionsManager::ShowTabBarAction)) ? ToolBarsManager::AlwaysVisibleToolBar : ToolBarsManager::AlwaysHiddenToolBar);
 
 				ToolBarsManager::setToolBar(definition);
 			}
 
 			break;
 		case ActionsManager::ShowSidebarAction:
-			SettingsManager::setValue(QLatin1String("Sidebar/Visible"), parameters.value(QLatin1String("isChecked")).toBool());
+			SettingsManager::setValue(SettingsManager::Sidebar_VisibleOption, Action::calculateCheckedState(parameters, getAction(ActionsManager::ShowSidebarAction)));
 
 			break;
 		case ActionsManager::OpenPanelAction:
-			if (m_sidebar && m_sidebar->getCurrentPanel())
+			if (m_sidebar)
 			{
-				m_windowsManager->open(m_sidebar->getCurrentPanel()->getUrl(), WindowsManager::NewTabOpen);
+				ContentsWidget *widget(qobject_cast<ContentsWidget*>(m_sidebar->getCurrentPanel()));
+
+				if (widget)
+				{
+					m_windowsManager->open(widget->getUrl(), WindowsManager::NewTabOpen);
+				}
 			}
 
 			break;
 		case ActionsManager::ClosePanelAction:
-			SettingsManager::setValue(QLatin1String("Sidebar/CurrentPanel"), QString());
+			SettingsManager::setValue(SettingsManager::Sidebar_CurrentPanelOption, QString());
 
 			break;
 		case ActionsManager::ShowErrorConsoleAction:
-			m_ui->consoleDockWidget->setVisible(parameters.value(QLatin1String("isChecked")).toBool());
+			m_ui->consoleDockWidget->setVisible(Action::calculateCheckedState(parameters, getAction(ActionsManager::ShowErrorConsoleAction)));
 
 			break;
 		case ActionsManager::LockToolBarsAction:
-			ToolBarsManager::setToolBarsLocked(parameters.value(QLatin1String("isChecked")).toBool());
+			ToolBarsManager::setToolBarsLocked(Action::calculateCheckedState(parameters, getAction(ActionsManager::LockToolBarsAction)));
+
+			break;
+		case ActionsManager::ResetToolBarsAction:
+			ToolBarsManager::resetToolBars();
 
 			break;
 		case ActionsManager::ContentBlockingAction:
@@ -577,10 +633,21 @@ void MainWindow::triggerAction(int identifier, const QVariantMap &parameters)
 			break;
 		case ActionsManager::ClearHistoryAction:
 			{
-				ClearHistoryDialog dialog(SettingsManager::getValue(QLatin1String("History/ManualClearOptions")).toStringList(), false, this);
+				ClearHistoryDialog dialog(SettingsManager::getValue(SettingsManager::History_ManualClearOptionsOption).toStringList(), false, this);
 				dialog.exec();
 			}
 
+			break;
+		case ActionsManager::AddonsAction:
+			{
+				const QUrl url(QLatin1String("about:addons"));
+
+				if (!SessionsManager::hasUrl(url, true))
+				{
+					m_windowsManager->open(url);
+				}
+			}
+
 			break;
 		case ActionsManager::NotesAction:
 			{
@@ -592,6 +659,17 @@ void MainWindow::triggerAction(int identifier, const QVariantMap &parameters)
 				}
 			}
 
+			break;
+		case ActionsManager::PasswordsAction:
+			{
+				const QUrl url(QLatin1String("about:passwords"));
+
+				if (!SessionsManager::hasUrl(url, true))
+				{
+					m_windowsManager->open(url);
+				}
+			}
+
 			break;
 		case ActionsManager::TransfersAction:
 			{
@@ -631,31 +709,37 @@ void MainWindow::triggerAction(int identifier, const QVariantMap &parameters)
 			break;
 		case ActionsManager::CheckForUpdatesAction:
 			{
-				UpdateCheckerDialog *dialog = new UpdateCheckerDialog(this);
+				UpdateCheckerDialog *dialog(new UpdateCheckerDialog(this));
 				dialog->show();
 			}
 
 			break;
 		case ActionsManager::DiagnosticReportAction:
 			{
-				ReportDialog *dialog = new ReportDialog(this);
+				ReportDialog *dialog(new ReportDialog(Application::FullReport, this));
 				dialog->show();
 			}
 
 			break;
 		case ActionsManager::AboutApplicationAction:
 			{
-				WebBackend *backend = AddonsManager::getWebBackend();
-				QString about = tr("<b>Otter %1</b><br>Web browser controlled by the user, not vice-versa.").arg(Application::getInstance()->getFullVersion());
-				about.append(QLatin1String("<br><br>") + tr("Web backend: %1 %2.").arg(backend->getTitle()).arg(backend->getEngineVersion()) + QLatin1String("<br><br>"));
+				WebBackend *webBackend(AddonsManager::getWebBackend());
+				QString about = tr("<b>Otter %1</b><br>Web browser controlled by the user, not vice-versa.<br><a href=\"https://www.otter-browser.org/\">https://www.otter-browser.org/</a>").arg(Application::getFullVersion());
 
-				if (QSslSocket::supportsSsl())
+				if (webBackend)
 				{
-					 about.append(tr("SSL library version: %1.").arg(QSslSocket::sslLibraryVersionString()));
-				}
-				else
-				{
-					about.append(tr("SSL library not available."));
+					const QString sslVersion(webBackend->getSslVersion());
+
+					about.append(QLatin1String("<br><br>") + tr("Web backend: %1 %2.").arg(webBackend->getTitle()).arg(webBackend->getEngineVersion()) + QLatin1String("<br><br>"));
+
+					if (sslVersion.isEmpty())
+					{
+						about.append(tr("SSL library not available."));
+					}
+					else
+					{
+						about.append(tr("SSL library version: %1.").arg(sslVersion));
+					}
 				}
 
 				QMessageBox::about(this, QLatin1String("Otter"), about);
@@ -679,7 +763,7 @@ void MainWindow::triggerAction(int identifier, const QVariantMap &parameters)
 
 void MainWindow::triggerAction()
 {
-	QShortcut *shortcut = qobject_cast<QShortcut*>(sender());
+	QShortcut *shortcut(qobject_cast<QShortcut*>(sender()));
 
 	if (shortcut)
 	{
@@ -687,13 +771,13 @@ void MainWindow::triggerAction()
 		{
 			if (m_actionShortcuts[i].second.contains(shortcut))
 			{
-				const ActionsManager::ActionDefinition definition = ActionsManager::getActionDefinition(m_actionShortcuts[i].first);
+				const ActionsManager::ActionDefinition definition(ActionsManager::getActionDefinition(m_actionShortcuts[i].first));
 
 				if (definition.identifier >= 0)
 				{
 					if (definition.flags.testFlag(ActionsManager::IsCheckableFlag))
 					{
-						Action *action = getAction(m_actionShortcuts[i].first);
+						Action *action(getAction(m_actionShortcuts[i].first));
 
 						if (action)
 						{
@@ -713,7 +797,7 @@ void MainWindow::triggerAction()
 		return;
 	}
 
-	Action *action = qobject_cast<Action*>(sender());
+	Action *action(qobject_cast<Action*>(sender()));
 
 	if (action)
 	{
@@ -725,11 +809,11 @@ void MainWindow::triggerAction(bool checked)
 {
 	Q_UNUSED(checked)
 
-	Action *action = qobject_cast<Action*>(sender());
+	Action *action(qobject_cast<Action*>(sender()));
 
 	if (action)
 	{
-		QVariantMap parameters = action->data().toMap();
+		QVariantMap parameters(action->data().toMap());
 
 		if (action->isCheckable())
 		{
@@ -742,22 +826,33 @@ void MainWindow::triggerAction(bool checked)
 
 void MainWindow::startToolBarDragging()
 {
-	m_topToolBarArea->setContentsMargins(0, 4, 0, 4);
-	m_bottomToolBarArea->setContentsMargins(0, 4, 0, 4);
-	m_leftToolBarArea->setContentsMargins(4, 0, 4, 0);
-	m_rightToolBarArea->setContentsMargins(4, 0, 4, 0);
+	if (m_hasToolBars)
+	{
+		m_topToolBarArea->setContentsMargins(0, 4, 0, 4);
+		m_bottomToolBarArea->setContentsMargins(0, 4, 0, 4);
+		m_leftToolBarArea->setContentsMargins(4, 0, 4, 0);
+		m_rightToolBarArea->setContentsMargins(4, 0, 4, 0);
+	}
 }
 
 void MainWindow::endToolBarDragging()
 {
-	m_topToolBarArea->setContentsMargins(0, 0, 0, 0);
-	m_bottomToolBarArea->setContentsMargins(0, 0, 0, 0);
-	m_leftToolBarArea->setContentsMargins(0, 0, 0, 0);
-	m_rightToolBarArea->setContentsMargins(0, 0, 0, 0);
+	if (m_hasToolBars)
+	{
+		m_topToolBarArea->setContentsMargins(0, 0, 0, 0);
+		m_bottomToolBarArea->setContentsMargins(0, 0, 0, 0);
+		m_leftToolBarArea->setContentsMargins(0, 0, 0, 0);
+		m_rightToolBarArea->setContentsMargins(0, 0, 0, 0);
+	}
 }
 
 void MainWindow::moveToolBar(ToolBarWidget *toolBar, Qt::ToolBarArea area)
 {
+	if (!m_hasToolBars || !toolBar)
+	{
+		return;
+	}
+
 	switch (area)
 	{
 		case Qt::BottomToolBarArea:
@@ -779,23 +874,34 @@ void MainWindow::moveToolBar(ToolBarWidget *toolBar, Qt::ToolBarArea area)
 	}
 }
 
-void MainWindow::openUrl(const QString &text)
+void MainWindow::openUrl(const QString &text, bool isPrivate)
 {
+	WindowsManager::OpenHints hints(isPrivate ? WindowsManager::PrivateOpen : WindowsManager::DefaultOpen);
+
 	if (text.isEmpty())
 	{
-		m_windowsManager->triggerAction(ActionsManager::NewTabAction);
+		m_windowsManager->triggerAction(hints | ActionsManager::NewTabAction);
 
 		return;
 	}
 	else
 	{
-		InputInterpreter *interpreter = new InputInterpreter(this);
+		InputInterpreter *interpreter(new InputInterpreter(this));
 
 		connect(interpreter, SIGNAL(requestedOpenBookmark(BookmarksItem*,WindowsManager::OpenHints)), m_windowsManager, SLOT(open(BookmarksItem*,WindowsManager::OpenHints)));
 		connect(interpreter, SIGNAL(requestedOpenUrl(QUrl,WindowsManager::OpenHints)), m_windowsManager, SLOT(open(QUrl,WindowsManager::OpenHints)));
 		connect(interpreter, SIGNAL(requestedSearch(QString,QString,WindowsManager::OpenHints)), m_windowsManager, SLOT(search(QString,QString,WindowsManager::OpenHints)));
 
-		interpreter->interpret(text, ((!m_workspace->getActiveWindow() || Utils::isUrlEmpty(m_workspace->getActiveWindow()->getUrl())) ? WindowsManager::CurrentTabOpen : WindowsManager::NewTabOpen));
+		if (!m_workspace->getActiveWindow() || (m_workspace->getActiveWindow()->getLoadingState() == WindowsManager::FinishedLoadingState && Utils::isUrlEmpty(m_workspace->getActiveWindow()->getUrl())))
+		{
+			hints |= WindowsManager::CurrentTabOpen;
+		}
+		else
+		{
+			hints |= WindowsManager::NewTabOpen;
+		}
+
+		interpreter->interpret(text, hints);
 	}
 }
 
@@ -816,20 +922,20 @@ void MainWindow::raiseWindow()
 
 void MainWindow::addBookmark(const QUrl &url, const QString &title, const QString &description, bool warn)
 {
-	const QUrl bookmarkUrl = (url.isValid() ? url : m_windowsManager->getUrl()).adjusted(QUrl::RemovePassword);
+	const QUrl bookmarkUrl((url.isValid() ? url : m_windowsManager->getUrl()).adjusted(QUrl::RemovePassword));
 
 	if (bookmarkUrl.isEmpty() || (warn && BookmarksManager::hasBookmark(bookmarkUrl) && QMessageBox::warning(this, tr("Warning"), tr("You already have this address in your bookmarks.\nDo you want to continue?"), (QMessageBox::Yes | QMessageBox::Cancel)) == QMessageBox::Cancel))
 	{
 		return;
 	}
 
-	BookmarkPropertiesDialog dialog(bookmarkUrl, (url.isValid() ? title : m_windowsManager->getTitle()), description, NULL, -1, true, this);
+	BookmarkPropertiesDialog dialog(bookmarkUrl, (url.isValid() ? title : m_windowsManager->getTitle()), description, nullptr, -1, true, this);
 	dialog.exec();
 }
 
 void MainWindow::editBookmark(const QUrl &url)
 {
-	const QList<BookmarksItem*> bookmarks = BookmarksManager::getModel()->getBookmarks(url);
+	const QList<BookmarksItem*> bookmarks(BookmarksManager::getModel()->getBookmarks(url));
 
 	if (!bookmarks.isEmpty())
 	{
@@ -842,14 +948,14 @@ void MainWindow::toolBarModified(int identifier)
 {
 	if (identifier == ToolBarsManager::MenuBar)
 	{
-		const bool showMenuBar = (ToolBarsManager::getToolBarDefinition(ToolBarsManager::MenuBar).visibility != ToolBarsManager::AlwaysHiddenToolBar);
+		const bool showMenuBar(ToolBarsManager::getToolBarDefinition(ToolBarsManager::MenuBar).normalVisibility != ToolBarsManager::AlwaysHiddenToolBar);
 
 		if (m_menuBar && !showMenuBar)
 		{
 			m_menuBar->deleteLater();
-			m_menuBar = NULL;
+			m_menuBar = nullptr;
 
-			setMenuBar(NULL);
+			setMenuBar(nullptr);
 		}
 		else if (!m_menuBar && showMenuBar)
 		{
@@ -862,14 +968,14 @@ void MainWindow::toolBarModified(int identifier)
 	}
 	else if (identifier == ToolBarsManager::StatusBar)
 	{
-		const bool showStatusBar = (ToolBarsManager::getToolBarDefinition(ToolBarsManager::StatusBar).visibility != ToolBarsManager::AlwaysHiddenToolBar);
+		const bool showStatusBar(ToolBarsManager::getToolBarDefinition(ToolBarsManager::StatusBar).normalVisibility != ToolBarsManager::AlwaysHiddenToolBar);
 
 		if (m_statusBar && !showStatusBar)
 		{
 			m_statusBar->deleteLater();
-			m_statusBar = NULL;
+			m_statusBar = nullptr;
 
-			setStatusBar(NULL);
+			setStatusBar(nullptr);
 		}
 		else if (!m_statusBar && showStatusBar)
 		{
@@ -882,7 +988,7 @@ void MainWindow::toolBarModified(int identifier)
 
 void MainWindow::transferStarted()
 {
-	const QString action = SettingsManager::getValue(QLatin1String("Browser/TransferStartingAction")).toString();
+	const QString action(SettingsManager::getValue(SettingsManager::Browser_TransferStartingActionOption).toString());
 
 	if (action == QLatin1String("openTab"))
 	{
@@ -910,6 +1016,11 @@ void MainWindow::transferStarted()
 
 void MainWindow::placeSidebars()
 {
+	if (!m_hasToolBars)
+	{
+		return;
+	}
+
 	if (m_sidebarToggle)
 	{
 		m_splitter->addWidget(m_sidebarToggle);
@@ -922,7 +1033,7 @@ void MainWindow::placeSidebars()
 
 	m_splitter->addWidget(m_workspace);
 
-	if (SettingsManager::getValue(QString("Sidebar/Reverse")).toBool())
+	if (SettingsManager::getValue(SettingsManager::Sidebar_ReverseOption).toBool())
 	{
 		for (int i = m_splitter->count() - 1; i >= 0; --i)
 		{
@@ -936,9 +1047,9 @@ void MainWindow::placeSidebars()
 void MainWindow::updateSidebars()
 {
 	QList<int> sizes;
-	const int sidebarSize = ((m_sidebar && m_sidebar->isVisible()) ? m_sidebar->sizeHint().width() : 0);
-	const int sidebarToggleSize = (m_sidebarToggle ? m_sidebarToggle->width() : 0);
-	const bool isReversed = SettingsManager::getValue(QLatin1String("Sidebar/Reverse")).toBool();
+	const int sidebarSize((m_sidebar && m_sidebar->isVisible()) ? m_sidebar->sizeHint().width() : 0);
+	const int sidebarToggleSize(m_sidebarToggle ? m_sidebarToggle->width() : 0);
+	const bool isReversed(SettingsManager::getValue(SettingsManager::Sidebar_ReverseOption).toBool());
 
 	if (m_sidebarToggle)
 	{
@@ -964,7 +1075,7 @@ void MainWindow::updateSidebars()
 		}
 	}
 
-	const int sidebarToggleIndex = (m_sidebarToggle ? (m_splitter->indexOf(m_sidebarToggle) + (isReversed ? 0 : 1)) : -1);
+	const int sidebarToggleIndex(m_sidebarToggle ? (m_splitter->indexOf(m_sidebarToggle) + (isReversed ? 0 : 1)) : -1);
 
 	for (int i = 0; i < m_splitter->count(); ++i)
 	{
@@ -979,7 +1090,7 @@ void MainWindow::updateSidebars()
 
 void MainWindow::updateWindowTitle(const QString &title)
 {
-	m_windowTitle = (title.isEmpty() ? QStringLiteral("Otter") : QStringLiteral("%1 - Otter").arg(title));
+	m_windowTitle = title;
 
 	setWindowTitle(m_windowTitle);
 }
@@ -993,9 +1104,8 @@ void MainWindow::updateShortcuts()
 
 	m_actionShortcuts.clear();
 
-	const QVector<ActionsManager::ActionDefinition> definitions = ActionsManager::getActionDefinitions();
-	QList<QKeySequence> standardShortcuts;
-	standardShortcuts << QKeySequence(QKeySequence::Copy) << QKeySequence(QKeySequence::Cut) << QKeySequence(QKeySequence::Delete) << QKeySequence(QKeySequence::Paste) << QKeySequence(QKeySequence::Redo) << QKeySequence(QKeySequence::SelectAll) << QKeySequence(QKeySequence::Undo);
+	const QVector<ActionsManager::ActionDefinition> definitions(ActionsManager::getActionDefinitions());
+	const QList<QKeySequence> standardShortcuts({QKeySequence(QKeySequence::Copy), QKeySequence(QKeySequence::Cut), QKeySequence(QKeySequence::Delete), QKeySequence(QKeySequence::Paste), QKeySequence(QKeySequence::Redo), QKeySequence(QKeySequence::SelectAll), QKeySequence(QKeySequence::Undo)});
 
 	for (int i = 0; i < definitions.count(); ++i)
 	{
@@ -1006,7 +1116,7 @@ void MainWindow::updateShortcuts()
 		{
 			if (!standardShortcuts.contains(definitions[i].shortcuts[j]))
 			{
-				QShortcut *shortcut = new QShortcut(definitions[i].shortcuts[j], this);
+				QShortcut *shortcut(new QShortcut(definitions[i].shortcuts[j], this));
 
 				shortcuts.append(shortcut);
 
@@ -1020,7 +1130,7 @@ void MainWindow::updateShortcuts()
 
 void MainWindow::setCurrentWindow(Window *window)
 {
-	Window *previousWindow = m_currentWindow;
+	Window *previousWindow(m_currentWindow);
 
 	m_currentWindow = window;
 
@@ -1028,16 +1138,18 @@ void MainWindow::setCurrentWindow(Window *window)
 	{
 		if (m_standardActions[i] && Action::isLocal(m_standardActions[i]->getIdentifier()))
 		{
-			const int identifier = m_standardActions[i]->getIdentifier();
-			Action *previousAction = ((previousWindow && previousWindow->getContentsWidget()) ? previousWindow->getContentsWidget()->getAction(identifier) : NULL);
-			Action *currentAction = (window ? window->getContentsWidget()->getAction(identifier) : NULL);
+			const int identifier(m_standardActions[i]->getIdentifier());
+			Action *previousAction((previousWindow && previousWindow->getContentsWidget()) ? previousWindow->getContentsWidget()->getAction(identifier) : nullptr);
+			Action *currentAction(window ? window->getContentsWidget()->getAction(identifier) : nullptr);
 
 			if (previousAction)
 			{
 				disconnect(previousAction, SIGNAL(changed()), m_standardActions[i], SLOT(setup()));
 			}
 
+			m_standardActions[i]->blockSignals(true);
 			m_standardActions[i]->setup(currentAction);
+			m_standardActions[i]->blockSignals(false);
 
 			if (currentAction)
 			{
@@ -1056,7 +1168,7 @@ MainWindow* MainWindow::findMainWindow(QObject *parent)
 {
 	if (!parent)
 	{
-		return NULL;
+		return nullptr;
 	}
 
 	if (parent->metaObject()->className() == QLatin1String("Otter::MainWindow"))
@@ -1064,8 +1176,8 @@ MainWindow* MainWindow::findMainWindow(QObject *parent)
 		return qobject_cast<MainWindow*>(parent);
 	}
 
-	MainWindow *window = NULL;
-	QWidget *widget = qobject_cast<QWidget*>(parent);
+	MainWindow *window(nullptr);
+	QWidget *widget(qobject_cast<QWidget*>(parent));
 
 	if (widget && widget->window())
 	{
@@ -1096,13 +1208,13 @@ Action* MainWindow::getAction(int identifier)
 {
 	if (identifier < 0 || identifier >= m_standardActions.count())
 	{
-		return NULL;
+		return nullptr;
 	}
 
 	if (!m_standardActions[identifier])
 	{
-		const ActionsManager::ActionDefinition definition = ActionsManager::getActionDefinition(identifier);
-		Action *action = new Action(identifier, this);
+		const ActionsManager::ActionDefinition definition(ActionsManager::getActionDefinition(identifier));
+		Action *action(new Action(identifier, this));
 
 		m_standardActions[identifier] = action;
 
@@ -1118,7 +1230,7 @@ Action* MainWindow::getAction(int identifier)
 		}
 	}
 
-	return m_standardActions.value(identifier, NULL);
+	return m_standardActions.value(identifier, nullptr);
 }
 
 WorkspaceWidget* MainWindow::getWorkspace()
@@ -1136,6 +1248,11 @@ WindowsManager* MainWindow::getWindowsManager()
 	return m_windowsManager;
 }
 
+bool MainWindow::areControlsHidden() const
+{
+	return windowState().testFlag(Qt::WindowFullScreen);
+}
+
 bool MainWindow::event(QEvent *event)
 {
 	switch (event->type())
@@ -1148,7 +1265,7 @@ bool MainWindow::event(QEvent *event)
 			break;
 		case QEvent::WindowStateChange:
 			{
-				QWindowStateChangeEvent *stateChangeEvent = dynamic_cast<QWindowStateChangeEvent*>(event);
+				QWindowStateChangeEvent *stateChangeEvent(dynamic_cast<QWindowStateChangeEvent*>(event));
 
 				SessionsManager::markSessionModified();
 
@@ -1156,7 +1273,7 @@ bool MainWindow::event(QEvent *event)
 				{
 					if (isFullScreen())
 					{
-						getAction(ActionsManager::FullScreenAction)->setIcon(Utils::getIcon(QLatin1String("view-restore")));
+						getAction(ActionsManager::FullScreenAction)->setIcon(ThemesManager::getIcon(QLatin1String("view-restore")));
 
 						if (m_statusBar)
 						{
@@ -1168,11 +1285,16 @@ bool MainWindow::event(QEvent *event)
 							m_menuBar->hide();
 						}
 
+						if (m_sidebarToggle)
+						{
+							m_sidebarToggle->hide();
+						}
+
 						m_workspace->installEventFilter(this);
 					}
 					else
 					{
-						getAction(ActionsManager::FullScreenAction)->setIcon(Utils::getIcon(QLatin1String("view-fullscreen")));
+						getAction(ActionsManager::FullScreenAction)->setIcon(ThemesManager::getIcon(QLatin1String("view-fullscreen")));
 
 						if (m_statusBar)
 						{
@@ -1184,9 +1306,25 @@ bool MainWindow::event(QEvent *event)
 							m_menuBar->show();
 						}
 
+						if (m_sidebarToggle)
+						{
+							m_sidebarToggle->show();
+						}
+
 						m_workspace->removeEventFilter(this);
 					}
 
+					if (!windowState().testFlag(Qt::WindowFullScreen))
+					{
+						killTimer(m_mouseTrackerTimer);
+
+						m_mouseTrackerTimer = 0;
+					}
+					else
+					{
+						m_mouseTrackerTimer = startTimer(250);
+					}
+
 					emit controlsHiddenChanged(windowState().testFlag(Qt::WindowFullScreen));
 				}
 
@@ -1219,7 +1357,7 @@ bool MainWindow::event(QEvent *event)
 			break;
 		case QEvent::StatusTip:
 			{
-				QStatusTipEvent *statusTipEvent = static_cast<QStatusTipEvent*>(event);
+				QStatusTipEvent *statusTipEvent(static_cast<QStatusTipEvent*>(event));
 
 				if (statusTipEvent)
 				{
@@ -1243,7 +1381,7 @@ bool MainWindow::eventFilter(QObject *object, QEvent *event)
 {
 	if (object == m_workspace && event->type() == QEvent::KeyPress)
 	{
-		QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
+		QKeyEvent *keyEvent(static_cast<QKeyEvent*>(event));
 
 		if (keyEvent && keyEvent->key() == Qt::Key_Escape)
 		{
