@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include "../core/NetworkManagerFactory.h"
 #include "../core/NotesManager.h"
 #include "../core/SessionsManager.h"
+#include "../core/ThemesManager.h"
 #include "../core/ToolBarsManager.h"
 #include "../core/Utils.h"
 #include "../core/WindowsManager.h"
@@ -42,8 +43,8 @@ namespace Otter
 {
 
 Menu::Menu(MenuRole role, QWidget *parent) : QMenu(parent),
-	m_actionGroup(NULL),
-	m_bookmark(NULL),
+	m_actionGroup(nullptr),
+	m_bookmark(nullptr),
 	m_role(role)
 {
 	switch (role)
@@ -54,7 +55,7 @@ Menu::Menu(MenuRole role, QWidget *parent) : QMenu(parent),
 			{
 				installEventFilter(this);
 
-				Menu *parentMenu = qobject_cast<Menu*>(parent);
+				Menu *parentMenu(qobject_cast<Menu*>(parent));
 
 				if (!parentMenu || parentMenu->getRole() != m_role)
 				{
@@ -79,9 +80,9 @@ Menu::Menu(MenuRole role, QWidget *parent) : QMenu(parent),
 			break;
 		case ClosedWindowsMenu:
 			{
-				setIcon(Utils::getIcon(QLatin1String("user-trash")));
+				setIcon(ThemesManager::getIcon(QLatin1String("user-trash")));
 
-				MainWindow *mainWindow = MainWindow::findMainWindow(parent);
+				MainWindow *mainWindow(MainWindow::findMainWindow(parent));
 
 				if (mainWindow)
 				{
@@ -96,14 +97,35 @@ Menu::Menu(MenuRole role, QWidget *parent) : QMenu(parent),
 
 			break;
 		case ImportExportMenuRole:
-			QMenu::addAction(tr("Import Opera Bookmarks…"))->setData(QLatin1String("OperaBookmarks"));
-			QMenu::addAction(tr("Import HTML Bookmarks…"))->setData(QLatin1String("HtmlBookmarks"));
-			QMenu::addSeparator();
-			QMenu::addAction(tr("Import Opera Notes…"))->setData(QLatin1String("OperaNotes"));
-			QMenu::addSeparator();
-			QMenu::addAction(tr("Import Opera Session…"))->setData(QLatin1String("OperaSession"));
+			{
+				Action *importOperaBookmarksAction(addAction());
+				importOperaBookmarksAction->setData(QLatin1String("OperaBookmarks"));
+				importOperaBookmarksAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Import Opera Bookmarks…"));
 
-			connect(this, SIGNAL(triggered(QAction*)), this, SLOT(openImporter(QAction*)));
+				Action *importHtmlBookmarksAction(addAction());
+				importHtmlBookmarksAction->setData(QLatin1String("HtmlBookmarks"));
+				importHtmlBookmarksAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Import HTML Bookmarks…"));
+
+				addSeparator();
+
+				Action *importOperaNotesAction(addAction());
+				importOperaNotesAction->setData(QLatin1String("OperaNotes"));
+				importOperaNotesAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Import Opera Notes…"));
+
+				addSeparator();
+
+				Action *importOperaSearchEnginesAction(addAction());
+				importOperaSearchEnginesAction->setData(QLatin1String("OperaSearchEngines"));
+				importOperaSearchEnginesAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Import Opera Search Engines…"));
+
+				addSeparator();
+
+				Action *importOperaSessionAction(addAction());
+				importOperaSessionAction->setData(QLatin1String("OperaSession"));
+				importOperaSessionAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Import Opera Session…"));
+
+				connect(this, SIGNAL(triggered(QAction*)), this, SLOT(openImporter(QAction*)));
+			}
 
 			break;
 		case SessionsMenuRole:
@@ -149,11 +171,11 @@ void Menu::mouseReleaseEvent(QMouseEvent *event)
 {
 	if (m_role == BookmarksMenuRole && (event->button() == Qt::LeftButton || event->button() == Qt::MiddleButton))
 	{
-		QAction *action = actionAt(event->pos());
+		QAction *action(actionAt(event->pos()));
 
 		if (action && action->isEnabled() && action->data().type() == QVariant::ModelIndex)
 		{
-			QWidget *menu = this;
+			QWidget *menu(this);
 
 			while (menu)
 			{
@@ -166,11 +188,11 @@ void Menu::mouseReleaseEvent(QMouseEvent *event)
 				}
 			}
 
-			MainWindow *mainWindow = MainWindow::findMainWindow(parent());
+			MainWindow *mainWindow(MainWindow::findMainWindow(parent()));
 
 			if (mainWindow)
 			{
-				mainWindow->getWindowsManager()->open(BookmarksManager::getModel()->getBookmark(action->data().toModelIndex()), WindowsManager::calculateOpenHints(event->modifiers(), event->button()));
+				mainWindow->getWindowsManager()->open(BookmarksManager::getModel()->getBookmark(action->data().toModelIndex()), WindowsManager::calculateOpenHints(WindowsManager::DefaultOpen, event->button(), event->modifiers()));
 
 				return;
 			}
@@ -184,14 +206,14 @@ void Menu::contextMenuEvent(QContextMenuEvent *event)
 {
 	if (m_role == BookmarksMenuRole)
 	{
-		QAction *action = actionAt(event->pos());
+		QAction *action(actionAt(event->pos()));
 
 		if (action && action->isEnabled() && action->data().type() == QVariant::ModelIndex)
 		{
 			m_bookmark = BookmarksManager::getModel()->getBookmark(action->data().toModelIndex());
 
 			QMenu contextMenu(this);
-			contextMenu.addAction(Utils::getIcon(QLatin1String("document-open")), tr("Open"), this, SLOT(openBookmark()));
+			contextMenu.addAction(ThemesManager::getIcon(QLatin1String("document-open")), tr("Open"), this, SLOT(openBookmark()));
 			contextMenu.addAction(tr("Open in New Tab"), this, SLOT(openBookmark()))->setData(WindowsManager::NewTabOpen);
 			contextMenu.addAction(tr("Open in New Background Tab"), this, SLOT(openBookmark()))->setData(static_cast<int>(WindowsManager::NewTabOpen | WindowsManager::BackgroundOpen));
 			contextMenu.addSeparator();
@@ -209,7 +231,7 @@ void Menu::contextMenuEvent(QContextMenuEvent *event)
 void Menu::load(const QString &path, const QStringList &options)
 {
 	QFile file(SessionsManager::getReadableDataPath(path));
-	file.open(QFile::ReadOnly);
+	file.open(QIODevice::ReadOnly);
 
 	load(QJsonDocument::fromJson(file.readAll()).object(), options);
 
@@ -218,7 +240,7 @@ void Menu::load(const QString &path, const QStringList &options)
 
 void Menu::load(const QJsonObject &definition, const QStringList &options)
 {
-	const QString identifier = definition.value(QLatin1String("identifier")).toString();
+	const QString identifier(definition.value(QLatin1String("identifier")).toString());
 
 	if (m_role == NoMenuRole)
 	{
@@ -230,20 +252,20 @@ void Menu::load(const QJsonObject &definition, const QStringList &options)
 	setObjectName(identifier);
 	setTitle(QCoreApplication::translate("actions", m_title.toUtf8().constData()));
 
-	const QJsonArray actions = definition.value(QLatin1String("actions")).toArray();
+	const QJsonArray actions(definition.value(QLatin1String("actions")).toArray());
 
 	for (int i = 0; i < actions.count(); ++i)
 	{
 		if (actions.at(i).isObject())
 		{
-			const QJsonObject object = actions.at(i).toObject();
+			const QJsonObject object(actions.at(i).toObject());
 
 			if (object.contains(QLatin1String("includeIn")) && !options.contains(object.value(QLatin1String("includeIn")).toString()))
 			{
 				continue;
 			}
 
-			Menu *menu = new Menu(Menu::getRole(object.value(QLatin1String("identifier")).toString()), this);
+			Menu *menu(new Menu(Menu::getRole(object.value(QLatin1String("identifier")).toString()), this));
 			menu->load(object, options);
 
 			if (object.value(QLatin1String("type")).toString() == QLatin1String("menu"))
@@ -260,7 +282,7 @@ void Menu::load(const QJsonObject &definition, const QStringList &options)
 		}
 		else
 		{
-			const QString rawAction = actions.at(i).toString();
+			const QString rawAction(actions.at(i).toString());
 
 			if (rawAction == QLatin1String("separator"))
 			{
@@ -268,11 +290,11 @@ void Menu::load(const QJsonObject &definition, const QStringList &options)
 			}
 			else
 			{
-				const int action = ActionsManager::getActionIdentifier(rawAction);
+				const int action(ActionsManager::getActionIdentifier(rawAction));
 
 				if (action >= 0)
 				{
-					WindowsManager *manager = SessionsManager::getWindowsManager();
+					WindowsManager *manager(SessionsManager::getWindowsManager());
 
 					if (manager && Action::isLocal(action) && manager->getAction(action))
 					{
@@ -280,7 +302,7 @@ void Menu::load(const QJsonObject &definition, const QStringList &options)
 					}
 					else
 					{
-						QMenu::addAction(ActionsManager::getAction(action, parentWidget()));
+						addAction(action, true);
 					}
 				}
 			}
@@ -290,14 +312,14 @@ void Menu::load(const QJsonObject &definition, const QStringList &options)
 
 void Menu::populateModelMenu()
 {
-	QMenu *menu = qobject_cast<QMenu*>(sender());
+	Menu *menu(qobject_cast<Menu*>(sender()));
 
 	if (!menu || !menu->menuAction())
 	{
 		return;
 	}
 
-	QModelIndex index = menu->menuAction()->data().toModelIndex();
+	QModelIndex index(menu->menuAction()->data().toModelIndex());
 
 	if (!menu->actions().isEmpty() && !(m_role == BookmarksMenuRole && !index.isValid() && menu->actions().count() == 3))
 	{
@@ -309,7 +331,7 @@ void Menu::populateModelMenu()
 		index = ((m_role == NotesMenuRole) ? NotesManager::getModel() : BookmarksManager::getModel())->getRootItem()->index();
 	}
 
-	const QAbstractItemModel *model = index.model();
+	const QAbstractItemModel *model(index.model());
 
 	if (!model)
 	{
@@ -318,38 +340,54 @@ void Menu::populateModelMenu()
 
 	if (model->rowCount(index) > 1 && m_role == BookmarksMenuRole)
 	{
-		QAction *openAllAction = menu->addAction(Utils::getIcon(QLatin1String("document-open-folder")), tr("Open All"), this, SLOT(openBookmark()));
+		Action *openAllAction(menu->addAction());
 		openAllAction->setData(index);
+		openAllAction->setIcon(ThemesManager::getIcon(QLatin1String("document-open-folder")));
+		openAllAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Open All"));
 
 		menu->addSeparator();
+
+		connect(openAllAction, SIGNAL(triggered()), this, SLOT(openBookmark()));
 	}
 
 	if (m_role == BookmarkSelectorMenuRole)
 	{
-		QAction *addFolderAction = menu->addAction(Utils::getIcon(QLatin1String("document-open-folder")), tr("This Folder"));
+		Action *addFolderAction(menu->addAction());
 		addFolderAction->setData(index);
+		addFolderAction->setIcon(ThemesManager::getIcon(QLatin1String("document-open-folder")));
+		addFolderAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "This Folder"));
 
 		menu->addSeparator();
 	}
 
 	for (int i = 0; i < model->rowCount(index); ++i)
 	{
-		const QModelIndex childIndex = index.child(i, 0);
+		const QModelIndex childIndex(index.child(i, 0));
 
 		if (!childIndex.isValid())
 		{
 			continue;
 		}
 
-		const BookmarksModel::BookmarkType type = static_cast<BookmarksModel::BookmarkType>(childIndex.data(BookmarksModel::TypeRole).toInt());
+		const BookmarksModel::BookmarkType type(static_cast<BookmarksModel::BookmarkType>(childIndex.data(BookmarksModel::TypeRole).toInt()));
 
 		if (type == BookmarksModel::RootBookmark || type == BookmarksModel::FolderBookmark || type == BookmarksModel::UrlBookmark)
 		{
-			QAction *action = menu->addAction(childIndex.data(Qt::DecorationRole).value<QIcon>(), (childIndex.data(BookmarksModel::TitleRole).toString().isEmpty() ? tr("(Untitled)") : Utils::elideText(QString(childIndex.data(BookmarksModel::TitleRole).toString()).replace(QLatin1Char('&'), QLatin1String("&&")), menu)));
+			Action *action(menu->addAction());
 			action->setData(childIndex);
+			action->setIcon(childIndex.data(Qt::DecorationRole).value<QIcon>());
 			action->setToolTip(childIndex.data(BookmarksModel::DescriptionRole).toString());
 			action->setStatusTip(childIndex.data(BookmarksModel::UrlRole).toString());
 
+			if (childIndex.data(BookmarksModel::TitleRole).toString().isEmpty())
+			{
+				action->setOverrideText(QT_TRANSLATE_NOOP("actions", "(Untitled)"));
+			}
+			else
+			{
+				action->setText(Utils::elideText(QString(childIndex.data(BookmarksModel::TitleRole).toString()).replace(QLatin1Char('&'), QLatin1String("&&")), menu));
+			}
+
 			if (type == BookmarksModel::UrlBookmark && m_role == BookmarksMenuRole)
 			{
 				connect(action, SIGNAL(triggered()), this, SLOT(openBookmark()));
@@ -377,15 +415,15 @@ void Menu::populateCharacterEncodingMenu()
 {
 	if (!m_actionGroup)
 	{
-		QList<int> textCodecs;
-		textCodecs << 106 << 1015 << 1017 << 4 << 5 << 6 << 7 << 8 << 82 << 10 << 85 << 12 << 13 << 109 << 110 << 112 << 2250 << 2251 << 2252 << 2253 << 2254 << 2255 << 2256 << 2257 << 2258 << 18 << 39 << 17 << 38 << 2026;
+		const QList<int> textCodecs({106, 1015, 1017, 4, 5, 6, 7, 8, 82, 10, 85, 12, 13, 109, 110, 112, 2250, 2251, 2252, 2253, 2254, 2255, 2256, 2257, 2258, 18, 39, 17, 38, 2026});
 
 		m_actionGroup = new QActionGroup(this);
 		m_actionGroup->setExclusive(true);
 
-		QAction *defaultAction = QMenu::addAction(tr("Auto Detect"));
+		Action *defaultAction(addAction());
 		defaultAction->setData(-1);
 		defaultAction->setCheckable(true);
+		defaultAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Auto Detect"));
 
 		m_actionGroup->addAction(defaultAction);
 
@@ -393,14 +431,14 @@ void Menu::populateCharacterEncodingMenu()
 
 		for (int i = 0; i < textCodecs.count(); ++i)
 		{
-			QTextCodec *codec = QTextCodec::codecForMib(textCodecs.at(i));
+			QTextCodec *codec(QTextCodec::codecForMib(textCodecs.at(i)));
 
 			if (!codec)
 			{
 				continue;
 			}
 
-			QAction *textCodecAction = QMenu::addAction(Utils::elideText(codec->name(), this));
+			QAction *textCodecAction(QMenu::addAction(Utils::elideText(codec->name(), this)));
 			textCodecAction->setData(textCodecs.at(i));
 			textCodecAction->setCheckable(true);
 
@@ -408,12 +446,12 @@ void Menu::populateCharacterEncodingMenu()
 		}
 	}
 
-	MainWindow *mainWindow = MainWindow::findMainWindow(parent());
-	const QString encoding = (mainWindow ? mainWindow->getWindowsManager()->getOption(QLatin1String("Content/DefaultCharacterEncoding")).toString().toLower() : QString());
+	MainWindow *mainWindow(MainWindow::findMainWindow(parent()));
+	const QString encoding(mainWindow ? mainWindow->getWindowsManager()->getOption(SettingsManager::Content_DefaultCharacterEncodingOption).toString().toLower() : QString());
 
 	for (int i = 2; i < actions().count(); ++i)
 	{
-		QAction *action = actions().at(i);
+		QAction *action(actions().at(i));
 
 		if (!action)
 		{
@@ -438,11 +476,14 @@ void Menu::populateClosedWindowsMenu()
 {
 	clear();
 
-	QMenu::addAction(Utils::getIcon(QLatin1String("edit-clear")), tr("Clear"), this, SLOT(clearClosedWindows()))->setData(0);
+	Action *clearAction(addAction());
+	clearAction->setData(0);
+	clearAction->setIcon(ThemesManager::getIcon(QLatin1String("edit-clear")));
+	clearAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Clear"));
 
 	addSeparator();
 
-	const QStringList windows = SessionsManager::getClosedWindows();
+	const QStringList windows(SessionsManager::getClosedWindows());
 
 	if (!windows.isEmpty())
 	{
@@ -454,19 +495,21 @@ void Menu::populateClosedWindowsMenu()
 		addSeparator();
 	}
 
-	MainWindow *mainWindow = MainWindow::findMainWindow(parent());
+	MainWindow *mainWindow(MainWindow::findMainWindow(parent()));
 
 	if (mainWindow)
 	{
-		const QList<ClosedWindow> tabs = mainWindow->getWindowsManager()->getClosedWindows();
+		const QList<ClosedWindow> tabs(mainWindow->getWindowsManager()->getClosedWindows());
 
 		for (int i = 0; i < tabs.count(); ++i)
 		{
-			QAction *action = QMenu::addAction(HistoryManager::getIcon(QUrl(tabs.at(i).window.getUrl())), Utils::elideText(tabs.at(i).window.getTitle(), this), this, SLOT(restoreClosedWindow()));
+			QAction *action(QMenu::addAction(HistoryManager::getIcon(QUrl(tabs.at(i).window.getUrl())), Utils::elideText(tabs.at(i).window.getTitle().replace(QLatin1Char('&'), QLatin1String("&&")), this), this, SLOT(restoreClosedWindow())));
 			action->setData(i + 1);
 			action->setStatusTip(tabs.at(i).window.getUrl());
 		}
 	}
+
+	connect(clearAction, SIGNAL(triggered()), this, SLOT(clearClosedWindows()));
 }
 
 void Menu::populateSessionsMenu()
@@ -478,37 +521,37 @@ void Menu::populateSessionsMenu()
 
 	clear();
 
-	QMenu::addAction(ActionsManager::getAction(ActionsManager::SaveSessionAction, parent()));
-	QMenu::addAction(ActionsManager::getAction(ActionsManager::SessionsAction, parent()));
+	addAction(ActionsManager::SaveSessionAction, true);
+	addAction(ActionsManager::SessionsAction, true);
 
 	addSeparator();
 
 	m_actionGroup = new QActionGroup(this);
 	m_actionGroup->setExclusive(true);
 
-	const QStringList sessions = SessionsManager::getSessions();
+	const QStringList sessions(SessionsManager::getSessions());
 	QMultiHash<QString, SessionInformation> information;
 
 	for (int i = 0; i < sessions.count(); ++i)
 	{
-		const SessionInformation session = SessionsManager::getSession(sessions.at(i));
+		const SessionInformation session(SessionsManager::getSession(sessions.at(i)));
 
 		information.insert((session.title.isEmpty() ? tr("(Untitled)") : session.title), session);
 	}
 
-	const QList<SessionInformation> sorted = information.values();
-	const QString currentSession = SessionsManager::getCurrentSession();
+	const QList<SessionInformation> sorted(information.values());
+	const QString currentSession(SessionsManager::getCurrentSession());
 
 	for (int i = 0; i < sorted.count(); ++i)
 	{
-		int windows = 0;
+		int windows(0);
 
 		for (int j = 0; j < sorted.at(i).windows.count(); ++j)
 		{
 			windows += sorted.at(i).windows.at(j).windows.count();
 		}
 
-		QAction *action = QMenu::addAction(tr("%1 (%n tab(s))", "", windows).arg(sorted.at(i).title.isEmpty() ? tr("(Untitled)") : QString(sorted.at(i).title).replace(QLatin1Char('&'), QLatin1String("&&"))));
+		QAction *action(QMenu::addAction(tr("%1 (%n tab(s))", "", windows).arg(sorted.at(i).title.isEmpty() ? tr("(Untitled)") : QString(sorted.at(i).title).replace(QLatin1Char('&'), QLatin1String("&&")))));
 		action->setData(sorted.at(i).path);
 		action->setCheckable(true);
 		action->setChecked(sorted.at(i).path == currentSession);
@@ -521,19 +564,19 @@ void Menu::populateStyleSheetsMenu()
 {
 	clear();
 
-	QAction *defaultAction = QMenu::addAction(tr("Default"));
+	MainWindow *mainWindow(MainWindow::findMainWindow(parent()));
+	Action *defaultAction(addAction());
 	defaultAction->setData(-1);
 	defaultAction->setCheckable(true);
 	defaultAction->setChecked(true);
-
-	MainWindow *mainWindow = MainWindow::findMainWindow(parent());
+	defaultAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Default Style"));
 
 	if (!mainWindow)
 	{
 		return;
 	}
 
-	Window *window = mainWindow->getWindowsManager()->getWindowByIndex(mainWindow->getWindowsManager()->getWindowIndex(-1));
+	Window *window(mainWindow->getWindowsManager()->getWindowByIndex(mainWindow->getWindowsManager()->getWindowIndex(-1)));
 
 	if (!window)
 	{
@@ -542,15 +585,15 @@ void Menu::populateStyleSheetsMenu()
 
 	addSeparator();
 
-	const QString activeStyleSheet = window->getContentsWidget()->getActiveStyleSheet();
-	const QStringList styleSheets = window->getContentsWidget()->getStyleSheets();
-	QActionGroup *actionGroup = new QActionGroup(this);
+	const QString activeStyleSheet(window->getContentsWidget()->getActiveStyleSheet());
+	const QStringList styleSheets(window->getContentsWidget()->getStyleSheets());
+	QActionGroup *actionGroup(new QActionGroup(this));
 	actionGroup->setExclusive(true);
 	actionGroup->addAction(defaultAction);
 
 	for (int i = 0; i < styleSheets.count(); ++i)
 	{
-		QAction *action = QMenu::addAction(styleSheets.at(i));
+		QAction *action(QMenu::addAction(styleSheets.at(i)));
 
 		action->setCheckable(true);
 		action->setChecked(styleSheets.at(i) == activeStyleSheet);
@@ -565,30 +608,49 @@ void Menu::populateToolBarsMenu()
 {
 	clear();
 
-	const QVector<ToolBarsManager::ToolBarDefinition> definitions = ToolBarsManager::getToolBarDefinitions();
+	const QVector<ToolBarsManager::ToolBarDefinition> definitions(ToolBarsManager::getToolBarDefinitions());
 
 	for (int i = 0; i < definitions.count(); ++i)
 	{
-		QAction *toolBarAction = QMenu::addAction(definitions.at(i).title.isEmpty() ? tr("(Untitled)") : definitions.at(i).title);
-		toolBarAction->setData(definitions.at(i).identifier);
-		toolBarAction->setCheckable(true);
-		toolBarAction->setChecked(definitions.at(i).visibility != ToolBarsManager::AlwaysHiddenToolBar);
+		Action *action(addAction());
+		action->setData(definitions.at(i).identifier);
+		action->setCheckable(true);
+		action->setChecked(definitions.at(i).normalVisibility != ToolBarsManager::AlwaysHiddenToolBar);
 
-		connect(toolBarAction, SIGNAL(toggled(bool)), this, SLOT(setToolBarVisibility(bool)));
+		if (definitions.at(i).getTitle().isEmpty())
+		{
+			action->setOverrideText(QT_TRANSLATE_NOOP("actions", "(Untitled)"));
+		}
+		else
+		{
+			action->setText(definitions.at(i).getTitle());
+		}
+
+		connect(action, SIGNAL(toggled(bool)), this, SLOT(setToolBarVisibility(bool)));
 	}
 
 	addAction(ActionsManager::ShowSidebarAction, true);
 	addAction(ActionsManager::ShowErrorConsoleAction, true);
 	addSeparator();
 
-	QMenu *menu = addMenu(tr("Add New"));
-	menu->addAction(tr("Add Toolbar"), ToolBarsManager::getInstance(), SLOT(addToolBar()));
-	menu->addAction(tr("Add Bookmarks Bar…"), ToolBarsManager::getInstance(), SLOT(addBookmarksBar()));
+	Menu *addNewMenu(new Menu(NoMenuRole, this));
+	Action *addNewAction(addAction());
+	addNewAction->setMenu(addNewMenu);
+	addNewAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Add New"));
+
+	Action *addToolBarAction(addNewMenu->addAction());
+	addToolBarAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Add Toolbar…"));
+
+	Action *addBookmarksBarAction(addNewMenu->addAction());
+	addBookmarksBarAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Add Bookmarks Bar…"));
 
 	addSeparator();
 	addAction(ActionsManager::LockToolBarsAction, true);
 	addSeparator();
-	QMenu::addAction(tr("Reset to Defaults…"), ToolBarsManager::getInstance(), SLOT(resetToolBars()));
+	addAction(ActionsManager::ResetToolBarsAction, true);
+
+	connect(addToolBarAction, SIGNAL(triggered()), ToolBarsManager::getInstance(), SLOT(addToolBar()));
+	connect(addBookmarksBarAction, SIGNAL(triggered()), ToolBarsManager::getInstance(), SLOT(addBookmarksBar()));
 }
 
 void Menu::populateUserAgentMenu()
@@ -600,17 +662,18 @@ void Menu::populateUserAgentMenu()
 		clear();
 	}
 
-	MainWindow *mainWindow = MainWindow::findMainWindow(parent());
-	const QStringList userAgents = NetworkManagerFactory::getUserAgents();
-	const QString userAgent = (mainWindow ? mainWindow->getWindowsManager()->getOption(QLatin1String("Network/UserAgent")).toString() : QString());
+	MainWindow *mainWindow(MainWindow::findMainWindow(parent()));
+	const QStringList userAgents(NetworkManagerFactory::getUserAgents());
+	const QString userAgent(mainWindow ? mainWindow->getWindowsManager()->getOption(SettingsManager::Network_UserAgentOption).toString() : QString());
 
 	m_actionGroup = new QActionGroup(this);
 	m_actionGroup->setExclusive(true);
 
-	QAction *defaultAction = QMenu::addAction(tr("Default"));
+	Action *defaultAction(addAction());
 	defaultAction->setData(QLatin1String("default"));
 	defaultAction->setCheckable(true);
 	defaultAction->setChecked(userAgent == QLatin1String("default"));
+	defaultAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Default User Agent"));
 
 	m_actionGroup->addAction(defaultAction);
 
@@ -618,21 +681,29 @@ void Menu::populateUserAgentMenu()
 
 	for (int i = 0; i < userAgents.count(); ++i)
 	{
-		const QString title = NetworkManagerFactory::getUserAgent(userAgents.at(i)).title;
-		QAction *userAgentAction = QMenu::addAction((title.isEmpty() ? tr("(Untitled)") : Utils::elideText(title, this)));
-		userAgentAction->setData(userAgents.at(i));
-		userAgentAction->setCheckable(true);
-		userAgentAction->setChecked(userAgent == userAgents.at(i));
+		if (userAgents.at(i).isEmpty())
+		{
+			addSeparator();
+		}
+		else
+		{
+			Action *action(addAction());
+			action->setData(userAgents.at(i));
+			action->setCheckable(true);
+			action->setChecked(userAgent == userAgents.at(i));
+			action->setText(Utils::elideText(NetworkManagerFactory::getUserAgent(userAgents.at(i)).getTitle(), this));
 
-		m_actionGroup->addAction(userAgentAction);
+			m_actionGroup->addAction(action);
+		}
 	}
 
 	addSeparator();
 
-	QAction *customAction = QMenu::addAction(tr("Custom"));
+	Action *customAction(addAction());
 	customAction->setData(QLatin1String("custom"));
 	customAction->setCheckable(true);
 	customAction->setChecked(userAgent.startsWith(QLatin1String("custom;")));
+	customAction->setOverrideText(QT_TRANSLATE_NOOP("actions", "Custom User Agent…"));
 
 	m_actionGroup->addAction(customAction);
 
@@ -646,7 +717,7 @@ void Menu::populateWindowsMenu()
 {
 	if (actions().isEmpty())
 	{
-		MainWindow *mainWindow = MainWindow::findMainWindow(this);
+		MainWindow *mainWindow(MainWindow::findMainWindow(this));
 
 		if (mainWindow)
 		{
@@ -660,7 +731,7 @@ void Menu::populateWindowsMenu()
 
 	clear();
 
-	MainWindow *mainWindow = MainWindow::findMainWindow(this);
+	MainWindow *mainWindow(MainWindow::findMainWindow(this));
 
 	if (!mainWindow)
 	{
@@ -669,18 +740,29 @@ void Menu::populateWindowsMenu()
 
 	for (int i = 0; i < mainWindow->getWindowsManager()->getWindowCount(); ++i)
 	{
-		Window *window = mainWindow->getWindowsManager()->getWindowByIndex(i);
+		Window *window(mainWindow->getWindowsManager()->getWindowByIndex(i));
 
 		if (window)
 		{
-			QMenu::addAction(window->getIcon(), (window->getTitle().isEmpty() ? tr("(Untitled)") : Utils::elideText(window->getTitle())))->setData(window->getIdentifier());
+			Action *action(addAction());
+			action->setData(window->getIdentifier());
+			action->setIcon(window->getIcon());
+
+			if (window->getTitle().isEmpty())
+			{
+				action->setOverrideText(QT_TRANSLATE_NOOP("actions", "(Untitled)"));
+			}
+			else
+			{
+				action->setText(Utils::elideText(window->getTitle()));
+			}
 		}
 	}
 }
 
 void Menu::clearModelMenu()
 {
-	const int offset = ((m_role == BookmarksMenuRole && menuAction() && !menuAction()->data().toModelIndex().isValid()) ? 3 : 0);
+	const int offset((m_role == BookmarksMenuRole && menuAction() && !menuAction()->data().toModelIndex().isValid()) ? 3 : 0);
 
 	for (int i = (actions().count() - 1); i >= offset; --i)
 	{
@@ -691,7 +773,7 @@ void Menu::clearModelMenu()
 
 	if (m_role != BookmarksMenuRole && menuAction())
 	{
-		const QAbstractItemModel *model = menuAction()->data().toModelIndex().model();
+		const QAbstractItemModel *model(menuAction()->data().toModelIndex().model());
 
 		if (model)
 		{
@@ -704,7 +786,7 @@ void Menu::clearModelMenu()
 
 void Menu::clearClosedWindows()
 {
-	MainWindow *mainWindow = MainWindow::findMainWindow(parent());
+	MainWindow *mainWindow(MainWindow::findMainWindow(parent()));
 
 	if (mainWindow)
 	{
@@ -716,12 +798,12 @@ void Menu::clearClosedWindows()
 
 void Menu::restoreClosedWindow()
 {
-	QAction *action = qobject_cast<QAction*>(sender());
-	const int index = ((action && action->data().type() == QVariant::Int) ? action->data().toInt() : 0);
+	QAction *action(qobject_cast<QAction*>(sender()));
+	const int index((action && action->data().type() == QVariant::Int) ? action->data().toInt() : 0);
 
 	if (index > 0)
 	{
-		MainWindow *mainWindow = MainWindow::findMainWindow(parent());
+		MainWindow *mainWindow(MainWindow::findMainWindow(parent()));
 
 		if (mainWindow)
 		{
@@ -736,7 +818,7 @@ void Menu::restoreClosedWindow()
 
 void Menu::openBookmark()
 {
-	QWidget *menu = this;
+	QWidget *menu(this);
 
 	while (menu)
 	{
@@ -749,8 +831,8 @@ void Menu::openBookmark()
 		}
 	}
 
-	MainWindow *mainWindow = MainWindow::findMainWindow(parent());
-	QAction *action = qobject_cast<QAction*>(sender());
+	MainWindow *mainWindow(MainWindow::findMainWindow(parent()));
+	QAction *action(qobject_cast<QAction*>(sender()));
 
 	if (action && action->data().type() == QVariant::ModelIndex)
 	{
@@ -759,12 +841,12 @@ void Menu::openBookmark()
 
 	if (mainWindow)
 	{
-		const WindowsManager::OpenHints hints = (action ? static_cast<WindowsManager::OpenHints>(action->data().toInt()) : WindowsManager::DefaultOpen);
+		const WindowsManager::OpenHints hints(action ? static_cast<WindowsManager::OpenHints>(action->data().toInt()) : WindowsManager::DefaultOpen);
 
 		mainWindow->getWindowsManager()->open(m_bookmark, ((hints == WindowsManager::DefaultOpen) ? WindowsManager::calculateOpenHints() : hints));
 	}
 
-	m_bookmark = NULL;
+	m_bookmark = nullptr;
 }
 
 void Menu::openImporter(QAction *action)
@@ -779,24 +861,24 @@ void Menu::openSession(QAction *action)
 {
 	if (!action->data().isNull())
 	{
-		SessionsManager::restoreSession(SessionsManager::getSession(action->data().toString()), (SettingsManager::getValue(QLatin1String("Sessions/OpenInExistingWindow")).toBool() ? SessionsManager::getActiveWindow() : NULL));
+		SessionsManager::restoreSession(SessionsManager::getSession(action->data().toString()), (SettingsManager::getValue(SettingsManager::Sessions_OpenInExistingWindowOption).toBool() ? SessionsManager::getActiveWindow() : nullptr));
 	}
 }
 
 void Menu::selectCharacterEncoding(QAction *action)
 {
-	MainWindow *mainWindow = MainWindow::findMainWindow(parent());
+	MainWindow *mainWindow(MainWindow::findMainWindow(parent()));
 
 	if (!mainWindow)
 	{
 		return;
 	}
 
-	QString encoding;
+	QString encoding(QLatin1String("auto"));
 
 	if (action && action->data().toInt() > 0)
 	{
-		QTextCodec *codec = QTextCodec::codecForMib(action->data().toInt());
+		QTextCodec *codec(QTextCodec::codecForMib(action->data().toInt()));
 
 		if (codec)
 		{
@@ -804,19 +886,19 @@ void Menu::selectCharacterEncoding(QAction *action)
 		}
 	}
 
-	mainWindow->getWindowsManager()->setOption(QLatin1String("Content/DefaultCharacterEncoding"), encoding.toLower());
+	mainWindow->getWindowsManager()->setOption(SettingsManager::Content_DefaultCharacterEncodingOption, encoding.toLower());
 }
 
 void Menu::selectStyleSheet(QAction *action)
 {
-	MainWindow *mainWindow = MainWindow::findMainWindow(parent());
+	MainWindow *mainWindow(MainWindow::findMainWindow(parent()));
 
 	if (!mainWindow)
 	{
 		return;
 	}
 
-	Window *window = mainWindow->getWindowsManager()->getWindowByIndex(mainWindow->getWindowsManager()->getWindowIndex(-1));
+	Window *window(mainWindow->getWindowsManager()->getWindowByIndex(mainWindow->getWindowsManager()->getWindowIndex(-1)));
 
 	if (window && action)
 	{
@@ -826,11 +908,11 @@ void Menu::selectStyleSheet(QAction *action)
 
 void Menu::selectUserAgent(QAction *action)
 {
-	MainWindow *mainWindow = MainWindow::findMainWindow(parent());
+	MainWindow *mainWindow(MainWindow::findMainWindow(parent()));
 
 	if (action && mainWindow)
 	{
-		mainWindow->getWindowsManager()->setOption(QLatin1String("Network/UserAgent"), action->data().toString());
+		mainWindow->getWindowsManager()->setOption(SettingsManager::Network_UserAgentOption, action->data().toString());
 	}
 }
 
@@ -847,19 +929,19 @@ void Menu::selectWindow(QAction *action)
 
 void Menu::updateClosedWindowsMenu()
 {
-	MainWindow *mainWindow = MainWindow::findMainWindow(parent());
+	MainWindow *mainWindow(MainWindow::findMainWindow(parent()));
 
 	setEnabled((mainWindow && mainWindow->getWindowsManager()->getClosedWindows().count() > 0) || SessionsManager::getClosedWindows().count() > 0);
 }
 
 void Menu::setToolBarVisibility(bool visible)
 {
-	QAction *action = qobject_cast<QAction*>(sender());
+	QAction *action(qobject_cast<QAction*>(sender()));
 
 	if (action)
 	{
-		ToolBarsManager::ToolBarDefinition definition = ToolBarsManager::getToolBarDefinition(action->data().toInt());
-		definition.visibility = (visible ? ToolBarsManager::AlwaysVisibleToolBar : ToolBarsManager::AlwaysHiddenToolBar);
+		ToolBarsManager::ToolBarDefinition definition(ToolBarsManager::getToolBarDefinition(action->data().toInt()));
+		definition.normalVisibility = (visible ? ToolBarsManager::AlwaysVisibleToolBar : ToolBarsManager::AlwaysHiddenToolBar);
 
 		ToolBarsManager::setToolBar(definition);
 	}
@@ -867,7 +949,7 @@ void Menu::setToolBarVisibility(bool visible)
 
 Action* Menu::addAction(int identifier, bool useGlobal)
 {
-	Action *action = (useGlobal ? ActionsManager::getAction(identifier, this) : new Action(identifier, this));
+	Action *action(useGlobal ? ActionsManager::getAction(identifier, this) : new Action(identifier, this));
 
 	QMenu::addAction(action);
 
