@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2014 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -36,7 +36,7 @@ public:
 
 public slots:
 	void setPosition(const QPoint &position);
-	void setPreview(const QString &text, const QPixmap &pixmap = QPixmap());
+	void setPreview(const QString &text, const QPixmap &pixmap = QPixmap(), bool showFullText = false);
 
 private:
 	QLabel *m_textLabel;
