@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2013 - 2014 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2013 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #ifndef OTTER_TEXTLABELWIDGET_H
 #define OTTER_TEXTLABELWIDGET_H
 
+#include <QtCore/QUrl>
 #include <QtWidgets/QLineEdit>
 
 namespace Otter
@@ -30,9 +31,27 @@ class TextLabelWidget : public QLineEdit
 	Q_OBJECT
 
 public:
-	explicit TextLabelWidget(QWidget *parent = NULL);
+	explicit TextLabelWidget(QWidget *parent = nullptr);
 
+	bool event(QEvent *event);
+
+public slots:
+	void clear();
 	void setText(const QString &text);
+	void setUrl(const QUrl &url);
+
+protected:
+	void mousePressEvent(QMouseEvent *event);
+	void mouseReleaseEvent(QMouseEvent *event);
+	void contextMenuEvent(QContextMenuEvent *event);
+	void updateStyle();
+
+protected slots:
+	void copyUrl();
+
+private:
+	QUrl m_url;
+	QPoint m_dragStartPosition;
 };
 
 }
