@@ -1,6 +1,6 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
 #include "../core/BookmarksManager.h"
 #include "../core/BookmarksModel.h"
 #include "../core/SearchEnginesManager.h"
-#include "../core/Utils.h"
+#include "../core/ThemesManager.h"
 
 #include "ui_ToolBarDialog.h"
 
@@ -33,81 +33,60 @@
 namespace Otter
 {
 
-ToolBarDialog::ToolBarDialog(int identifier, QWidget *parent) : Dialog(parent),
-	m_definition(ToolBarsManager::getToolBarDefinition(identifier)),
+ToolBarDialog::ToolBarDialog(const ToolBarsManager::ToolBarDefinition &definition, QWidget *parent) : Dialog(parent),
+	m_definition(definition),
 	m_ui(new Ui::ToolBarDialog)
 {
 	m_ui->setupUi(this);
-	m_ui->removeButton->setIcon(Utils::getIcon(QLatin1String("go-previous")));
-	m_ui->addButton->setIcon(Utils::getIcon(QLatin1String("go-next")));
-	m_ui->moveUpButton->setIcon(Utils::getIcon(QLatin1String("go-up")));
-	m_ui->moveDownButton->setIcon(Utils::getIcon(QLatin1String("go-down")));
+	m_ui->removeButton->setIcon(ThemesManager::getIcon(QGuiApplication::isLeftToRight() ? QLatin1String("go-previous") : QLatin1String("go-next")));
+	m_ui->addButton->setIcon(ThemesManager::getIcon(QGuiApplication::isLeftToRight() ? QLatin1String("go-next") : QLatin1String("go-previous")));
+	m_ui->moveUpButton->setIcon(ThemesManager::getIcon(QLatin1String("go-up")));
+	m_ui->moveDownButton->setIcon(ThemesManager::getIcon(QLatin1String("go-down")));
 	m_ui->addEntryButton->setMenu(new QMenu(m_ui->addEntryButton));
 	m_ui->buttonBox->button(QDialogButtonBox::RestoreDefaults)->setEnabled(m_definition.canReset);
-	m_ui->titleLineEdit->setText(m_definition.title.isEmpty() ? tr("Custom Toolbar") : m_definition.title);
+	m_ui->titleLineEdit->setText(m_definition.getTitle().isEmpty() ? tr("Custom Toolbar") : m_definition.getTitle());
 	m_ui->iconSizeSpinBox->setValue(qMax(0, m_definition.iconSize));
 	m_ui->maximumButtonSizeSpinBox->setValue(qMax(0, m_definition.maximumButtonSize));
+	m_ui->normalVisibilityComboBox->addItem(tr("Always visible"), ToolBarsManager::AlwaysVisibleToolBar);
+	m_ui->normalVisibilityComboBox->addItem(tr("Always hidden"), ToolBarsManager::AlwaysHiddenToolBar);
+	m_ui->normalVisibilityComboBox->addItem(tr("Visible only when needed"), ToolBarsManager::AutoVisibilityToolBar);
 
-	switch (m_definition.visibility)
-	{
-		case ToolBarsManager::AlwaysHiddenToolBar:
-			m_ui->visibilityComboBox->setCurrentIndex(1);
+	const int normalVisibilityIndex(m_ui->normalVisibilityComboBox->findData(m_definition.normalVisibility));
 
-			break;
-		case ToolBarsManager::AutoVisibilityToolBar:
-			m_ui->visibilityComboBox->setCurrentIndex(2);
+	m_ui->normalVisibilityComboBox->setCurrentIndex((normalVisibilityIndex < 0) ? 0 : normalVisibilityIndex);
+	m_ui->fullScreenVisibilityComboBox->addItem(tr("Always visible"), ToolBarsManager::AlwaysVisibleToolBar);
+	m_ui->fullScreenVisibilityComboBox->addItem(tr("Always hidden"), ToolBarsManager::AlwaysHiddenToolBar);
+	m_ui->fullScreenVisibilityComboBox->addItem(tr("Visible only when needed"), ToolBarsManager::AutoVisibilityToolBar);
+	m_ui->fullScreenVisibilityComboBox->addItem(tr("Visible only when cursor is close to screen edge"), ToolBarsManager::OnHoverVisibleToolBar);
 
-			break;
-		default:
-			m_ui->visibilityComboBox->setCurrentIndex(0);
+	const int fullScreenVisibilityIndex(m_ui->fullScreenVisibilityComboBox->findData(m_definition.fullScreenVisibility));
 
-			break;
-	}
+	m_ui->fullScreenVisibilityComboBox->setCurrentIndex((fullScreenVisibilityIndex < 0) ? 1 : fullScreenVisibilityIndex);
+	m_ui->buttonStyleComboBox->addItem(tr("Follow style"), Qt::ToolButtonFollowStyle);
+	m_ui->buttonStyleComboBox->addItem(tr("Icon only"), Qt::ToolButtonIconOnly);
+	m_ui->buttonStyleComboBox->addItem(tr("Text only"), Qt::ToolButtonTextOnly);
+	m_ui->buttonStyleComboBox->addItem(tr("Text beside icon"), Qt::ToolButtonTextBesideIcon);
+	m_ui->buttonStyleComboBox->addItem(tr("Text under icon"), Qt::ToolButtonTextUnderIcon);
 
-	switch (m_definition.buttonStyle)
-	{
-		case Qt::ToolButtonFollowStyle:
-			m_ui->buttonStyleComboBox->setCurrentIndex(0);
+	const int buttonStyleIndex(m_ui->buttonStyleComboBox->findData(m_definition.buttonStyle));
 
-			break;
-		case Qt::ToolButtonTextOnly:
-			m_ui->buttonStyleComboBox->setCurrentIndex(2);
-
-			break;
-		case Qt::ToolButtonTextBesideIcon:
-			m_ui->buttonStyleComboBox->setCurrentIndex(3);
-
-			break;
-		case Qt::ToolButtonTextUnderIcon:
-			m_ui->buttonStyleComboBox->setCurrentIndex(4);
-
-			break;
-		default:
-			m_ui->buttonStyleComboBox->setCurrentIndex(1);
-
-			break;
-	}
+	m_ui->buttonStyleComboBox->setCurrentIndex((buttonStyleIndex < 0) ? 1 : buttonStyleIndex);
 
 	if (m_definition.bookmarksPath.isEmpty())
 	{
-		m_ui->folderLabel->setParent(NULL);
-		m_ui->folderLabel->deleteLater();
-		m_ui->folderComboBox->setParent(NULL);
-		m_ui->folderComboBox->deleteLater();
+		m_ui->bookmarksWidget->hide();
 	}
 	else
 	{
 		m_ui->folderComboBox->setCurrentFolder(BookmarksManager::getModel()->getItem(m_definition.bookmarksPath));
-		m_ui->optionsHeader->hide();
-		m_ui->arrangementWidget->hide();
+		m_ui->entriesEditorWidget->hide();
 	}
 
-	QStandardItemModel *availableEntriesModel = new QStandardItemModel(this);
+	QStandardItemModel *availableEntriesModel(new QStandardItemModel(this));
 	availableEntriesModel->appendRow(createEntry(QLatin1String("separator")));
 	availableEntriesModel->appendRow(createEntry(QLatin1String("spacer")));
 
-	QStringList widgets;
-	widgets << QLatin1String("CustomMenu") << QLatin1String("ClosedWindowsMenu") << QLatin1String("AddressWidget") << QLatin1String("MenuButtonWidget") << QLatin1String("PanelChooserWidget") << QLatin1String("SearchWidget") << QLatin1String("StatusMessageWidget") << QLatin1String("ZoomWidget");
+	const QStringList widgets({QLatin1String("CustomMenu"), QLatin1String("ClosedWindowsMenu"), QLatin1String("AddressWidget"), QLatin1String("ContentBlockingInformationWidget"), QLatin1String("MenuButtonWidget"), QLatin1String("PanelChooserWidget"), QLatin1String("SearchWidget"), QLatin1String("StatusMessageWidget"), QLatin1String("ZoomWidget")});
 
 	for (int i = 0; i < widgets.count(); ++i)
 	{
@@ -115,7 +94,7 @@ ToolBarDialog::ToolBarDialog(int identifier, QWidget *parent) : Dialog(parent),
 
 		if (widgets.at(i) == QLatin1String("SearchWidget"))
 		{
-			const QStringList searchEngines = SearchEnginesManager::getSearchEngines();
+			const QStringList searchEngines(SearchEnginesManager::getSearchEngines());
 
 			for (int j = 0; j < searchEngines.count(); ++j)
 			{
@@ -127,20 +106,26 @@ ToolBarDialog::ToolBarDialog(int identifier, QWidget *parent) : Dialog(parent),
 		}
 	}
 
-	const QVector<ActionsManager::ActionDefinition> actions = ActionsManager::getActionDefinitions();
+	const QVector<ActionsManager::ActionDefinition> actions(ActionsManager::getActionDefinitions());
 
 	for (int i = 0; i < actions.count(); ++i)
 	{
-		QStandardItem *item = new QStandardItem(actions.at(i).icon, QCoreApplication::translate("actions", (actions.at(i).description.isEmpty() ? actions.at(i).text : actions.at(i).description).toUtf8().constData()));
-		item->setData(ActionsManager::getActionName(actions.at(i).identifier) + QLatin1String("Action"), Qt::UserRole);
-		item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemNeverHasChildren);
+		QStandardItem *item(new QStandardItem(QCoreApplication::translate("actions", (actions.at(i).description.isEmpty() ? actions.at(i).text : actions.at(i).description).toUtf8().constData())));
+		item->setData(QColor(Qt::transparent), Qt::DecorationRole);
+		item->setData(ActionsManager::getActionName(actions.at(i).identifier) + QLatin1String("Action"), IdentifierRole);
+		item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled | Qt::ItemNeverHasChildren);
 		item->setToolTip(item->text());
 
+		if (!actions.at(i).icon.isNull())
+		{
+			item->setIcon(actions.at(i).icon);
+		}
+
 		availableEntriesModel->appendRow(item);
 	}
 
 	m_ui->availableEntriesItemView->setModel(availableEntriesModel);
-	m_ui->availableEntriesItemView->viewport()->setAcceptDrops(false);
+	m_ui->availableEntriesItemView->viewport()->installEventFilter(this);
 	m_ui->currentEntriesItemView->setModel(new QStandardItemModel(this));
 	m_ui->currentEntriesItemView->setViewMode(ItemViewWidget::TreeViewMode);
 	m_ui->currentEntriesItemView->setRootIsDecorated(false);
@@ -152,7 +137,7 @@ ToolBarDialog::ToolBarDialog(int identifier, QWidget *parent) : Dialog(parent),
 
 	m_definition.entries.clear();
 
-	Menu *bookmarksMenu = new Menu(Menu::BookmarkSelectorMenuRole, m_ui->addEntryButton);
+	Menu *bookmarksMenu(new Menu(Menu::BookmarkSelectorMenuRole, m_ui->addEntryButton));
 
 	m_ui->addEntryButton->setMenu(bookmarksMenu);
 	m_ui->addEntryButton->setEnabled(BookmarksManager::getModel()->getRootItem()->rowCount() > 0);
@@ -189,7 +174,7 @@ void ToolBarDialog::changeEvent(QEvent *event)
 
 void ToolBarDialog::addEntry(const ActionsManager::ActionEntryDefinition &entry, QStandardItem *parent)
 {
-	QStandardItem *item = createEntry(entry.action, entry.options);
+	QStandardItem *item(createEntry(entry.action, entry.options));
 
 	if (parent)
 	{
@@ -197,7 +182,7 @@ void ToolBarDialog::addEntry(const ActionsManager::ActionEntryDefinition &entry,
 	}
 	else
 	{
-		m_ui->currentEntriesItemView->getModel()->appendRow(item);
+		m_ui->currentEntriesItemView->getSourceModel()->appendRow(item);
 	}
 
 	if (entry.action == QLatin1String("CustomMenu"))
@@ -213,21 +198,21 @@ void ToolBarDialog::addEntry(const ActionsManager::ActionEntryDefinition &entry,
 
 void ToolBarDialog::addEntry()
 {
-	QStandardItem *sourceItem = m_ui->availableEntriesItemView->getItem(m_ui->availableEntriesItemView->getCurrentRow());
+	QStandardItem *sourceItem(m_ui->availableEntriesItemView->getItem(m_ui->availableEntriesItemView->getCurrentRow()));
 
 	if (sourceItem)
 	{
-		QStandardItem *newItem = sourceItem->clone();
-		QStandardItem *targetItem = m_ui->currentEntriesItemView->getItem(m_ui->currentEntriesItemView->currentIndex());
+		QStandardItem *newItem(sourceItem->clone());
+		QStandardItem *targetItem(m_ui->currentEntriesItemView->getItem(m_ui->currentEntriesItemView->currentIndex()));
 
-		if (targetItem && targetItem->data(Qt::UserRole).toString() != QLatin1String("CustomMenu"))
+		if (targetItem && targetItem->data(IdentifierRole).toString() != QLatin1String("CustomMenu"))
 		{
 			targetItem = targetItem->parent();
 		}
 
-		if (targetItem && targetItem->data(Qt::UserRole).toString() == QLatin1String("CustomMenu"))
+		if (targetItem && targetItem->data(IdentifierRole).toString() == QLatin1String("CustomMenu"))
 		{
-			int row = -1;
+			int row(-1);
 
 			if (targetItem->index() != m_ui->currentEntriesItemView->currentIndex())
 			{
@@ -255,16 +240,16 @@ void ToolBarDialog::addEntry()
 
 void ToolBarDialog::editEntry()
 {
-	const QModelIndex index = m_ui->currentEntriesItemView->currentIndex();
-	const QString identifier = index.data(Qt::UserRole).toString();
-	QVariantMap options = index.data(Qt::UserRole + 1).toMap();
+	const QModelIndex index(m_ui->currentEntriesItemView->currentIndex());
+	const QString identifier(index.data(IdentifierRole).toString());
+	QVariantMap options(index.data(OptionsRole).toMap());
 	QList<QPair<QString, OptionWidget*> > widgets;
 
 	if (identifier == QLatin1String("SearchWidget"))
 	{
-		const QStringList searchEngines = SearchEnginesManager::getSearchEngines();
+		const QStringList searchEngines(SearchEnginesManager::getSearchEngines());
 		QList<OptionWidget::EnumerationChoice> searchEngineChoices;
-		OptionWidget *searchEngineWidget = new OptionWidget(QLatin1String("searchEngine"), options.value(QLatin1String("searchEngine")), OptionWidget::EnumerationType, this);
+		OptionWidget *searchEngineWidget(new OptionWidget(QLatin1String("searchEngine"), options.value(QLatin1String("searchEngine")), SettingsManager::EnumerationType, this));
 		OptionWidget::EnumerationChoice defaultSearchEngineChoice;
 		defaultSearchEngineChoice.text = tr("All");
 
@@ -272,7 +257,7 @@ void ToolBarDialog::editEntry()
 
 		for (int i = 0; i < searchEngines.count(); ++i)
 		{
-			const SearchEnginesManager::SearchEngineDefinition searchEngine = SearchEnginesManager::getSearchEngine(searchEngines.at(i));
+			const SearchEnginesManager::SearchEngineDefinition searchEngine(SearchEnginesManager::getSearchEngine(searchEngines.at(i)));
 			OptionWidget::EnumerationChoice searchEngineChoice;
 			searchEngineChoice.text = (searchEngine.title.isEmpty() ? tr("Unknown") : searchEngine.title);
 			searchEngineChoice.value = searchEngines.at(i);
@@ -284,12 +269,12 @@ void ToolBarDialog::editEntry()
 		searchEngineWidget->setChoices(searchEngineChoices);
 
 		widgets.append(qMakePair(tr("Show search engine:"), searchEngineWidget));
-		widgets.append(qMakePair(tr("Show search button:"), new OptionWidget(QLatin1String("showSearchButton"), options.value(QLatin1String("showSearchButton"), true), OptionWidget::BooleanType, this)));
+		widgets.append(qMakePair(tr("Show search button:"), new OptionWidget(QLatin1String("showSearchButton"), options.value(QLatin1String("showSearchButton"), true), SettingsManager::BooleanType, this)));
 	}
-	else if (identifier == QLatin1String("MenuButtonWidget") || identifier == QLatin1String("PanelChooserWidget") || identifier.startsWith(QLatin1String("bookmarks:")) || identifier.endsWith(QLatin1String("Action")) || identifier.endsWith(QLatin1String("Menu")))
+	else if (identifier == QLatin1String("ContentBlockingInformationWidget") || identifier == QLatin1String("MenuButtonWidget") || identifier == QLatin1String("PanelChooserWidget") || identifier.startsWith(QLatin1String("bookmarks:")) || identifier.endsWith(QLatin1String("Action")) || identifier.endsWith(QLatin1String("Menu")))
 	{
-		widgets.append(qMakePair(tr("Custom icon:"), new OptionWidget(QLatin1String("icon"), options.value(QLatin1String("icon")), OptionWidget::IconType, this)));
-		widgets.append(qMakePair(tr("Custom text:"), new OptionWidget(QLatin1String("text"), options.value(QLatin1String("text")), OptionWidget::StringType, this)));
+		widgets.append(qMakePair(tr("Custom icon:"), new OptionWidget(QLatin1String("icon"), options.value(QLatin1String("icon")), SettingsManager::IconType, this)));
+		widgets.append(qMakePair(tr("Custom text:"), new OptionWidget(QLatin1String("text"), options.value(QLatin1String("text")), SettingsManager::StringType, this)));
 	}
 
 	if (widgets.isEmpty())
@@ -300,12 +285,12 @@ void ToolBarDialog::editEntry()
 	QDialog dialog(this);
 	dialog.setWindowTitle(tr("Edit Entry"));
 
-	QDialogButtonBox *buttonBox = new QDialogButtonBox(&dialog);
+	QDialogButtonBox *buttonBox(new QDialogButtonBox(&dialog));
 	buttonBox->addButton(QDialogButtonBox::Cancel);
 	buttonBox->addButton(QDialogButtonBox::Ok);
 
-	QFormLayout *formLayout = new QFormLayout();
-	QVBoxLayout *mainLayout = new QVBoxLayout(&dialog);
+	QFormLayout *formLayout(new QFormLayout());
+	QVBoxLayout *mainLayout(new QVBoxLayout(&dialog));
 	mainLayout->addLayout(formLayout);
 	mainLayout->addWidget(buttonBox);
 
@@ -334,12 +319,12 @@ void ToolBarDialog::editEntry()
 		}
 	}
 
-	QStandardItem *item = createEntry(identifier, options);
+	QStandardItem *item(createEntry(identifier, options));
 
 	m_ui->currentEntriesItemView->setData(index, item->text(), Qt::DisplayRole);
 	m_ui->currentEntriesItemView->setData(index, item->text(), Qt::ToolTipRole);
 	m_ui->currentEntriesItemView->setData(index, item->icon(), Qt::DecorationRole);
-	m_ui->currentEntriesItemView->setData(index, options, (Qt::UserRole + 1));
+	m_ui->currentEntriesItemView->setData(index, options, OptionsRole);
 
 	delete item;
 }
@@ -361,18 +346,19 @@ void ToolBarDialog::restoreDefaults()
 
 void ToolBarDialog::updateActions()
 {
-	const QString sourceIdentifier = m_ui->availableEntriesItemView->currentIndex().data(Qt::UserRole).toString();
-	const QString targetIdentifier = m_ui->currentEntriesItemView->currentIndex().data(Qt::UserRole).toString();
+	const QString sourceIdentifier(m_ui->availableEntriesItemView->currentIndex().data(IdentifierRole).toString());
+	const QString targetIdentifier(m_ui->currentEntriesItemView->currentIndex().data(IdentifierRole).toString());
 
-	m_ui->addButton->setEnabled(!sourceIdentifier.isEmpty() && (!(m_ui->currentEntriesItemView->currentIndex().data(Qt::UserRole).toString() == QLatin1String("CustomMenu") || m_ui->currentEntriesItemView->currentIndex().parent().data(Qt::UserRole).toString() == QLatin1String("CustomMenu")) || (sourceIdentifier == QLatin1String("separator") || sourceIdentifier.endsWith(QLatin1String("Action")) || sourceIdentifier.endsWith(QLatin1String("Menu")))));
+	m_ui->addButton->setEnabled(!sourceIdentifier.isEmpty() && (!(m_ui->currentEntriesItemView->currentIndex().data(IdentifierRole).toString() == QLatin1String("CustomMenu") || m_ui->currentEntriesItemView->currentIndex().parent().data(IdentifierRole).toString() == QLatin1String("CustomMenu")) || (sourceIdentifier == QLatin1String("separator") || sourceIdentifier.endsWith(QLatin1String("Action")) || sourceIdentifier.endsWith(QLatin1String("Menu")))));
 	m_ui->removeButton->setEnabled(m_ui->currentEntriesItemView->currentIndex().isValid() && targetIdentifier != QLatin1String("MenuBarWidget") && targetIdentifier != QLatin1String("TabBarWidget"));
-	m_ui->editEntryButton->setEnabled(targetIdentifier == QLatin1String("MenuButtonWidget") || targetIdentifier == QLatin1String("PanelChooserWidget") || targetIdentifier == QLatin1String("SearchWidget") || targetIdentifier.startsWith(QLatin1String("bookmarks:")) || targetIdentifier.endsWith(QLatin1String("Action")) || targetIdentifier.endsWith(QLatin1String("Menu")));
+	m_ui->editEntryButton->setEnabled(targetIdentifier == QLatin1String("ContentBlockingInformationWidget") || targetIdentifier == QLatin1String("MenuButtonWidget") || targetIdentifier == QLatin1String("PanelChooserWidget") || targetIdentifier == QLatin1String("SearchWidget") || targetIdentifier.startsWith(QLatin1String("bookmarks:")) || targetIdentifier.endsWith(QLatin1String("Action")) || targetIdentifier.endsWith(QLatin1String("Menu")));
 }
 
 QStandardItem* ToolBarDialog::createEntry(const QString &identifier, const QVariantMap &options)
 {
-	QStandardItem *item = new QStandardItem();
-	item->setData(identifier, Qt::UserRole);
+	QStandardItem *item(new QStandardItem());
+	item->setData(QColor(Qt::transparent), Qt::DecorationRole);
+	item->setData(identifier, IdentifierRole);
 	item->setFlags(Qt::ItemIsSelectable | Qt::ItemIsEnabled | Qt::ItemIsDragEnabled | Qt::ItemNeverHasChildren);
 
 	if (identifier == QLatin1String("separator"))
@@ -398,6 +384,10 @@ QStandardItem* ToolBarDialog::createEntry(const QString &identifier, const QVari
 	{
 		item->setText(tr("Address Field"));
 	}
+	else if (identifier == QLatin1String("ContentBlockingInformationWidget"))
+	{
+		item->setText(tr("Content Blocking Details"));
+	}
 	else if (identifier == QLatin1String("MenuBarWidget"))
 	{
 		item->setText(tr("Menu Bar"));
@@ -410,14 +400,42 @@ QStandardItem* ToolBarDialog::createEntry(const QString &identifier, const QVari
 	{
 		item->setText(tr("Sidebar Panel Chooser"));
 	}
+	else if (identifier == QLatin1String("ProgressInformationDocumentProgressWidget"))
+	{
+		item->setText(tr("Progress Information (Document Progress)"));
+	}
+	else if (identifier == QLatin1String("ProgressInformationTotalSizeWidget"))
+	{
+		item->setText(tr("Progress Information (Total Progress)"));
+	}
+	else if (identifier == QLatin1String("ProgressInformationElementsWidget"))
+	{
+		item->setText(tr("Progress Information (Loaded Elements)"));
+	}
+	else if (identifier == QLatin1String("ProgressInformationSpeedWidget"))
+	{
+		item->setText(tr("Progress Information (Loading Speed)"));
+	}
+	else if (identifier == QLatin1String("ProgressInformationElapsedTimeWidget"))
+	{
+		item->setText(tr("Progress Information (Elapsed Time)"));
+	}
+	else if (identifier == QLatin1String("ProgressInformationMessageWidget"))
+	{
+		item->setText(tr("Progress Information (Status Message)"));
+	}
 	else if (identifier == QLatin1String("SearchWidget"))
 	{
 		if (options.contains(QLatin1String("searchEngine")))
 		{
-			const SearchEnginesManager::SearchEngineDefinition definition = SearchEnginesManager::getSearchEngine(options[QLatin1String("searchEngine")].toString());
+			const SearchEnginesManager::SearchEngineDefinition definition(SearchEnginesManager::getSearchEngine(options[QLatin1String("searchEngine")].toString()));
 
 			item->setText(tr("Search Field (%1)").arg(definition.title.isEmpty() ? tr("Unknown") : definition.title));
-			item->setIcon(definition.icon);
+
+			if (!definition.icon.isNull())
+			{
+				item->setIcon(definition.icon);
+			}
 		}
 		else
 		{
@@ -438,12 +456,18 @@ QStandardItem* ToolBarDialog::createEntry(const QString &identifier, const QVari
 	}
 	else if (identifier.startsWith(QLatin1String("bookmarks:")))
 	{
-		BookmarksItem *bookmark = (identifier.startsWith(QLatin1String("bookmarks:/")) ? BookmarksManager::getModel()->getItem(identifier.mid(11)) : BookmarksManager::getBookmark(identifier.mid(10).toULongLong()));
+		BookmarksItem *bookmark(identifier.startsWith(QLatin1String("bookmarks:/")) ? BookmarksManager::getModel()->getItem(identifier.mid(11)) : BookmarksManager::getBookmark(identifier.mid(10).toULongLong()));
 
 		if (bookmark)
 		{
+			const QIcon icon(bookmark->data(Qt::DecorationRole).value<QIcon>());
+
 			item->setText(bookmark->data(BookmarksModel::TitleRole).isValid() ? bookmark->data(BookmarksModel::TitleRole).toString() : tr("(Untitled)"));
-			item->setIcon(bookmark->data(Qt::DecorationRole).value<QIcon>());
+
+			if (!icon.isNull())
+			{
+				item->setIcon(icon);
+			}
 		}
 		else
 		{
@@ -452,7 +476,7 @@ QStandardItem* ToolBarDialog::createEntry(const QString &identifier, const QVari
 	}
 	else if (identifier.endsWith(QLatin1String("Action")))
 	{
-		const int actionIdentifier = ActionsManager::getActionIdentifier(identifier.left(identifier.length() - 6));
+		const int actionIdentifier(ActionsManager::getActionIdentifier(identifier.left(identifier.length() - 6)));
 
 		if (actionIdentifier < 0)
 		{
@@ -460,10 +484,14 @@ QStandardItem* ToolBarDialog::createEntry(const QString &identifier, const QVari
 		}
 		else
 		{
-			const ActionsManager::ActionDefinition definition = ActionsManager::getActionDefinition(actionIdentifier);
+			const ActionsManager::ActionDefinition definition(ActionsManager::getActionDefinition(actionIdentifier));
 
 			item->setText(QCoreApplication::translate("actions", (definition.description.isEmpty() ? definition.text : definition.description).toUtf8().constData()));
-			item->setIcon(definition.icon);
+
+			if (!definition.icon.isNull())
+			{
+				item->setIcon(definition.icon);
+			}
 		}
 	}
 	else
@@ -473,19 +501,25 @@ QStandardItem* ToolBarDialog::createEntry(const QString &identifier, const QVari
 
 	if (!options.isEmpty())
 	{
-		item->setData(options, (Qt::UserRole + 1));
+		item->setData(options, OptionsRole);
 
 		if (options.contains(QLatin1String("icon")))
 		{
-			const QString data = options[QLatin1String("icon")].toString();
+			const QString data(options[QLatin1String("icon")].toString());
+			QIcon icon;
 
 			if (data.startsWith(QLatin1String("data:image/")))
 			{
-				item->setIcon(QIcon(QPixmap::fromImage(QImage::fromData(QByteArray::fromBase64(data.mid(data.indexOf(QLatin1String("base64,")) + 7).toUtf8())))));
+				icon = QIcon(QPixmap::fromImage(QImage::fromData(QByteArray::fromBase64(data.mid(data.indexOf(QLatin1String("base64,")) + 7).toUtf8()))));
 			}
 			else
 			{
-				item->setIcon(Utils::getIcon(data));
+				icon = ThemesManager::getIcon(data);
+			}
+
+			if (!icon.isNull())
+			{
+				item->setIcon(icon);
 			}
 		}
 
@@ -509,8 +543,8 @@ ActionsManager::ActionEntryDefinition ToolBarDialog::getEntry(QStandardItem *ite
 		return definition;
 	}
 
-	definition.action = item->data(Qt::UserRole).toString();
-	definition.options = item->data(Qt::UserRole + 1).toMap();
+	definition.action = item->data(IdentifierRole).toString();
+	definition.options = item->data(OptionsRole).toMap();
 
 	if (definition.action == QLatin1String("CustomMenu"))
 	{
@@ -527,49 +561,12 @@ ToolBarsManager::ToolBarDefinition ToolBarDialog::getDefinition() const
 {
 	ToolBarsManager::ToolBarDefinition definition(m_definition);
 	definition.title = m_ui->titleLineEdit->text();
+	definition.normalVisibility = static_cast<ToolBarsManager::ToolBarVisibility>(m_ui->normalVisibilityComboBox->currentData().toInt());
+	definition.fullScreenVisibility = static_cast<ToolBarsManager::ToolBarVisibility>(m_ui->fullScreenVisibilityComboBox->currentData().toInt());
+	definition.buttonStyle = static_cast<Qt::ToolButtonStyle>(m_ui->buttonStyleComboBox->currentData().toInt());
 	definition.iconSize = m_ui->iconSizeSpinBox->value();
 	definition.maximumButtonSize = m_ui->maximumButtonSizeSpinBox->value();
 
-	switch (m_ui->visibilityComboBox->currentIndex())
-	{
-		case 1:
-			definition.visibility = ToolBarsManager::AlwaysHiddenToolBar;
-
-			break;
-		case 2:
-			definition.visibility = ToolBarsManager::AutoVisibilityToolBar;
-
-			break;
-		default:
-			definition.visibility = ToolBarsManager::AlwaysVisibleToolBar;
-
-			break;
-	}
-
-	switch (m_ui->buttonStyleComboBox->currentIndex())
-	{
-		case 0:
-			definition.buttonStyle = Qt::ToolButtonFollowStyle;
-
-			break;
-		case 2:
-			definition.buttonStyle = Qt::ToolButtonTextOnly;
-
-			break;
-		case 3:
-			definition.buttonStyle = Qt::ToolButtonTextBesideIcon;
-
-			break;
-		case 4:
-			definition.buttonStyle = Qt::ToolButtonTextUnderIcon;
-
-			break;
-		default:
-			definition.buttonStyle = Qt::ToolButtonIconOnly;
-
-			break;
-	}
-
 	for (int i = 0; i < m_ui->currentEntriesItemView->model()->rowCount(); ++i)
 	{
 		definition.entries.append(getEntry(m_ui->currentEntriesItemView->getItem(i)));
@@ -583,4 +580,16 @@ ToolBarsManager::ToolBarDefinition ToolBarDialog::getDefinition() const
 	return definition;
 }
 
+bool ToolBarDialog::eventFilter(QObject *object, QEvent *event)
+{
+	if (object == m_ui->availableEntriesItemView->viewport() && event->type() == QEvent::Drop)
+	{
+		event->accept();
+
+		return true;
+	}
+
+	return Dialog::eventFilter(object, event);;
+}
+
 }
