@@ -1,6 +1,7 @@
 /**************************************************************************
 * Otter Browser: Web browser controlled by the user, not vice-versa.
-* Copyright (C) 2015 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2015 - 2016 Michal Dutkiewicz aka Emdek <michal@emdek.pl>
+* Copyright (C) 2016 Piotr Wójcik <chocimier@tlen.pl>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -23,27 +24,22 @@
 #include "Menu.h"
 #include "TabBarWidget.h"
 #include "ToolBarAreaWidget.h"
+#include "WidgetFactory.h"
 #include "Window.h"
-#include "toolbars/ActionWidget.h"
-#include "toolbars/AddressWidget.h"
-#include "toolbars/BookmarkWidget.h"
-#include "toolbars/GoBackActionWidget.h"
-#include "toolbars/GoForwardActionWidget.h"
-#include "toolbars/MenuButtonWidget.h"
-#include "toolbars/PanelChooserWidget.h"
-#include "toolbars/SearchWidget.h"
-#include "toolbars/StatusMessageWidget.h"
-#include "toolbars/ZoomWidget.h"
 #include "../core/BookmarksManager.h"
 #include "../core/GesturesManager.h"
-#include "../core/Utils.h"
+#include "../core/ThemesManager.h"
 #include "../core/WindowsManager.h"
+#include "../modules/widgets/bookmark/BookmarkWidget.h"
 
 #include <QtCore/QMimeData>
+#include <QtCore/QTimer>
 #include <QtGui/QDrag>
 #include <QtGui/QMouseEvent>
 #include <QtGui/QPainter>
 #include <QtWidgets/QApplication>
+#include <QtWidgets/QStyle>
+#include <QtWidgets/QStyleOption>
 
 namespace Otter
 {
@@ -100,10 +96,10 @@ void ToolBarDragAreaWidget::mouseMoveEvent(QMouseEvent *event)
 
 	m_toolBar->startToolBarDragging();
 
-	QMimeData *mimeData = new QMimeData();
-	mimeData->setData(QLatin1String("x-toolbar-identifier"), QString::number(m_toolBar->getIdentifier()).toUtf8());
+	QMimeData *mimeData(new QMimeData());
+	mimeData->setProperty("x-toolbar-identifier", m_toolBar->getIdentifier());
 
-	QDrag *drag = new QDrag(this);
+	QDrag *drag(new QDrag(this));
 	drag->setMimeData(mimeData);
 	drag->exec(Qt::MoveAction);
 }
@@ -118,8 +114,8 @@ void ToolBarDragAreaWidget::mouseReleaseEvent(QMouseEvent *event)
 ToolBarWidget::ToolBarWidget(int identifier, Window *window, QWidget *parent) : QToolBar(parent),
 	m_mainWindow(MainWindow::findMainWindow(parent)),
 	m_window(window),
-	m_bookmark(NULL),
-	m_dragArea(NULL),
+	m_bookmark(nullptr),
+	m_dragArea(nullptr),
 	m_identifier(identifier)
 {
 	setStyleSheet(QLatin1String("QToolBar {padding:0 3px;spacing:3px;}"));
@@ -128,7 +124,7 @@ ToolBarWidget::ToolBarWidget(int identifier, Window *window, QWidget *parent) :
 
 	layout()->setContentsMargins(0, 0, 0, 0);
 
-	if (identifier >= 0)
+	if (identifier >= 0 && identifier != ToolBarsManager::MenuBar)
 	{
 		setToolBarLocked(ToolBarsManager::areToolBarsLocked());
 		reload();
@@ -138,17 +134,109 @@ ToolBarWidget::ToolBarWidget(int identifier, Window *window, QWidget *parent) :
 		connect(ToolBarsManager::getInstance(), SIGNAL(toolBarsLockedChanged(bool)), this, SLOT(setToolBarLocked(bool)));
 	}
 
-	if (m_mainWindow && m_identifier != ToolBarsManager::NavigationBar)
+	if (m_mainWindow && m_identifier != ToolBarsManager::NavigationBar && m_identifier != ToolBarsManager::ProgressBar)
 	{
-		connect(m_mainWindow->getWindowsManager(), SIGNAL(currentWindowChanged(qint64)), this, SLOT(notifyWindowChanged(qint64)));
+		connect(m_mainWindow->getWindowsManager(), SIGNAL(currentWindowChanged(quint64)), this, SLOT(notifyWindowChanged(quint64)));
+	}
+}
+
+void ToolBarWidget::changeEvent(QEvent *event)
+{
+	QToolBar::changeEvent(event);
+
+	if (event->type() == QEvent::StyleChange)
+	{
+		const int iconSize(getIconSize());
+
+		setIconSize(QSize(iconSize, iconSize));
+
+		emit iconSizeChanged(iconSize);
 	}
 }
 
 void ToolBarWidget::paintEvent(QPaintEvent *event)
 {
-	if (m_identifier >= 0 && m_identifier != ToolBarsManager::StatusBar)
+	if (m_identifier == ToolBarsManager::StatusBar)
 	{
-		QToolBar::paintEvent(event);
+		return;
+	}
+
+	QToolBar::paintEvent(event);
+
+	if (m_identifier != ToolBarsManager::TabBar)
+	{
+		return;
+	}
+
+	const Qt::ToolBarArea area(getArea());
+	QStyleOptionTab tabOption;
+
+	switch (area)
+	{
+		case Qt::BottomToolBarArea:
+			tabOption.shape = QTabBar::RoundedSouth;
+
+			break;
+		case Qt::LeftToolBarArea:
+			tabOption.shape = QTabBar::RoundedWest;
+
+			break;
+		case Qt::RightToolBarArea:
+			tabOption.shape = QTabBar::RoundedEast;
+
+			break;
+		default:
+			tabOption.shape = QTabBar::RoundedNorth;
+
+			break;
+	}
+
+	const int overlap(style()->pixelMetric(QStyle::PM_TabBarBaseOverlap, &tabOption));
+	QPainter painter(this);
+	QStyleOptionTabBarBase tabBarBaseOption;
+	tabBarBaseOption.initFrom(this);
+	tabBarBaseOption.documentMode = true;
+	tabBarBaseOption.rect = contentsRect();
+	tabBarBaseOption.shape = tabOption.shape;
+	tabBarBaseOption.tabBarRect = contentsRect();
+
+	if (overlap > 0)
+	{
+		const QSize size(contentsRect().size());
+
+		switch (area)
+		{
+			case Qt::BottomToolBarArea:
+				tabBarBaseOption.rect.setRect(0, 0, size.width(), overlap);
+
+				break;
+			case Qt::LeftToolBarArea:
+				tabBarBaseOption.rect.setRect((size.width() - overlap), 0, overlap, size.height());
+
+				break;
+			case Qt::RightToolBarArea:
+				tabBarBaseOption.rect.setRect(0, 0, overlap, size.height());
+
+				break;
+			default:
+				tabBarBaseOption.rect.setRect(0, (size.height() - overlap), size.width(), overlap);
+
+				break;
+		}
+	}
+
+	style()->drawPrimitive(QStyle::PE_FrameTabBarBase, &tabBarBaseOption, &painter, this);
+}
+
+void ToolBarWidget::resizeEvent(QResizeEvent *event)
+{
+	QToolBar::resizeEvent(event);
+
+	QWidget *tabBar((m_identifier == ToolBarsManager::TabBar && m_mainWindow) ? m_mainWindow->getTabBar() : nullptr);
+
+	if (tabBar)
+	{
+		QTimer::singleShot(200, tabBar, SLOT(updateSize()));
 	}
 }
 
@@ -161,34 +249,49 @@ void ToolBarWidget::contextMenuEvent(QContextMenuEvent *event)
 		return;
 	}
 
+	if (event->reason() == QContextMenuEvent::Mouse)
+	{
+		event->accept();
+
+		return;
+	}
+
 	if (m_identifier != ToolBarsManager::TabBar)
 	{
-		QMenu *menu = createCustomizationMenu(m_identifier);
+		QMenu *menu(createCustomizationMenu(m_identifier));
 		menu->exec(event->globalPos());
 		menu->deleteLater();
 
 		return;
 	}
 
-	QList<QAction*> actions;
-	QAction *cycleAction = new QAction(tr("Switch tabs using the mouse wheel"), this);
+	QAction *cycleAction(new QAction(tr("Switch Tabs Using the Mouse Wheel"), this));
 	cycleAction->setCheckable(true);
-	cycleAction->setChecked(!SettingsManager::getValue(QLatin1String("TabBar/RequireModifierToSwitchTabOnScroll")).toBool());
-	cycleAction->setEnabled(m_mainWindow->getTabBar());
+	cycleAction->setChecked(!SettingsManager::getValue(SettingsManager::TabBar_RequireModifierToSwitchTabOnScrollOption).toBool());
 
-	actions.append(cycleAction);
+	QAction *thumbnailsAction(new QAction(tr("Show Thumbnails in Tabs"), this));
+	thumbnailsAction->setCheckable(true);
+	thumbnailsAction->setChecked(SettingsManager::getValue(SettingsManager::TabBar_EnableThumbnailsOption).toBool());
 
-	if (m_mainWindow->getTabBar())
+	connect(cycleAction, &QAction::toggled, [&](bool isEnabled)
 	{
-		connect(cycleAction, SIGNAL(toggled(bool)), m_mainWindow->getTabBar(), SLOT(setCycle(bool)));
-	}
+		SettingsManager::setValue(SettingsManager::TabBar_RequireModifierToSwitchTabOnScrollOption, !isEnabled);
+	});
+	connect(thumbnailsAction, &QAction::toggled, [&](bool areEnabled)
+	{
+		SettingsManager::setValue(SettingsManager::TabBar_EnableThumbnailsOption, areEnabled);
+	});
+
+	QList<QAction*> actions;
+	actions.append(cycleAction);
+	actions.append(thumbnailsAction);
 
 	QMenu menu(this);
 	menu.addAction(ActionsManager::getAction(ActionsManager::NewTabAction, this));
 	menu.addAction(ActionsManager::getAction(ActionsManager::NewTabPrivateAction, this));
 	menu.addSeparator();
 
-	QMenu *arrangeMenu = menu.addMenu(tr("Arrange"));
+	QMenu *arrangeMenu(menu.addMenu(tr("Arrange")));
 	arrangeMenu->addAction(ActionsManager::getAction(ActionsManager::RestoreTabAction, this));
 	arrangeMenu->addSeparator();
 	arrangeMenu->addAction(ActionsManager::getAction(ActionsManager::RestoreAllAction, this));
@@ -202,11 +305,13 @@ void ToolBarWidget::contextMenuEvent(QContextMenuEvent *event)
 	menu.exec(event->globalPos());
 
 	cycleAction->deleteLater();
+
+	thumbnailsAction->deleteLater();
 }
 
 void ToolBarWidget::startToolBarDragging()
 {
-	ToolBarAreaWidget *toolBarArea = qobject_cast<ToolBarAreaWidget*>(parentWidget());
+	ToolBarAreaWidget *toolBarArea(qobject_cast<ToolBarAreaWidget*>(parentWidget()));
 
 	if (toolBarArea)
 	{
@@ -216,7 +321,7 @@ void ToolBarWidget::startToolBarDragging()
 
 void ToolBarWidget::endToolBarDragging()
 {
-	ToolBarAreaWidget *toolBarArea = qobject_cast<ToolBarAreaWidget*>(parentWidget());
+	ToolBarAreaWidget *toolBarArea(qobject_cast<ToolBarAreaWidget*>(parentWidget()));
 
 	if (toolBarArea)
 	{
@@ -226,13 +331,40 @@ void ToolBarWidget::endToolBarDragging()
 
 void ToolBarWidget::reload()
 {
-	const ToolBarsManager::ToolBarDefinition definition = ToolBarsManager::getToolBarDefinition(m_identifier);
+	const ToolBarsManager::ToolBarDefinition definition(ToolBarsManager::getToolBarDefinition(m_identifier));
 
 	setDefinition(definition);
 
 	emit areaChanged(definition.location);
 }
 
+void ToolBarWidget::resetGeometry()
+{
+	for (int i = 0; i < actions().count(); ++i)
+	{
+		QWidget *widget(widgetForAction(actions().at(i)));
+
+		if (widget)
+		{
+			widget->setMaximumSize(0, 0);
+		}
+	}
+
+	setMaximumSize(0, 0);
+
+	for (int i = 0; i < actions().count(); ++i)
+	{
+		QWidget *widget(widgetForAction(actions().at(i)));
+
+		if (widget)
+		{
+			widget->setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
+		}
+	}
+
+	setMaximumSize(QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
+}
+
 void ToolBarWidget::toolBarModified(int identifier)
 {
 	if (identifier == m_identifier)
@@ -269,7 +401,7 @@ void ToolBarWidget::bookmarkRemoved(BookmarksItem *bookmark)
 {
 	if (bookmark == m_bookmark)
 	{
-		m_bookmark = NULL;
+		m_bookmark = nullptr;
 
 		loadBookmarks();
 	}
@@ -291,7 +423,7 @@ void ToolBarWidget::loadBookmarks()
 {
 	clear();
 
-	m_dragArea = NULL;
+	m_dragArea = nullptr;
 
 	if (!ToolBarsManager::areToolBarsLocked() && qobject_cast<ToolBarAreaWidget*>(parentWidget()))
 	{
@@ -307,7 +439,7 @@ void ToolBarWidget::loadBookmarks()
 
 	for (int i = 0; i < m_bookmark->rowCount(); ++i)
 	{
-		BookmarksItem *bookmark = dynamic_cast<BookmarksItem*>(m_bookmark->child(i));
+		BookmarksItem *bookmark(dynamic_cast<BookmarksItem*>(m_bookmark->child(i)));
 
 		if (bookmark)
 		{
@@ -323,7 +455,7 @@ void ToolBarWidget::loadBookmarks()
 	}
 }
 
-void ToolBarWidget::notifyWindowChanged(qint64 identifier)
+void ToolBarWidget::notifyWindowChanged(quint64 identifier)
 {
 	m_window = m_mainWindow->getWindowsManager()->getWindowByIdentifier(identifier);
 
@@ -332,37 +464,20 @@ void ToolBarWidget::notifyWindowChanged(qint64 identifier)
 
 void ToolBarWidget::updateVisibility()
 {
-	if (m_identifier == ToolBarsManager::TabBar && ToolBarsManager::getToolBarDefinition(ToolBarsManager::TabBar).visibility == ToolBarsManager::AutoVisibilityToolBar && m_mainWindow->getTabBar())
+	if (m_identifier == ToolBarsManager::TabBar && ToolBarsManager::getToolBarDefinition(ToolBarsManager::TabBar).normalVisibility == ToolBarsManager::AutoVisibilityToolBar && m_mainWindow->getTabBar())
 	{
 		setVisible(m_mainWindow->getTabBar()->count() > 1);
 	}
 }
 
-void ToolBarWidget::setToolBarLocked(bool locked)
-{
-	setMovable(!locked);
-
-	if (locked && m_dragArea)
-	{
-		m_dragArea->deleteLater();
-		m_dragArea = NULL;
-	}
-	else if (!locked && !m_dragArea && qobject_cast<ToolBarAreaWidget*>(parentWidget()))
-	{
-		m_dragArea = new ToolBarDragAreaWidget(this);
-
-		insertWidget(((actions().count() > 0) ? actions().at(0) : NULL), m_dragArea);
-	}
-}
-
 void ToolBarWidget::setDefinition(const ToolBarsManager::ToolBarDefinition &definition)
 {
-	TabBarWidget *tabBar = ((m_identifier == ToolBarsManager::TabBar && m_mainWindow) ? m_mainWindow->getTabBar() : NULL);
+	QWidget *tabBar((m_identifier == ToolBarsManager::TabBar && m_mainWindow) ? m_mainWindow->getTabBar() : nullptr);
 
-	setVisible(definition.visibility != ToolBarsManager::AlwaysHiddenToolBar);
+	setVisible(definition.normalVisibility != ToolBarsManager::AlwaysHiddenToolBar);
 	setOrientation((definition.location == Qt::LeftToolBarArea || definition.location == Qt::RightToolBarArea) ? Qt::Vertical : Qt::Horizontal);
 
-	m_dragArea = NULL;
+	m_dragArea = nullptr;
 
 	if (m_identifier == ToolBarsManager::TabBar)
 	{
@@ -373,18 +488,21 @@ void ToolBarWidget::setDefinition(const ToolBarsManager::ToolBarDefinition &defi
 				removeAction(actions().at(i));
 			}
 		}
+
+		resetGeometry();
 	}
 	else
 	{
 		clear();
 	}
 
-	setToolButtonStyle(definition.buttonStyle);
+	const int iconSize(getIconSize());
 
-	if (definition.iconSize > 0)
-	{
-		setIconSize(QSize(definition.iconSize, definition.iconSize));
-	}
+	setToolButtonStyle(definition.buttonStyle);
+	setIconSize(QSize(iconSize, iconSize));
+
+	emit buttonStyleChanged(definition.buttonStyle);
+	emit iconSizeChanged(iconSize);
 
 	if (!definition.bookmarksPath.isEmpty())
 	{
@@ -408,6 +526,8 @@ void ToolBarWidget::setDefinition(const ToolBarsManager::ToolBarDefinition &defi
 		addWidget(m_dragArea);
 	}
 
+	const bool isHorizontal(definition.location != Qt::LeftToolBarArea && definition.location != Qt::RightToolBarArea);
+
 	for (int i = 0; i < definition.entries.count(); ++i)
 	{
 		if (definition.entries.at(i).action == QLatin1String("separator"))
@@ -422,142 +542,89 @@ void ToolBarWidget::setDefinition(const ToolBarsManager::ToolBarDefinition &defi
 			}
 			else
 			{
-				QWidget *widget = createWidget(definition.entries.at(i));
+				const bool isTabBar(definition.entries.at(i).action == QLatin1String("TabBarWidget"));
+
+				if (isTabBar && (m_identifier != ToolBarsManager::TabBar || !m_mainWindow || m_mainWindow->getTabBar()))
+				{
+					continue;
+				}
+
+				QWidget *widget(WidgetFactory::createToolBarItem(definition.entries.at(i), this, m_window));
 
 				if (widget)
 				{
 					addWidget(widget);
+
+					if (isTabBar)
+					{
+						tabBar = widget;
+
+						connect(widget, SIGNAL(tabsAmountChanged(int)), this, SLOT(updateVisibility()));
+
+						updateVisibility();
+					}
+					else
+					{
+						layout()->setAlignment(widget, (isHorizontal ? Qt::AlignVCenter : Qt::AlignHCenter));
+					}
 				}
 			}
 		}
 	}
+
+	if (tabBar)
+	{
+		switch (definition.location)
+		{
+			case Qt::BottomToolBarArea:
+				layout()->setAlignment(tabBar, Qt::AlignTop);
+
+				break;
+			case Qt::LeftToolBarArea:
+				layout()->setAlignment(tabBar, Qt::AlignRight);
+
+				break;
+			case Qt::RightToolBarArea:
+				layout()->setAlignment(tabBar, Qt::AlignLeft);
+
+				break;
+			default:
+				layout()->setAlignment(tabBar, Qt::AlignBottom);
+
+				break;
+		}
+
+		QTimer::singleShot(200, tabBar, SLOT(updateSize()));
+	}
 }
 
-QWidget* ToolBarWidget::createWidget(const ActionsManager::ActionEntryDefinition &definition)
+void ToolBarWidget::setToolBarLocked(bool locked)
 {
-	if (definition.action == QLatin1String("spacer"))
+	setMovable(!locked);
+
+	if (locked && m_dragArea)
 	{
-		QWidget *spacer = new QWidget(this);
-		spacer->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
-
-		return spacer;
+		m_dragArea->deleteLater();
+		m_dragArea = nullptr;
 	}
-
-	if (!definition.entries.isEmpty())
+	else if (!locked && !m_dragArea && qobject_cast<ToolBarAreaWidget*>(parentWidget()))
 	{
-		return new ToolButtonWidget(definition, this);
+		m_dragArea = new ToolBarDragAreaWidget(this);
+
+		insertWidget(((actions().count() > 0) ? actions().at(0) : nullptr), m_dragArea);
 	}
-
-	if (definition.action == QLatin1String("AddressWidget"))
-	{
-		return new AddressWidget(m_window, this);
-	}
-
-	if (definition.action == QLatin1String("MenuButtonWidget"))
-	{
-		return new MenuButtonWidget(definition, this);
-	}
-
-	if (definition.action == QLatin1String("PanelChooserWidget"))
-	{
-		return new PanelChooserWidget(definition, this);
-	}
-
-	if (definition.action == QLatin1String("SearchWidget"))
-	{
-		SearchWidget *searchWidget = new SearchWidget(m_window, this);
-		searchWidget->setOptions(definition.options);
-
-		return searchWidget;
-	}
-
-	if (definition.action == QLatin1String("StatusMessageWidget"))
-	{
-		return new StatusMessageWidget(this);
-	}
-
-	if (definition.action == QLatin1String("TabBarWidget"))
-	{
-		if (m_identifier != ToolBarsManager::TabBar)
-		{
-			return NULL;
-		}
-
-		if (!m_mainWindow || m_mainWindow->getTabBar())
-		{
-			return NULL;
-		}
-
-		TabBarWidget *tabBar = new TabBarWidget(this);
-
-		connect(tabBar, SIGNAL(tabsAmountChanged(int)), this, SLOT(updateVisibility()));
-
-		updateVisibility();
-
-		return tabBar;
-	}
-
-	if (definition.action == QLatin1String("ZoomWidget"))
-	{
-		return new ZoomWidget(this);
-	}
-
-	if (definition.action.startsWith(QLatin1String("bookmarks:")))
-	{
-		BookmarksItem *bookmark = (definition.action.startsWith(QLatin1String("bookmarks:/")) ? BookmarksManager::getModel()->getItem(definition.action.mid(11)) : BookmarksManager::getBookmark(definition.action.mid(10).toULongLong()));
-
-		if (bookmark)
-		{
-			return new BookmarkWidget(bookmark, definition, this);
-		}
-	}
-
-	if (definition.action.endsWith(QLatin1String("Action")))
-	{
-		const int identifier = ActionsManager::getActionIdentifier(definition.action.left(definition.action.length() - 6));
-
-		if (identifier >= 0)
-		{
-			ActionWidget *actionWidget = NULL;
-
-			if (identifier == ActionsManager::GoBackAction)
-			{
-				actionWidget = new GoBackActionWidget(m_window, definition, this);
-			}
-			else if (identifier == ActionsManager::GoForwardAction)
-			{
-				actionWidget = new GoForwardActionWidget(m_window, definition, this);
-			}
-			else
-			{
-				actionWidget = new ActionWidget(identifier, m_window, definition, this);;
-			}
-
-			actionWidget->setOptions(definition.options);
-
-			return actionWidget;
-		}
-	}
-
-	if (definition.action.endsWith(QLatin1String("Menu")))
-	{
-		return new ToolButtonWidget(definition, this);
-	}
-
-	return NULL;
 }
 
 QMenu* ToolBarWidget::createCustomizationMenu(int identifier, QList<QAction*> actions, QWidget *parent)
 {
-	const ToolBarsManager::ToolBarDefinition definition = ToolBarsManager::getToolBarDefinition(identifier);
-
-	QMenu *menu = new QMenu(parent);
+	const ToolBarsManager::ToolBarDefinition definition(ToolBarsManager::getToolBarDefinition(identifier));
+	QMenu *menu(new QMenu(parent));
 	menu->setTitle(tr("Customize"));
 
-	QMenu *toolBarMenu = menu->addMenu(definition.title.isEmpty() ? tr("(Untitled)") : definition.title);
+	QMenu *toolBarMenu(menu->addMenu(definition.getTitle().isEmpty() ? tr("(Untitled)") : definition.getTitle()));
 	toolBarMenu->addAction(tr("Configure…"), ToolBarsManager::getInstance(), SLOT(configureToolBar()))->setData(identifier);
 
-	QAction *resetAction = toolBarMenu->addAction(tr("Reset to Defaults…"), ToolBarsManager::getInstance(), SLOT(resetToolBar()));
+	QAction *resetAction(toolBarMenu->addAction(tr("Reset to Defaults…"), ToolBarsManager::getInstance(), SLOT(resetToolBar())));
 	resetAction->setData(identifier);
 	resetAction->setEnabled(definition.canReset);
 
@@ -574,7 +641,7 @@ QMenu* ToolBarWidget::createCustomizationMenu(int identifier, QList<QAction*> ac
 
 	toolBarMenu->addSeparator();
 
-	QAction *removeAction = toolBarMenu->addAction(Utils::getIcon(QLatin1String("list-remove")), tr("Remove…"), ToolBarsManager::getInstance(), SLOT(removeToolBar()));
+	QAction *removeAction(toolBarMenu->addAction(ThemesManager::getIcon(QLatin1String("list-remove")), tr("Remove…"), ToolBarsManager::getInstance(), SLOT(removeToolBar())));
 	removeAction->setData(identifier);
 	removeAction->setEnabled(definition.identifier >= ToolBarsManager::OtherToolBar);
 
@@ -585,7 +652,7 @@ QMenu* ToolBarWidget::createCustomizationMenu(int identifier, QList<QAction*> ac
 
 Qt::ToolBarArea ToolBarWidget::getArea() const
 {
-	ToolBarAreaWidget *toolBarArea = qobject_cast<ToolBarAreaWidget*>(parentWidget());
+	ToolBarAreaWidget *toolBarArea(qobject_cast<ToolBarAreaWidget*>(parentWidget()));
 
 	if (toolBarArea)
 	{
@@ -595,11 +662,28 @@ Qt::ToolBarArea ToolBarWidget::getArea() const
 	return Qt::NoToolBarArea;
 }
 
+Qt::ToolButtonStyle ToolBarWidget::getButtonStyle() const
+{
+	return ToolBarsManager::getToolBarDefinition(m_identifier).buttonStyle;
+}
+
 int ToolBarWidget::getIdentifier() const
 {
 	return m_identifier;
 }
 
+int ToolBarWidget::getIconSize() const
+{
+	const int iconSize(ToolBarsManager::getToolBarDefinition(m_identifier).iconSize);
+
+	if (iconSize > 0)
+	{
+		return iconSize;
+	}
+
+	return style()->pixelMetric(QStyle::PM_ToolBarIconSize);
+}
+
 int ToolBarWidget::getMaximumButtonSize() const
 {
 	return ToolBarsManager::getToolBarDefinition(m_identifier).maximumButtonSize;
@@ -609,17 +693,24 @@ bool ToolBarWidget::event(QEvent *event)
 {
 	if (event->type() == QEvent::LayoutRequest)
 	{
-		const bool result = QToolBar::event(event);
+		const bool result(QToolBar::event(event));
 
 		layout()->setContentsMargins(0, 0, 0, 0);
 
 		return result;
 	}
 
-	if (m_identifier == ToolBarsManager::TabBar && (event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonDblClick || event->type() == QEvent::Wheel) && !GesturesManager::isTracking())
+	if (!GesturesManager::isTracking() && (event->type() == QEvent::MouseButtonPress || event->type() == QEvent::MouseButtonDblClick || event->type() == QEvent::Wheel))
 	{
 		QList<GesturesManager::GesturesContext> contexts;
-		contexts << GesturesManager::NoTabHandleGesturesContext;
+
+		if (m_identifier == ToolBarsManager::TabBar)
+		{
+			contexts.append(GesturesManager::NoTabHandleGesturesContext);
+		}
+
+		contexts.append(GesturesManager::ToolBarGesturesContext);
+		contexts.append(GesturesManager::GenericGesturesContext);
 
 		GesturesManager::startGesture(this, event, contexts);
 	}
